
# Zeitreihen_Four_V1.R
# Written by: Wolfgang Marwan
# Version vom: 24-07-2019

# Zeitreihen-Analyse für Datenformat Anna Pretschner
# Von jedem Plasmodium wurden zu jedem Zeitpunkt 2 Proben genommen,
# und von jeder der beiden RNAs eine Doppelbestimmung durchgeführt
# Das ergibt also 2 x 2 = 4 Messwerte für jeden Zeitpunkt 






# Voreinstellungen

rm(list=ls()) # Clear Workspace: löscht alle Einträge in Variablen
cat("\014")   # Clear Console (entspricht ClrScr)

setwd("~/Desktop/myR")


library(openxlsx)
library (tibble)
library(purrr)
library(stringr)

library(dplyr)



# Name des zu analysierenden Datenfiles:
Filetag <- "anna_zeitreihe_neu01"   # ggf. Stamm als: _WT31

Gene <- 35




# Bleibt unverändert:

Custom_Numbers_File_Name <- "Custom Numbers.xlsx"

# Definition von Grenzen, nur zur Visualisierung in der Tabelle 
# (True / False), bewirkt also keine Selektion von Daten:
obere_Gr <- 2      # Liefert True, wenn Werte inner- 
untere_Gr <- 0.5   # halb dieser Grenzen liegen





# Original-Daten einlesen und NA's konvertieren:
# **********************************************

# csv-File einlesen:
csv <- paste0 (Filetag, ".csv")
myFile<-read.csv(csv,sep="\t",header=T)

# Zahl der Zeilen des Dataframe:
Zeilen <- nrow(myFile)
Zeilen

# Neue Spalte "amount or limit" einfügen und
# zeilenweise NAs gegen Limits austauschen:

# Neue Spalte erzeugen:
amount_or_limit <- myFile [, "amount"]

# und nach "amount" einfügen:
NewFile <- add_column (myFile, "amount_or_limit" = amount_or_limit, .after = "amount")

# # dann zeilenweise NAs suchen und in amount_or_limit 
# # den entsprechenden Limit-Wert eintragen: 
# for (i in 1:Zeilen)
# { if (is.na(NewFile [i, "amount_or_limit"]))
# { NewFile [i, "amount_or_limit"] <- NewFile [i, "limit"] }}


# in dieser Version alternativ auch in amount den limit-Wert einsetzen,
# damit die Zeilen später via na.omit nicht rausfliegen:

# dann zeilenweise NAs suchen und in amount UND amount_or_limit 
# den entsprechenden Limit-Wert eintragen: 
for (i in 1:Zeilen)
{ if (is.na(NewFile [i, "amount_or_limit"]))
{ NewFile [i, "amount_or_limit"] <- NewFile [i, "limit"]
  NewFile [i, "amount"] <- NewFile [i, "limit"]}}



myFile <- NewFile   # Original-Daten unsortiert


## Neues Script für die Daten von Anna Pretschner:
## ***********************************************

## Qualitätskontrolle der Datensätze
## **************************************************************************************
# Zunächst wird analysiert, wieviel vollständige und wieviel unvollständige
# Datensätze vorhanden sind
# Dann werden zwei Dataframes erzeugt (hier noch nicht als Files ausgegeben), 
# einer enthält alle Plasmodien, bei denen 2 Proben genommen wurden
# der andere alle Plasmodien, bei denen bei einem der Zeitpunkte nur 1 Probe vorhanden ist


# Häufigkeitsverteilung: Zahl der Messungen für jedes Plasmodium:

table (myFile [, "plasmodium"])

x <- table (myFile [, "plasmodium"])

class (x)

x <- as.data.frame(x)



mainDir <- "~/Desktop/myR"
subDir <- Filetag
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))

# Zur Ausgabe mit Kopfzeile versehen:
Messungen_pro_Plasmodium <- x
colnames(Messungen_pro_Plasmodium) <- c("Plasmodium", "Messwerte")

# und als Excel speichern:
filename <- paste(sep="", "(1) Messungen_pro_Plasmodium_", Filetag, ".xlsx")
write.xlsx(Messungen_pro_Plasmodium, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)

setwd(file.path(mainDir))

# Vollständiger Datensatz für ein Plasmodium:
# 11 Zeitpunkte * 4 Werte * 35 Gene = 1540 Werte

# Die mit "table" erzeugte Verteilung zeigt, dass nur Plasmodien
# vorhanden sind, die entweder einen vollständigen Datensatz 
# aufweisen, also 4 Messungen aus 2 Proben für jeden Zeitpunkt
# oder Plasmodien, bei denen bei einem der Zeitpunkte nur 
# 1 Probe vorhanden ist, so dass 2 * 35 = 70 Werte fehlen und
# somit nur 1540 - 70 = 1470 Werte vorhanden sind.


# Die Zahl und die Namen der jeweiligen Plasmodien werden ermittelt:

# Plasmodien zählen:
Plasmodien_mit_vollstaendigem_Datensatz <- 
   length(subset (x [,"Var1"], x [,"Freq"]==1540))

Plasmodien_mit_un_vollstaendigem_Datensatz <-
   length(subset (x [,"Var1"], x [,"Freq"]==1470))

Plasmodien_mit_vollstaendigem_Datensatz
Plasmodien_mit_un_vollstaendigem_Datensatz


# Plasmodien-Namen bestimmen: 

Complete <- subset (x [,"Var1"], x [,"Freq"]==1540)

Incomplete <- subset (x [,"Var1"], x [,"Freq"]==1470)

# class (Complete)
# Ergebnisse sind vom Typ "factor" und müssen in "numeric" umgewandelt werden:

# factor to numeric conversion:
as.numeric.factor <- function(x) {as.numeric(levels(x))[x]}

Complete_Plasmodia_Names   <- as.numeric.factor (Complete)
Incomplete_Plasmodia_Names <- as.numeric.factor (Incomplete)

# Und dann je einen Datensatz erzeugen, in dem nur vollständige bzw.
# nur unvollständige Plasmodien vorhanden sind:

# subset: wenn Kritierium "Complete_Plasmodia" ein Vektor von Kriterien ist,
# (und nicht nur ein Kriterium) dann wird das logische Statement == durch %in% ersetzt:

Complete_data_set   <- subset(myFile, plasmodium %in% Complete_Plasmodia_Names)
Incomplete_data_set <- subset(myFile, plasmodium %in% Incomplete_Plasmodia_Names)

# # Just to verify:
# nrow(Complete_data_set) / 1540
# nrow(Incomplete_data_set) / 1470


# Sortieren zur weiteren Verarbeitung:

Complete_sorted_file <- arrange (Complete_data_set,
                           frag,
                           plasmodium,
                           time)


Incomplete_sorted_file <- arrange (Incomplete_data_set,
                                 frag,
                                 plasmodium,
                                 time)

# Ergebnisse werden in dieser Version nicht weiter verwendet





# Hier den kompletten Datensatz abarbeiten und für alle
# fehlenden Werte Zeilen mit NA's einfügen: 
##***************************************************************

# Die Daten werden zunächst so sortiert, dass die vier Messungen 
# einer Bedingung direkt untereinander stehen. 

my_sorted_file <- arrange (myFile,
                           frag,
                           plasmodium,
                           time)

# Dann wird der data.frame zeilenweise durchsucht und diejenigen,
# Datensätze bestimmt, bei denen keine 4 Werte vorhanden sind.
# Diese Zeilen werden in "missing_values" eingetragen
# und die Expressionswerte später als NA's ergänzt:

missing_values <- data.frame()
c <- 1

for (z in 1:(nrow (my_sorted_file)-1))   # Tabelle zeilenweise durchsuchen
{
  if (my_sorted_file [z, "time"] != my_sorted_file [z+1, "time"]) # wenn die nächste Zeile nicht mehr 
                                                                 # zum selben Block gehört
    {  
    if (c < 4)  # wenn weniger als 4 gelesene Zeilen den selben Wert hatten.....
     {
      frag <- my_sorted_file [z, "frag"]  # Fragment (Gen) dieser Zeile
      missing_line <- cbind (my_sorted_file [z, 1:4], frag) # an Zeile andocken
      # je zweimal die selbe Zeile einfügen:
      missing_values <- rbind (missing_values, missing_line, missing_line)
     } # end of if (c<4).....
    c <- 0
  } # end of if (my_sorted_file)....
  c <- c+1
} # end of for.....


# # nur zur Kontrolle:
#  nrow(missing_values)
# # 6 Plasmodien * 2 Werte * 35 Gene = 420 Zeilen = ok, d.h. alle da

# Vektor mit NA's erzeugen, dient als Spalte
NA_col <- rep(NA, nrow(missing_values))

# Spaltenweise in Tabelle einfügen:
NA_Table <- add_column(missing_values, rnaid=NA_col, .before = "frag")
NA_Table <- add_column(NA_Table, runid=NA_col, .before = "frag")
NA_Table <- add_column(NA_Table, amount=NA_col, .after = "frag")
NA_Table <- add_column(NA_Table, limit=NA_col, .after = "amount")
NA_Table <- add_column(NA_Table, amount_or_limit=NA_col, .after = "limit")

# und an die bestehende Tabelle anhängen
NAs_added <- rbind (NA_Table, myFile)

# dann neu sortieren, damit wieder alle 4 Messungen hintereinander zu liegen kommen: 
Complete_sorted_file <- arrange (NAs_added,
                             frag,
                             plasmodium,
                             time,
                             rnaid)
# Diese Tabelle enthält jetzt alle Daten incl. der NA-Werte für die fehlenden Proben!



# Jetzt wird von den ersten, zweiten, dritten und vierten Zeilen
# je eine Datei angelegt:


Erste_Probe_Erste_Messung  <- data.frame()
Erste_Probe_Zweite_Messung <- data.frame()
Zweite_Probe_Erste_Messung  <- data.frame()
Zweite_Probe_Zweite_Messung <- data.frame()


# Jeden vierten Wert in einen separaten Datensatz packen:

Erste_Probe_Erste_Messung = Complete_sorted_file[seq(1, nrow(Complete_sorted_file), 4), ]
Erste_Probe_Zweite_Messung = Complete_sorted_file[seq(2, nrow(Complete_sorted_file), 4), ]
Zweite_Probe_Erste_Messung = Complete_sorted_file[seq(3, nrow(Complete_sorted_file), 4), ]
Zweite_Probe_Zweite_Messung = Complete_sorted_file[seq(4, nrow(Complete_sorted_file), 4), ]


# nur zur Kontrolle, dass es keine Doppelungen gibt:
nrow(Erste_Probe_Erste_Messung)
nrow(Erste_Probe_Zweite_Messung)
nrow(Zweite_Probe_Erste_Messung)
nrow(Zweite_Probe_Zweite_Messung)
nrow (Complete_sorted_file) / 4







# ******************************************************
# Hier beginnt der "alte Code" aus "Zeitreihen_V3.R"
# der für 4 Messungen (Anna Pretschner) angepasst wurde 
# ******************************************************


# Zum Erzeugen einer Tabelle in der alle 4 Messungen in einer Zeile angeordnet werden,
# dürfen die Zeilen mit NA's NICHT entfernt werden:

Erste_Messung  <- arrange (Erste_Probe_Erste_Messung,
                           light,
                           plasmodium,
                           time,
                           frag)


Zweite_Messung <- arrange (Erste_Probe_Zweite_Messung,
                           light,
                           plasmodium,
                           time,
                           frag)




Dritte_Messung  <- arrange (Zweite_Probe_Erste_Messung,
                           light,
                           plasmodium,
                           time,
                           frag)


Vierte_Messung <- arrange (Zweite_Probe_Zweite_Messung,
                           light,
                           plasmodium,
                           time,
                           frag)





nrow(Erste_Messung)
nrow(Zweite_Messung)
nrow(Dritte_Messung)
nrow(Vierte_Messung)
  


  

# Die vier Teil-Datensätze Seit an Seit verknüpfen:
Erste_bis_Vierte_Messung <- cbind (Erste_Messung, Zweite_Messung, Dritte_Messung, Vierte_Messung)


# und die Spalten-Namen entsprechend eindeutig machen:
Titel1 <- paste0(colnames(Erste_Messung), "_1st")
Titel2 <- paste0(colnames(Zweite_Messung), "_2nd")
Titel3 <- paste0(colnames(Zweite_Messung), "_3rd")
Titel4 <- paste0(colnames(Zweite_Messung), "_4th")

Titel  <- c(Titel1, Titel2, Titel3, Titel4)
Erste_bis_Vierte_Messung <- setNames(Erste_bis_Vierte_Messung, Titel)

# Mittels Checksums überprüfen, ob alle RNAs und Gene doppelt gemessen wurden
# und ob die Werte richtig sortiert wurden:
# verschiedene Läufe durchgeführt wurden (Werte doppelt eingetragen wurden)

# Für die Erste_Probe:
chks_RNAid <- Erste_bis_Vierte_Messung [,"rnaid_1st"] - Erste_bis_Vierte_Messung [,"rnaid_2nd"]
chks_Gene  <- Erste_bis_Vierte_Messung [,"frag_1st"]  - Erste_bis_Vierte_Messung [,"frag_2nd"]
chks_runid <- Erste_bis_Vierte_Messung [,"runid_1st"] - Erste_bis_Vierte_Messung [,"runid_2nd"]

# Auf dem Bildschirm ausgeben:
subset(chks_RNAid, chks_RNAid !=0)       # ist kein Wert <> 0 wie erwartet? soll "integer(0)" geben
subset(chks_Gene, chks_Gene !=0)         # ist kein Wert <> 0 wie erwartet? soll "integer(0)" geben
subset(chks_runid, chks_runid == 0)      # ist kein Wert = 0 wie erwartet?  soll "integer(0)" geben


# Für die Zweit_Probe:
chks_RNAid <- Erste_bis_Vierte_Messung [,"rnaid_3rd"] - Erste_bis_Vierte_Messung [,"rnaid_4th"]
chks_Gene  <- Erste_bis_Vierte_Messung [,"frag_3rd"]  - Erste_bis_Vierte_Messung [,"frag_4th"]
chks_runid <- Erste_bis_Vierte_Messung [,"runid_3rd"] - Erste_bis_Vierte_Messung [,"runid_4th"]

# Auf dem Bildschirm ausgeben:
subset(chks_RNAid, chks_RNAid !=0)       # ist kein Wert <> 0 wie erwartet? soll "integer(0)" geben
subset(chks_Gene, chks_Gene !=0)         # ist kein Wert <> 0 wie erwartet? soll "integer(0)" geben
subset(chks_runid, chks_runid == 0)      # ist kein Wert = 0 wie erwartet?  soll "integer(0)" geben




# Überprüfen, wie sehr jeder Messwert über der Schwelle liegt:
# xoT = x-fold over threshold
xoT_1st  <- Erste_bis_Vierte_Messung [,"amount_or_limit_1st"]  / Erste_bis_Vierte_Messung [,"limit_1st"]
xoT_2nd  <- Erste_bis_Vierte_Messung [,"amount_or_limit_2nd"] /  Erste_bis_Vierte_Messung [,"limit_2nd"]

xoT_3rd  <- Erste_bis_Vierte_Messung [,"amount_or_limit_3rd"] /  Erste_bis_Vierte_Messung [,"limit_3rd"]
xoT_4th  <- Erste_bis_Vierte_Messung [,"amount_or_limit_4th"] /  Erste_bis_Vierte_Messung [,"limit_4th"]

# und das Verhältnis ist ein Mass für die Diskrepanz der Messgenauigkeit:

xoT_1st_by_2nd <- xoT_1st / xoT_2nd

xoT_3rd_by_4th <- xoT_3rd / xoT_4th



# Spalten mit Custum Numbers erzeugen und anhängen:
# Erst File mit Custom Numbers einlesen
xlsxFile <- Custom_Numbers_File_Name
CustomNumbers <- read.xlsx(xlsxFile, sheet = 1, startRow = 1, colNames = TRUE,
                           rowNames = FALSE, detectDates = FALSE, skipEmptyRows = TRUE,
                           skipEmptyCols = TRUE, rows = NULL, cols = NULL, check.names = FALSE,
                           namedRegion = NULL, na.strings = "NA", fillMergedCells = FALSE)


# und dann gesamte Spalte mit den Custom Numbers erzeugen:
Proben <- nrow(Erste_Messung) / Gene
Custom_Number_Column <- CustomNumbers
for (i in 2:Proben) # Custom Number Blöcke hintereinander hängen
{ Custom_Number_Column <- rbind (Custom_Number_Column, CustomNumbers) }
# newcolNames <- c("Fragment_Nr", "Custom_Nr", "Gene")
# setNames(Custom_Number_Column, newcolNames)



# Nun werden die Messwerte auf den Median aller 35 Werte einer 
# jeden RNA-Probe normiert, für jede der beiden Messungen separat:
RNA_Proben <- nrow(Erste_Messung) / Gene
Erste_Med_norm  <- numeric()
Zweite_Med_norm <- numeric()
Dritte_Med_norm <- numeric()
Vierte_Med_norm <- numeric()

for (ProbeNr in 1:RNA_Proben)
{ # begin of for....
  von <- (Gene * (ProbeNr-1)) + 1
  bis <-  Gene * ProbeNr
  
  Median_Erste <- median(Erste_bis_Vierte_Messung [von:bis,"amount_or_limit_1st"])
  Median_Zweite <-median(Erste_bis_Vierte_Messung [von:bis,"amount_or_limit_2nd"])
  Median_Dritte <-median(Erste_bis_Vierte_Messung [von:bis,"amount_or_limit_3rd"])
  Median_Vierte <-median(Erste_bis_Vierte_Messung [von:bis,"amount_or_limit_4th"])
  
  Erste <- Erste_bis_Vierte_Messung [von:bis,"amount_or_limit_1st"]
  Erste_norm <- Erste / Median_Erste
  Erste_Med_norm <- append(Erste_Med_norm, Erste_norm)
  
  Zweite <- Erste_bis_Vierte_Messung [von:bis,"amount_or_limit_2nd"]
  Zweite_norm <- Zweite / Median_Zweite
  Zweite_Med_norm <- append(Zweite_Med_norm, Zweite_norm)
  
  Dritte <- Erste_bis_Vierte_Messung [von:bis,"amount_or_limit_3rd"]
  Dritte_norm <- Dritte / Median_Dritte
  Dritte_Med_norm <- append(Dritte_Med_norm, Dritte_norm)
  
  Vierte <- Erste_bis_Vierte_Messung [von:bis,"amount_or_limit_4th"]
  Vierte_norm <- Vierte / Median_Vierte
  Vierte_Med_norm <- append(Vierte_Med_norm, Vierte_norm)
  
} # end of for ProbeNr



# Geht so nicht mehr, wegen NAs, deshalb muss die Berechnung 
# der Mittelwerte neu implementiert werden: 
# Mean_norm <- (Erste_Med_norm + Zweite_Med_norm + Dritte_Med_norm + Vierte_Med_norm) / 4

# Jetzt neu: Mitteln, wobei NA's nicht berücksichtigt werden (na.rm=TRUE):
Median_Normierte_Werte <- cbind (Erste_Med_norm, Zweite_Med_norm, Dritte_Med_norm, Vierte_Med_norm)

Mean_norm <- rowMeans(Median_Normierte_Werte,na.rm=TRUE)



Norm_1st_by_2nd <- Erste_Med_norm  / Zweite_Med_norm
Norm_3rd_by_4th <- Dritte_Med_norm / Vierte_Med_norm

# Stürzt die Werte, wenn sie < 1 sind und schreibt sie in Norm_1st_by_2nd_Inv: 
Norm_1st_by_2nd_Inv <- Norm_1st_by_2nd
n <- length (Norm_1st_by_2nd_Inv)

Norm_3rd_by_4th_Inv <- Norm_3rd_by_4th
n <- length (Norm_3rd_by_4th_Inv)

# for (i in 1:n) {if (Norm_1st_by_2nd_Inv[i] < 1) {Norm_1st_by_2nd_Inv[i] <- 1/Norm_1st_by_2nd_Inv[i]}}

# Diese Tabelle enthält jetzt NA's der fehlenden Plasmodien-Proben, deshalb
# Werte nur stürzen, wenn sie nicht NA sind:
for (i in 1:n) 
    { if (!is.na (Norm_1st_by_2nd_Inv[i]) )  
          { if (Norm_1st_by_2nd_Inv[i] < 1) {Norm_1st_by_2nd_Inv[i] <- 1/Norm_1st_by_2nd_Inv[i]}
    }
       if (!is.na (Norm_3rd_by_4th_Inv[i]) )  
          { if (Norm_3rd_by_4th_Inv[i] < 1) {Norm_3rd_by_4th_Inv[i] <- 1/Norm_3rd_by_4th_Inv[i]}
  }}
 




# Prozentuale Abweichungen des n-ten Messwerts vom Mittel 
# der Werte aller durchgeführten Replikate
# Ergibt dann die technische Verteilung der Werte
# und gleichzeitig die Abweichungen innerhalb eines Plasmodiums
# da hier 2 Proben des selben Plasmodiums zum selben Zeitpunkt
# verglichen werden. 
Erste_Dev  <- Erste_Med_norm / Mean_norm         # entsprechend Erster  Probe, Erste  Messung  
Zweite_Dev <- Zweite_Med_norm / Mean_norm        # entsprechend Erster  Probe, Zweite Messung   
Dritte_Dev <- Dritte_Med_norm / Mean_norm        # entsprechend Zweiter Probe, Erste  Messung  
Vierte_Dev <- Vierte_Med_norm / Mean_norm        # entsprechend Zweiter Probe, Zweite Messung  


# und an die Tabelle anhängen
Erste_bis_Vierte_Messung <- cbind(Erste_bis_Vierte_Messung, 
                                  Custom_Number_Column,
                                  Erste_Med_norm,
                                  Zweite_Med_norm,
                                  Dritte_Med_norm,
                                  Vierte_Med_norm,
                                  Mean_norm,
                                  Norm_1st_by_2nd,
                                  Norm_1st_by_2nd_Inv,
                                  Norm_3rd_by_4th,
                                  Norm_3rd_by_4th_Inv,
                                  Erste_Dev,
                                  Zweite_Dev,
                                  Dritte_Dev,
                                  Vierte_Dev,
                                  xoT_1st, 
                                  xoT_2nd,
                                  xoT_1st_by_2nd,
                                  xoT_3rd, 
                                  xoT_4th,
                                  xoT_3rd_by_4th,
                                  chks_RNAid, 
                                  chks_Gene,
                                  chks_runid)


# Neu sortieren, jetzt auch nach Custum Nubers:
Erste_bis_Vierte_Messung <- arrange(Erste_bis_Vierte_Messung,  plasmodium_1st, Custom_Nr)


Erste_bis_Vierte_Messung <- arrange(Erste_bis_Vierte_Messung,
                                    light_1st,
                                    spo_1st,
                                    plasmodium_1st, 
                                    time_1st,
                                    Custom_Nr)






setwd(file.path(mainDir, subDir))

# und abspeichern als Excel: 
# newFileName <- paste0 ("(1) 1st_and_2nd_", Filetag, ".xlsx")
newFileName <- paste0 ("(2) 1st_to_4th_",  Filetag, ".xlsx")
write.xlsx(Erste_bis_Vierte_Messung, newFileName, asTable = FALSE, keepAN = TRUE, zoom = 150)
# openXL(newFileName)




Plasmodien_Ausschliessen <- function(ds, Excluded_Plasmodia)
{
# Dann einzelne Plasmodien ausschliessen, falls erwünscht:
# Wenn Excluded_Plasmodia nur eine 0 enthält dann wird 
# kein Plasmodium ausgesondert (oder Funktion einfach nicht aufrufen)  

# # Ausgeschlossene Plasmodien als Excel-File dokumentieren
# Excluded_Plasmodia <- as.data.frame(Exclude_Plasmodium_Nr)
# colnames(Excluded_Plasmodia) <- "Excluded_Plasmodia"
# setwd(file.path(mainDir))
# Excluded_File_Name <- paste0("Excluded_Plasmodia_", Filetag, ".xlsx")
# write.xlsx(Excluded_Plasmodia, Excluded_File_Name, asTable = FALSE, keepAN = TRUE, zoom = 150)
# setwd(file.path(mainDir, subDir))
# 


Cleaned_File <- ds

for (PL in 1:length(Excluded_Plasmodia))
{ Cleaned_File <- subset(Cleaned_File, plasmodium_1st!=Exclude_Plasmodium_Nr[PL]) 
} # end of for PL....

# Nur zum Testen, wieviele Plasmodien noch im Datensatz vorhanden sind:
# unique (Cleaned_File[,"plasmodium_1st"])


# und das bereinigte File unter neuem Namen abspeichern: 

sine <- paste(Exclude_Plasmodium_Nr, collapse = "_")
Name_tag_bereinigt <- paste0(Filetag, "_sine_", sine)

newFileName_bereinigte <- paste0("(1) 1st_and_2nd_", Name_tag_bereinigt, ".xlsx")

subDir <- Name_tag_bereinigt
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))

write.xlsx(Cleaned_File, newFileName_bereinigte, asTable = FALSE, keepAN = TRUE, zoom = 150)
# openXL(newFileName)
# In erstes Feld klicken und manuell abspeichern, damit drag & drop Import in FileMaker klappt

return (Cleaned_File)

} # End of Function "Plasmodien_Ausschliessen"




QC_Technical_Replicate <- function(df, Verteilung_von, Quantil_Display)
# Plottet die Verteilung der Abweichung der ersten und / oder zweiten Messung
# vom Mittelwert beider Messungen des Technischen Replikats 
# und bestimmt die entsprechenden Quantile
{  

# in der Vorversion "Zeitreihen_V3.R" entsprechend:
  
# Technische_Verteilung <- (df [, "Erste_Dev"])
# Technische_Verteilung <- c(Technische_Verteilung, df [, "Zweite_Dev"])
# Technische_Verteilung <- c(Technische_Verteilung, df [, "Dritte_Dev"])
# Technische_Verteilung <- c(Technische_Verteilung, df [, "Vierte_Dev"])
# Technische_Verteilung <- log2(Technische_Verteilung)
# 
# 
# # Wurde hier neu als Schleife implementiert und erlaubt dadurch die 
#   Erstellung von Histogrammen durch Kombination der Werte 
#   beliebig vieler Spalten: 
#


# Spalten aus der Tabelle zu einem Vektor vereinigen,
# aus dem dann die Verteilung berechnet wird:

Technische_Verteilung <- (df [, Verteilung_von [1]])

if (length (Verteilung_von) > 1)
  {  for (i in 2: length(Verteilung_von))  
       {  Technische_Verteilung <- c(Technische_Verteilung, df [, Verteilung_von [i]])
   }}    

Technische_Verteilung <- log2(Technische_Verteilung)

# NA's müssen vor der Berechnung der Häufigkeitsverteilung entfernt werden:
Technische_Verteilung <- na.omit(Technische_Verteilung)





# Quantil bestimmen:
Quantil <- quantile(Technische_Verteilung, probs = c(0.01, 0.05, 0.25, 0.5, 0.75, 0.95, 0.99)) 
Quantil_hoch_2 <- 2**Quantil

# in Tabelle umwandeln:
Quantil_Tab <- cbind(Quantil, Quantil_hoch_2)



Name_tag <- paste (Verteilung_von, collapse="_&_")

# und abspeichern als Excel: 
newFileName <- paste0 ("(3) Quantils_", Name_tag, "_",  Filetag, ".xlsx")
write.xlsx(Quantil_Tab, newFileName, asTable = FALSE, rowNames = TRUE, colNames = TRUE, 
           keepAN = TRUE, zoom = 150)
# openXL(paste(sep="/", subDir, newFileName))


# in Strings umwandeln zur graphischen Ausgabe im Histogramm-Plot:
Quantil_Str   <- paste(names(Quantil), collapse = ";  ")
Wert_Str_log2 <- paste(round (Quantil, digits=2), collapse = ";  ")
Wert_Str      <- paste(round (Quantil_hoch_2, digits=2), collapse = ";  ")

# Beschriftung einfügen:
Wert_Str_log2 <- paste0(" Log2:  ", Wert_Str_log2)
Wert_Str      <- paste0("Lin:    ", Wert_Str)

# Komma_Stellen im String anpassen:
Wert_Str_log2 <- str_replace(Wert_Str_log2, pattern="0;", "0.00;")
Wert_Str      <- str_replace(Wert_Str, pattern="1", "1.00")

# Zur Formatierung Leerstellen einfügen:
Quantil_Str <- paste0("           ", Quantil_Str)
Wert_Str    <- str_replace(Wert_Str, pattern="; ", ";    ")
Wert_Str <- paste0(" ", Wert_Str)


File_Name <- paste0 ("(4) Histogram_", Name_tag, "_",  Filetag, ".pdf")

# Grösse der Beschriftung im Histogramm über width, height einstellen
{pdf (File_Name,width=10,height=7)   # width=20,height=15
  histogram<-hist(Technische_Verteilung, 
                  # main=(paste0 ("Datafile: ", Datenfile, " // Normierung = ", Normierung, " // Total number of values: ", length(Alle_Mean_Norm_Werte))),
                  main=File_Name, 
                  sub= (paste0 ( "Total number of values: ", length(Technische_Verteilung))),
                  xlab= paste0("Log2 (x-fold deviation from mean of the replicates) "), 
                  border="black", 
                  col="grey",
                  # xlim=c(floor(min(Technische_Verteilung)),ceiling(max(Technische_Verteilung))),
                  xlim=c(-2, 2),
                  #ylim=c(0,800)
                  las=1, 
                  breaks=50)
  # Text-Box einfügen (relativ zu den Koordinaten des Plots:
  if (Quantil_Display)
    { text(1.5,5000, labels = Quantil_Str, vfont=NULL, cex=0.7)
      text(1.5, 4600, labels = Wert_Str_log2, cex=0.7)
      text(1.5, 4200, labels = Wert_Str, cex=0.7) }
  
  dev.off()}

return

} # end of function QC_Technical_Replicate




Umformatieren <- function (ds, Name_tag) 
# Umformatieren der Daten zur PN-Konstruktion
# Dabei werden Plasmodien-Namen automatisch erzeugt
{

# Spo = "1" gegen Spo = "+" austauschen:  
ds[, "spo_1st"] <- replace (ds[, "spo_1st"], ds[, "spo_1st"]==0, "-")
ds[, "spo_1st"] <- replace (ds[, "spo_1st"], ds[, "spo_1st"]==1, "+")

Plasmodien <- unique(ds[, "plasmodium_1st"])
Zeitpunkte <- unique(ds[, "time_1st"])
Genzeile   <- unique(ds[,  "Gene"])


# Leere Tabelle erzeugen:
Spalten_Namen <- c("Exp", "Pl", "Spo", "PlNr", Genzeile )

Tabelle <- data.frame (matrix (ncol = (Gene + 4), nrow = 0))
colnames(Tabelle) <- Spalten_Namen

newLine <- Tabelle


# Den ganzen Datensatz Plasmodienweise abarbeiten: 

for (P in 1:length(Plasmodien))
  {
   for (t in 1:length(Zeitpunkte))
    { 
  
# Datenblock kopieren:
Auswahl <- subset(ds, plasmodium_1st == Plasmodien [P] & time_1st == Zeitpunkte[t] )


# Standardisierten Unique Identifier der Probe des Datenblocks erzeugen:
Spo <- unique(Auswahl[, "spo_1st"])

Probe <- c("P", 
           unique(Auswahl[, "plasmodium_1st"]),
           "_FR",
           unique(Auswahl[, "light_1st"]),
           Spo)
Plasmodien_Name <- paste(Probe, collapse = "")


# Laufende Plasmodien-Nummer erzeugen:

Zeitpunkt <- unique(Auswahl[, "time_1st"])
LPN <- P # LPN = Laufende Plasmodien-Nummer
plnr <- c(Spo, "P", LPN, "_", Zeitpunkte[t], "h")
PlNr <- paste(plnr, collapse = "")



# Expressionswerte der Spalte in Vektor umwandeln:
Expressionswerte <- Auswahl [,"Mean_norm"]
names(Expressionswerte) <- Genzeile

# Neue Zeile der Tabelle als data.frame erzeugen:
newLine [1, "Exp"] <- 0
newLine [1, "Pl"]  <- Plasmodien_Name
newLine [1, "Spo"]  <- Spo
newLine [1, "PlNr"]  <- PlNr
newLine [1,5:39] <- Expressionswerte

Tabelle <- rbind(Tabelle, newLine)

  } # end of for t.....
} # end of for P.....

# und  als Excel abspeichern: 
newFileName <- paste0 (Filetag, "_4PN_", Name_tag, ".xlsx")
write.xlsx(Tabelle, newFileName, asTable = FALSE, keepAN = TRUE, zoom = 150)
# openXL(newFileName)

newFileName <- paste0 (Filetag, "_4PN_", Name_tag, ".csv")
write.table(Tabelle, newFileName, sep = ",", row.names = FALSE)

} # End of Function "Umformatieren"




# Im Folgenden werden nur noch die Funktionen aufgerufen:
# _________________________________________________________________
# Die Funktion "Plasmodien_Ausschliessen" wird hier nicht verwendet
# _________________________________________________________________



# Häufigkeits-Verteilungen in den gewünschten Kombinationen erzeugen:
# ********************************************************************

Quantil_Display_im_Plot <- TRUE

Verteilung_von <- c("Erste_Dev", "Zweite_Dev")
QC_Technical_Replicate (Erste_bis_Vierte_Messung, Verteilung_von, Quantil_Display_im_Plot)

Verteilung_von <- c("Dritte_Dev", "Vierte_Dev")
QC_Technical_Replicate (Erste_bis_Vierte_Messung, Verteilung_von, Quantil_Display_im_Plot)


Verteilung_von <- c("Erste_Dev", "Zweite_Dev", "Dritte_Dev", "Vierte_Dev")
QC_Technical_Replicate (Erste_bis_Vierte_Messung, Verteilung_von, Quantil_Display_im_Plot)

# Es kann nur eine Messreihe in's Verhältnis zum Mittel aller gesetzt 
# und geplottet werden:
# Verteilung_von <- c("Erste_Dev")
# QC_Technical_Replicate (Erste_bis_Vierte_Messung, Verteilung_von, Quantil_Display_im_Plot)




# Umformatieren der Daten zur Konstruktion der Petri-Netze:
# *********************************************************

# Neuen Ordner erzeugen:

PN_Dir <- paste0(Filetag, "/", "(5) For_PN_Construction")
dir.create(file.path(mainDir, PN_Dir), showWarnings = FALSE)
setwd(file.path(mainDir, PN_Dir))


# Daten für PN-Konstruktion formatieren: 

Umformatieren (Erste_bis_Vierte_Messung, "")





# Doku-File und Session speichern:
# **********************************************************

# Dir zurücksetzen:
setwd(file.path(mainDir, subDir))


# Doku-File erzeugen: 

Sys.setenv(TZ="Europe/Berlin")  # Setzt die Zeitzone
Zeitstempel <- as.character(Sys.time())

Source_Code_File_Name <- rstudioapi::getActiveDocumentContext()$path

Plasmodien_Gesamtzahl <- (Plasmodien_mit_vollstaendigem_Datensatz 
                          + Plasmodien_mit_un_vollstaendigem_Datensatz)

fileConn<-file(paste (sep="","(0) RDoku_", Filetag, ".txt"))
writeLines(c( "Ergebnisse berechnet und gespeichert am: ",
               Zeitstempel,
               "",
               "mit R_Code Source File Name: ", 
               Source_Code_File_Name, 
               "",
                paste0("Input File Name: ",csv) ,
               "",
                paste0("Plasmodien mit vollständigem Datensatz    : ", 
                       Plasmodien_mit_vollstaendigem_Datensatz),
                paste0("Plasmodien mit un-vollständigem Datensatz : ", 
                       Plasmodien_mit_un_vollstaendigem_Datensatz),
                paste0("Gesamtzahl der Plasmodien                 : ", 
                        Plasmodien_Gesamtzahl)), fileConn)
close(fileConn)



# Session speichern:

Session_Name <- paste0("(10) Complete_Session_", Filetag, ".RData")

save.image (Session_Name)


# ggf. wieder laden mit:
# setwd("~/Desktop/myR")
# load(paste0("~/Desktop/myR/", Session_Name))


setwd("~/Desktop/myR")










