
# Cluster_to_HiR_PN_V45.R

# Written by: Wolfgang Marwan
# Version vom 25-09-2019

# Clustering und Heatmaps mit MDS
# Version für High-Resolution Time Series
# Berechnung der geometrischen Mittel der Expressionswerte jedes Clusters 


# Voreinstellungen

rm(list=ls()) # Clear Workspace: löscht alle Einträge in Variablen
cat("\014")   # Clear Console (entspricht ClrScr)

setwd("~/Desktop/myR")

library(gplots)
library(clustsig)
library(rstudioapi)
library(RColorBrewer)

library(purrr)
library(stringr)
library(dplyr)
library(tibble)

library(openxlsx)




# File namen,  Zeitpunkte und Settings definieren: ------------------------

# Datenfile einlesen:
Datenfile <- "anna_zeitreihe_neu01_4PN_.csv"

# Settings:
FileTag <- "Anna_all_35"

# Genes analysed as defined by the Custom Numbers assigned to each gene
Genes_analysed <- c(1:35) # all 35

# Genes_analysed <- c(1, 2, 3, 4, 5, 6, 8, 9, 11, 15, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35)  Genes called up- or down-regulated 
# Genes_analysed <- c(26, 27, 28, 29, 30, 31, 32, 33, 34, 35)  Genes called up-regulated 
# Genes_analysed <- c(1, 2, 3, 4, 5, 6, 8, 9, 11, 15)  Genes called down-regulated 
# Genes_analysed <- c(7, 12, 13, 14)  Genes called pcnA group




simprof_alpha <- 0.05

LogScale     <- T
Zeitpunkte   <- 11

# "Time_Color" zum Plotten der Trajektorien an 11 Zeitpunkte angepasst (!)


# Char-Vektor mit Zeitpunkten automatisch erzeugen
Zeiten  <- seq(0, 10, by=1)
Stunden <- rep("h", Zeitpunkte)
Zeitpunkte.h <- paste0(format(Zeiten, nsmall=1), Stunden) # nsmall: 1 Stelle hinterm Komma im String
Zeitpunkte.h <- str_replace (Zeitpunkte.h, pattern = " ", "") # führende blanks raus


Gene         <- 35 # Zahl der Gene im gesamten Datensatz

Normierung   <- "Median" # Normierung der eingelesenen Expressionswerte, wie sie vorab erfolgte ist


Nur_HeatMaps      <- FALSE # Programmlauf nur bis zu den Heatmaps (ohne Simprof)
Two_Color_Sidebar <- TRUE

# Schriftgrösse der HeatMaps einstellen:
xsize <- 1        # 1.2
ysize <- 0.8       # 1


# FileTag <- paste(sep="_", FileTag, Normierung, "")
# in dieser Version keine Normierung der eingelesenen Daten (!)
#===============================================================

# Hier die Zahl der Plasmodien jeder Klasse für den
# gesamten Datensatz eintragen (siehe Tabelle):
# Enter the number of plasmodia of each class:
Klassenzimmer      <- c(4, 4, 8, 8)
# corresponding to class # 1, 2, 3, 4, etc.
# entsprechend Klasse   1, 2,  3,  4, 5,  6, 7,  8,  9, 10, 11
# für nur 1 Klasse, Zahl der Plasmodien insgesamt z.B. "Klassenzimmer <- 7"

Take_Subset <- FALSE
Take_Klasse <- c (7, 8, 9, 10, 11)

Sub_Klassenzimmer <- c (8, 13, 16, 13, 13)


# # zur Kontrolle muss Zahl der Plasmodien geben: 
# sum(Klassenzimmer)
# 
Plasmodien_Klassen <- length(Klassenzimmer)
Plasmodien_Klassen
Klassenzimmer_Gesamter_Datensatz <- Klassenzimmer
#=============================================================



# "Time_Color" zum Plotten der Trajektorien an die Zahl der Zeitpunkte hier anpassen (!)
# (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!)
# Ursprünglich Standard:
# Time_Color <- c ("black", "blue", "green", "orange", "red")

# Time_Color <- c ("black", "blue", "green", "purple", "red")
# Time_Color <- c ("black", "blue", "dark green", "purple", "red")



# # Jetzt mit Farben, die alle 21 Zeitpunkte abdecken, Start = Violett
# Time_Color <- c ("violet", "black", "black", "black", "black",
#                  "blue", "blue", "blue", "blue",
#                  "green", "green", "green", "green",
#                  "orange", "orange", "orange", "orange",
#                  "red", "red", "red", "red")


# Jetzt mit Farben, die alle 11 Zeitpunkte abdecken, Start = Violett
Time_Color <- c ("violet", "black", "black",
                 "blue",  "blue",
                 "green",  "green",
                 "orange", "orange",
                 "red", "red")

# (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!) (!)



# Skala für die Heatmap der Daten festlegen:
#=============================================================

if (LogScale) 
     { # begin of if...
         Heat_Map_Skala_von <- -2   # entsprechend log 1   (hier %) = 0
         Heat_Map_Skala_bis <-  2
     } else {
         Heat_Map_Skala_von <- 0   # entsprechend log 1   (hier %) = 0
         Heat_Map_Skala_bis <- 100
     } # end of else....
# alternativ Skala automatisch festlegen
# Skala_von <- min (data_norm)    # konkret nehmen die kleinen Werte hier zu viel von der Skala ein
# Skala_bis <- max (data_norm)


# Vektor mit Plasmodien-Klassen erzeugen

Auswahl <- function (Take_Klasse, Klassenzimmer)
{ Auswahl <- data.frame()
  for (i in 1:length(Take_Klasse))
    { Klasse <- Take_Klasse[i]
      if (Klasse > 1)
        { von_Zeile <-  sum (Klassenzimmer[1:Klasse-1]) * Zeitpunkte + 1
          bis_Zeile <-  sum (Klassenzimmer[1:Klasse]) * Zeitpunkte
        } else {
          von_Zeile <- 1
          bis_Zeile <- Klassenzimmer [1] * Zeitpunkte
         } # end of if Klasse 
  
       Auswahl <- rbind(Auswahl, my_Tab [von_Zeile:bis_Zeile,]) 
     }  # end of for i....
  return (Auswahl)
} # end of function Auswahl



# Daten einlesen, Data Preprocessing and Normalization ------------------------------------

# Die zu verwendenden Rownames stehen in der ersten Spalte der csv-Tabelle
# die zweite und dritte Spalte der csv-Tabelle werden als Daten interpretiert
# in Wirklichkeit enthalten sie ebenfalls Zeilen-Namen!

my_Tab <- read.csv(Datenfile,sep=",",header=T,row.names=4)


# Main-Directory ändern damit Ergebnis-Files in einen 
# entsprechenden Ordner kopiert werden:

mainDir <- getwd()
mainDir <- paste(mainDir, FileTag, sep="/")
dir.create(file.path(mainDir), showWarnings = FALSE)
setwd(file.path(mainDir))
# 
# subDir <- "(96) Petri Net Construction Doku"
# dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
# setwd(file.path(mainDir, subDir))
# 
# setwd(file.path(mainDir))



# Falls nur Teile des Datensatzes verarbeitet werden sollen:
if (Take_Subset) 
   { my_Tab <- Auswahl (Take_Klasse, Klassenzimmer)
     Klassenzimmer <- Sub_Klassenzimmer
     Plasmodien_Klassen <- length(Klassenzimmer)
     Plasmodien_Klassen
   }



# Vektor mit Plasmodien-Klassen erzeugen
Pcl <- as.integer()
for (i in 1:length (Klassenzimmer)) 
{
  Pcl <- c(Pcl, rep(i,Klassenzimmer[i]))
} # end
Plasmodial_Class_Assignment <- Pcl

length(Plasmodial_Class_Assignment)




# Die Expressionswerte in eine Matrix kopieren
# Erstes Gen ist in der vierten Spalte
Expressionswerte <- as.matrix (my_Tab [,4:(Gene+3)])


## Die Daten der grossen Plasmodien sind bereits auf den Median 
# aller in einer RNA-Probe gemessenen Expressionswerte normiert
# deshalb wurde der Code zur Normierung auf den Median,
# der sich an dieser Stelle befand, entfernt

Skalierter_Datensatz <- Expressionswerte



# Datensatz logarithmieren:
Logarithmierter_Datensatz <- log10(Skalierter_Datensatz)


# Zweite Normierung: auf das Geometrische Mittel --------------------------

# Geometrisches Mittel der Expressionswerte jedes Gens:
# Matrix splatenweise (,2,) mitteln; zeilenweise wäre (,1,)

Geometric_Means <- colMeans(Logarithmierter_Datensatz, na.rm=TRUE)

# Gleiches Ergebnis würde liefern:
# Geometric_Means <- apply(Logarithmierter_Datensatz, 2, mean)


# Normierung der Experssionswerte auf das geometrische Mittel jedes Gens:
# Vom Logarithmus jedes Einzelwertes wird das Geometrische Mittel 
# als Mittelwert der Log10-Werte des jeweiligen Gens subtrahiert. 
# Damit jedes Vektor-Element von den Werten der richtigen Spalte 
# subtrahiert wird, muss die Matrix zunächst transponiert und
# das Zwischen-Ergebnis anschliessend rück-transponiert werden
# um das richtige Ergebnis zu erhalten:

Geo_Norm_Datensatz <- t (t (Logarithmierter_Datensatz) - Geometric_Means)

# Normierten Datensatz zur Dokumentation als Excel abspeichern, erste Spalte einbeziehen:

my_Tab [,4:(Gene+3)] <- Geo_Norm_Datensatz [,1:Gene]

FileName <- paste(sep="", "(1) ", FileTag, "_Geo_Normalisiert.xlsx")
write.xlsx(cbind (rownames (my_Tab), my_Tab), FileName, asTable = FALSE, keepAN = TRUE, zoom = 150)

# Wurde durch Nachrechnen in Excel validiert.

Refgen <- my_Tab


# Dann muss der Datensatz zur Weiterverarbeitung noch transponiert werden:
data <- t(Geo_Norm_Datensatz)
data <- data[Genes_analysed,]
nrow(data) # wieviele Zeilen hat der data.frame jetzt noch?


# Werte ggf. entLogarithmieren und FileTag entsprechend ergänzen:
if (LogScale)
{ 
  data_norm<-data
  FileTag <- paste0(FileTag, "_Log")
} else {
  data_norm<-10^(data)
  FileTag <- paste0(FileTag, "_Lin")
} # end of else

# data_norm






# Histogramm aller normalisierten Messerte erzeugen:
LangeListe <- as.vector(data_norm)
{pdf (paste(sep="", "(2) Alle_Werte_", FileTag, ".pdf"),width=20,height=15)
  histogram<-hist(LangeListe, 
                  main=(paste0 ("Datafile: ", Datenfile, " // Normierung = ", Normierung, " // Total number of values: ", length(LangeListe))), 
                  xlab= paste0("Gene Expression Level, LogScale = ", LogScale), 
                  border="black", 
                  col="grey",
                  xlim=c(min(LangeListe),max(LangeListe)),
                  #ylim=c(0,800)
                  las=1, 
                  breaks=50)
  dev.off()}



#------------------- ----------------------------------------------------

# Simprof -----------------------------------------------------------------
# Zahl der signifikanten Cluster mit Simprof ermitteln:
# simprof_alpha <- 0.01 oben definiert

# Simprof nur ausführen, wenn Nur_HeatMaps = FALSE gesetzt ist

if(!Nur_HeatMaps) { # begin of if not Nur_Heatmaps 
probes<-simprof(t(data_norm),method.cluster="average", method.distance="euclidean",alpha=simprof_alpha)

pdf(paste(sep="", "(3) Simprof_plot_", FileTag, ".pdf") ,width=20,height=10)
simprof.plot(probes)
dev.off()

# Simprof Cluster als Tabelle ausgeben
x <- as.data.frame.vector(probes$significantclusters)
# x
cat(capture.output(print(x), file=paste(sep="", "(4) Simprof_Tab_", FileTag, ".txt") ))
# Gibt die Zahl der signifikanten cluster an

# Zeilen des data.frames gibt die Zahl der signifikanten Cluster
simprof_significant_Clusters <- nrow(x)

print (simprof_significant_Clusters)

# ----------------------------------------------------



# Simprof-Ergebnisse eindeutig benennen:
Simprof_Output <- x
simprof_significant_Clusters <- nrow(Simprof_Output)

# Leere Matrix erzeugen mit 4 Spalten und Zahl der Plasmodien-Namen als Zeilen:
Zahl_der_Plasmodien_Namen <- nrow(Refgen)
Sim_Prof_Cluster_Liste <- matrix(0,Zahl_der_Plasmodien_Namen,4)

Current_Line <- 1

# Simprof_Output sequentiell, d.h. clusterweise auslesen und in die Matrix 
# eintragen, wobei die Plasmodien-Namen auseinander gebaut werden müssen, damit 
# dann nach aufsteigender Plasmodien-Nummer und Zeitpunkt sortiert werden kann:
# Neu für Zeitpunkte mit Dezimalstelle: "ZeitWert <- as.numeric" sonst wird falsch sortiert

for (cl in 1:simprof_significant_Clusters) 
     { # begin of for cl....
        Plasmodien_des_Clusters <- Simprof_Output[cl,1]
        Plasmodien_im_Vektor <- unlist(Plasmodien_des_Clusters, use.names=FALSE)
 
        for (i in 1:length(Plasmodien_im_Vektor)) 
            { # begin of for i.....
              Sim_Prof_Cluster_Liste [Current_Line,1] <- Plasmodien_im_Vektor[i]
              
              InSpalten <- str_split(Plasmodien_im_Vektor[i], "_")
              Plasmodium <- map_chr(InSpalten, 1) # Ordnungsnummer 1-..
              Zeit       <- map_chr(InSpalten, 2) 
              Plasmodien_Nummer <- as.integer(str_replace(Plasmodium, pattern = "P", "")) # "P" entfernen
            # "as.numeric" für Zeitpunkte mit Dezimalstellen verwenden:
            # ZeitWert <- as.integer(str_replace(Zeit, pattern = "h", "")) # "h" entfernen
              ZeitWert <- as.numeric(str_replace(Zeit, pattern = "h", "")) # "h" entfernen
              Sim_Prof_Cluster_Liste [Current_Line,2] <- abs(Plasmodien_Nummer)
              Sim_Prof_Cluster_Liste [Current_Line,3] <- ZeitWert
              
              Sim_Prof_Cluster_Liste [Current_Line,4] <- cl
              Current_Line <- Current_Line + 1
             } # end of for i....
      } # end of for cl....


# Zum Sortieren muss die Matrix (die scheinbar immer nur von einem Typ sein kann)
# in einem data.frame umgewandelt werden, dessen Spalten dann einzeln
# in "numeric" convertiert und entsprechend numerisch sortiert werden können.
# This sorting works now
mydf <- as.data.frame (Sim_Prof_Cluster_Liste, stringsAsFactors=FALSE)
mydf[, 2] <- as.numeric(mydf[, 2]) # spaltenweise in numeric umwandeln
mydf[, 3] <- as.numeric(mydf[, 3])
mydf[, 4] <- as.numeric(mydf[, 4])
# und nach der 2. und 3. Spalte aufsteigend sortieren:
richtig_sortiert <- mydf[order(mydf[, 2], mydf[, 3]), ] 

# Umwandeln des sortierten data.frames in Vektor mit Namen:
myPlasmodialNames <- richtig_sortiert[,1]
myClusternumbers <- richtig_sortiert[,4]

my_Simprof_Cluster_Assignment <- as.integer(myClusternumbers)
names(my_Simprof_Cluster_Assignment) <- myPlasmodialNames

Simprof_Trajctories <- as.matrix(my_Simprof_Cluster_Assignment)

# Weiter mit: my_Simprof_Cluster_Assignment und Simprof_Trajctories
# Der dann statt mycl verwendet werden soll, um den colored side bar der Heatmap
# und, most important, die Annotation der Plasmodien-Namen, auch für die MDS-Plots
# zu erzeugen
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# hclust und Simprof liefern verschiedene Cluster-Verteilungen (!!!!)


# Simprof-Ergebnisse speichern 
# save.image("~/Desktop/myR/(4) Simprof_Results.RData")
save.image(paste0(getwd(), "/(5) Simprof_Results_", FileTag, ".RData"))


#===============================================================
# # ggf. wieder laden mit:
# load("~/Desktop/myR/(4) Simprof_Results.RData")
# # Hier müssen dann aber die Klassen neu zugeordnet werden
# # Anfang des Programms (siehe oben) (siehe Tabelle):
# Klassenzimmer <- c(13, 31, 22, 63)
# 
# # zur Kontrolle muss Zahl der Plasmodien geben:
# sum(Klassenzimmer)
# 
# Plasmodien_Klassen <- length(Klassenzimmer)
# Plasmodien_Klassen
#=============================================================

# Plasmodien und Simprof Cluster-Nummern in Reihenfolge der Zeitpunkte
# und somit die Trajektorien,  als Textfile abspeichern ausgeben:

filename <- paste(sep="", "(6) Simprof_Trajectories_List_", FileTag, ".txt")
cat(capture.output(print(Simprof_Trajctories), file=filename))



         } # end of if not Nur_Heatmaps 

# ---------------------------------------------------------------------------------------




# Wenn die Cluster identifiziert sind, kann die 
# Heatmap mit den Clustern als Sidebar erzeugt werden:
# ----------------------------------------------------

# Die Farben für die Heatmap werden bei linerarer Darstellung so eingestellt, 
# dass sie eine Skala von 0 bis 100 % abbilden:
# colors <- c(seq(0,15,length=100),seq(16,30,length=100),seq(31,45,length=100),seq(46,60,length=100),seq(61,75,length=100),seq(76,90,length=100),seq(91,100,length=100) )
# Da wir hier mit Logarithmen arbeiten, wird der Vektor "colors" besser automatisch erzeugt:

# Heatmap Skala wird am Anfang des Programms festgelegt,
# abhängig davon, ob die Werte logarithmisch geplottet werden.

# Constanten unverändert lassen:
Abschnitte  <- 7 
LengthSteps <- 100

# Vektor mit Wertintervallen "colors" automatisch erzeugen: 
Intervall_Breite <- (Heat_Map_Skala_bis - Heat_Map_Skala_von) / Abschnitte
Farbstufen <- seq (Heat_Map_Skala_von, Heat_Map_Skala_bis, by=Intervall_Breite)
Increment  <- Intervall_Breite / LengthSteps
Start <- 1
Stop  <- 2
colors <- numeric()  # Leere Variable "numeric" erzeugen

for (k in 1:7) 
       { colors <- c (colors, seq((Farbstufen[Start]+ Increment),Farbstufen[Stop],length=LengthSteps))
         Start <- Start + 1
         Stop  <- Stop  + 1
        } # end of for....


# Neue Coole Farb-Palette 4 (Blau/Rot) für die Heatmap
my_palette <- colorRampPalette(c("darkblue", "white", "red"))(n=((Abschnitte * LengthSteps)-1))



if (!Nur_HeatMaps)   # wird nur ausgeführt, wenn auch Simprof gelaufen ist
          # sonst erfolgt eine Fehlermeldung wegen fehlender ColorSideBars
  
       { # begin of if not Nur_HeatMaps
  
       # Rainbow ColorSideBar, wird aus der Simprof-Zuuordnung der Cluster erzeugt
       # get a color palette equal to the number of clustersmycl:
       clusterCols <- rainbow(length(unique(my_Simprof_Cluster_Assignment)))
       # create vector of colors for side bar
       myClusterSideBar <- clusterCols[my_Simprof_Cluster_Assignment]

if (Two_Color_Sidebar) 
      { # begin of if Two_Color_Sidebar
  # ---------------------------------------------------------
  # Zweifarbiger Cluster-Side-Bar für die Heatmap
  # ---------------------------------------------------------
  # Einen Vektor erzeugen, bei dem gerade Cluster-Nummern 
  # einer Farbe entsprechen, ungerade, einer anderen:
  
  # Erzeugen eines Bool'schen Vektors:
  # ist Wert durch 2 teilbar also eine gerade Zahl?
  myBoolean <- my_Simprof_Cluster_Assignment %% 2 == 0  
  # leeren Char-Vektor passender Länge erzeugen
  myColor <-character(length (myBoolean)) 
  # TRUE und FALSE durch Farben austauschen
  for (i in 1:length (myBoolean)) 
  { if (myBoolean[i] == "FALSE") 
  { myColor[i] <- "tan1" } else
  { myColor[i] <- "springgreen" }}
  # Dem Farbvektor für die Heat-Map zuordnen:
  myClusterSideBar <- myColor
  # --------------------------------------------------------
  } # end of if Two_Color_Sidebar


    # Plasmodien-Namen mit Simprof- Cluster-Nummern ausstatten:
    dieNamen <- colnames (data_norm)
    colnames (data_norm) <- paste0(dieNamen, "_c", my_Simprof_Cluster_Assignment)
 
        } else 
        { # begin of else...
            Zeilen <- length(Logarithmierter_Datensatz [,1])
            myClusterSideBar <- rep("white", Zeilen)
            # macht einen einfarbigen SideBar
        } # end of else....





hc <- hclust(dist(t(data_norm)),method="average")



if (Nur_HeatMaps) { # begin of if Nur_HeatMaps
# Wenn nur Heatmaps, dann ohne SideBar und Dendrogramm 
# entsprechend direkt an die Heatmap rücken

# Heatmap mit beiden Dendrogrammen:

{pdf (paste(sep="", "(7) heatmap_", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(t(data_norm)),
                     main=FileTag,
                     Rowv=as.dendrogram(hc),
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(data_norm) hclust(data_norm,method="average"),
                     cexCol=xsize, cexRow=ysize, 
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=2) 
                     # RowSideColors= myClusterSideBar)
  dev.off()}


# Heatmap mit Genen nach Custom numbers geordnet:

{pdf (paste(sep="", "(8) heatmap_cust_", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(t(data_norm)),
                     main=FileTag,
                     Rowv=as.dendrogram(hc),
                     Colv=FALSE,
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(data_norm) hclust(data_norm,method="average"),
                     cexCol=xsize, cexRow=ysize,
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=2)
                     # RowSideColors= myClusterSideBar)
  dev.off()}

} else { # ClusterSideBar einschliessen

  # Heatmap mit beiden Dendrogrammen:

{pdf (paste(sep="", "(7) heatmap_", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(t(data_norm)),
                     main=FileTag,
                     Rowv=as.dendrogram(hc),
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(data_norm) hclust(data_norm,method="average"),
                     cexCol=xsize, cexRow=ysize, 
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=2, 
                     RowSideColors= myClusterSideBar)
  dev.off()}


# Heatmap mit Genen nach Custom numbers geordnet:

{pdf (paste(sep="", "(8) heatmap_cust_", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(t(data_norm)),
                     main=FileTag,
                     Rowv=as.dendrogram(hc),
                     Colv=FALSE,
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(data_norm) hclust(data_norm,method="average"),
                     cexCol=xsize, cexRow=ysize,
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=2,
                     RowSideColors= myClusterSideBar)
  dev.off()}

  }  # end of else....




# Hier werden in die Matrix der Zeitserien Leerzeilen eingefügt,
# damit jede Zeitserie in der Heatmap durch eine Leerzeile 
# von der nächsten Zeitserie abgetrennt ist: 

# Das erfolgt in drei Schritten:
# 1. Erst wird ein Vektor mit den Rownames erzeugt, so dass 
#    in die Beschriftung Lücken eingeführt werden
# 2. Dann werden die Leerzeilen in die Matrix eingeschoben
# 3. Schliesslich wird die neue Matrix mit der passenden Beschriftung versehen

# 1. Erzeugung eines Vektors mit den neuen Rownames:
# mat = mat als Matrix
# Zeit_Serien_mit_Leerzeile = matLZ

mat <- as.matrix(t(data_norm))
Zeilen_Namen <- rownames(mat)
# Erzeugt einen Vektor mit Positionen, an denen die jeweils erste
# Zeile eines Blocks (z.B 0h bis 10h) steht (in der Matrix ohne Leerzeilen)
vec <- seq(Zeitpunkte,nrow(mat), by=Zeitpunkte)
# Hängt die Leerzeilen an, die benötigt werden, um die Blöcke zu trennen:
val <- c( Zeilen_Namen, rep(" ", length(vec))) 
# Ranks und Half-Ranks erzeugen; weist den Positionen der Leerzeilen halbe Ranks zu:
id  <- c( seq_along(Zeilen_Namen), vec+0.5 )
# Sortiert nach den Ranks, so dass die Leerzeilen nun die Blöcke trennen:
Neue_Zeilen_Namen <- val[order(id)]


# 2. Einschieben der Leerzeilen in die Matrix:

# Initialisiert a
a <- vec

# Erzeugt einen Vektor a mit Positionen an denen eine Leerzeile in die 
# Matrix eingeschoben werden soll. Mit jeder eingefügten Leerzeile 
# steigt die Position des nächsten Einschubs, daher die Schleife
  j <- 1
  for (i in 1:length(vec))
  { a[i] <- vec[i] + j
    j <- j+1
  }
  
# Zahl der Zeilen mit Leerzeilen: 
Neue_Zeilen_Zahl <- nrow(mat) + length(a)

# Erzeugt eine leere Matrix mit der Zeilenzahl incl. der einzufügenden Leerzeilen
matLZ <- matrix(0,nrow=Neue_Zeilen_Zahl,ncol=ncol(mat)) 

# Kopiert die Matrix ohne Leerzeilen in die Matrix mit Leerzeilen, wobei 
# die in a genannten Positionen der Leerzeilen ausgelassen werden.
matLZ[-a,] <- mat   

# 3. Beschriftung der neu erzeugten Matrix 
# Ordnet die altualisierten Rownames (mit Lücken an den Positionen der Leerzeilen) zu
rownames(matLZ) <- Neue_Zeilen_Namen
colnames(matLZ) <- colnames(mat)

# Die Matrix mit Leerzeilen steht nun in matLZ




# Heatmap der Genexpressionswerte als Zeitserie für jedes Plasmodium :

{pdf (paste(sep="", "(9) heatmap_Time_series_", FileTag, ".pdf"),width=10,height=30)
  # heatmap<-heatmap.2(as.matrix(t(data_norm)),
                     heatmap<-heatmap.2(matLZ,
                     main=FileTag,
                     Rowv=FALSE,
                     Colv=TRUE,
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(new_mat) hclust(new_mat,method="average"),
                     cexCol=xsize, cexRow=ysize,
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=3 )
                     # RowSideColors= myClusterSideBar)
  dev.off()}



{pdf (paste(sep="", "(10) heatmap_Time_series_cust_", FileTag, ".pdf"),width=10,height=30)
  heatmap<-heatmap.2(matLZ,
                     main=FileTag,
                     Rowv=FALSE,
                     Colv=FALSE,
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(data_norm) hclust(data_norm,method="average"),
                     cexCol=xsize, cexRow=ysize,
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=5) 
                     # RowSideColors= myClusterSideBar)
  dev.off()}








  
  
# Rest des Programms nur ausführen, wenn Nur_HeatMaps = FALSE ist:

 if(Nur_HeatMaps==FALSE) {
  
  Zahl_der_Zeilen <- nrow(Simprof_Trajctories) / Zeitpunkte
  Plasmodium_1_bis_n <- paste ( "P", rep(1:Zahl_der_Zeilen), sep='')  # P1 bis P26
# Trajektorien als Tabelle darstellen, mit Zeitpunkten als Spalten
# Leere Matrix mit Beschriftung Plasmodien vs. Zeitpunkte erzeugen

Trajektorien_Matrix <- matrix (0,Zahl_der_Zeilen,Zeitpunkte)  
dimnames (Trajektorien_Matrix) <- list (Plasmodium_1_bis_n, Zeitpunkte.h)

die_Cluster <- as.vector(Simprof_Trajctories [,1])
k <- 1

for (i in 1:Zahl_der_Zeilen)
    for (t in 1:Zeitpunkte)
        { Trajektorien_Matrix [i,t] <- die_Cluster[k]
          k <- k+1
        } # end of for i.....

# und als File ausgeben:
filename <- paste(sep="", "(11) Simprof_Trajektorien_tab_", FileTag, ".txt")
cat(capture.output(print(Trajektorien_Matrix), file=filename))

filename <- paste0("(12) Simprof_Trajektorien_tab_", FileTag, ".xlsx")
# write.xlsx(Trajektorien_Matrix, row.names=TRUE, col.names=TRUE, filename)
write.xlsx(Trajektorien_Matrix, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)


# Simprof-Trajektorien mit Plasmodien-Klassen Zuordnung:

P <- Plasmodial_Class_Assignment
Trajektorien_mit_Klassen <- cbind(P, Trajektorien_Matrix)
# und als File ausgeben:
filename <- paste(sep="", "(13) Simprof_Trajektorien_Classified_", FileTag, ".txt")
cat(capture.output(print(Trajektorien_mit_Klassen), file=filename))

filename <- paste(sep="", "(14) Simprof_Trajektorien_Classified_", FileTag, ".xlsx")
# write.xlsx(Trajektorien_mit_Klassen, row.names=TRUE, col.names=TRUE, filename)
write.xlsx(Trajektorien_mit_Klassen, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)

# }



# Hier neu:
# Stabilitätsberechung der Cluster (PN-Nodes):
# ***********************************************************************
# Die Trajektorien-Matrix wird in einzelne Zeilen zerlegt und in jeder
# Zeile die Stabilität der Cluster ermittelt:

# Letzte Spalte von der Trajektorien-Matrix wegnehmen, nur
# die Cluster, die vor der letzten Spalte vorkommen, dürfen
# gezählt werden:
# Cluster_Occurrences_to_be_Counted = Cotbc



Plasmodien_Zahl <- nrow(Trajektorien_Matrix)

Cl_Nr <- (1:max(Trajektorien_Matrix))

# Matrix zeilenweise in Vektor umwandeln (muss dazu transponiert werden):
Clusters_observed <- as.integer(t (Trajektorien_Matrix))

# Häufigkeit jedes Clusters:
Cl_freq <- (table(Clusters_observed))


# Initialisieren als Vektor mit Nullen:
Selber_Cluster <- integer(max(Trajektorien_Matrix))
Blöcke_eines_Clusters <- integer(max(Trajektorien_Matrix))
# Zahl der Separaten Blöcke eines Clusters

# Zahl der Separaten Blöcke eines Clusters ermitteln:
# Dabei wird die Trajektorien-Matrix zeilenweise (p) abgearbeitet,
# und jede Zeile Zeitpunkt für Zeitpunkt (i) abgearbeitet
# Jede Zeile entspricht einem Plasmodium
for (p in 1:Plasmodien_Zahl)
   { 
      Ctr <- Trajektorien_Matrix [p,]  # Ctr, Current trajectory
      for (i in 1:(length(Ctr)-1))   
        { if (Ctr[i] != Ctr[i+1]) # wenn sich die Cluster-Nummer ändert, gibt es einen neuen Block
            { Blöcke_eines_Clusters [Ctr[i]] <- Blöcke_eines_Clusters [Ctr[i]] + 1 } # end of if...
         } # end of for i..... 
  
  # Berücksichtigt die in der Trajektorien-Tabelle endständigen Cluster
  # einer jeden Zeile, weil hier keine Änderung auftreten kann. Daher
  # besteht besteht eine endständige Insel aus mindestens 2 Elementen
      # if (Ctr[length(Ctr)-1] == Ctr[length(Ctr)]) 
      #    { Blöcke_eines_Clusters [Ctr[length(Ctr)]] <- Blöcke_eines_Clusters [Ctr[length(Ctr)]] + 1 } # end of if...
      # 
      if (Ctr[length(Ctr)-1] == Ctr[length(Ctr)]) 
         { Blöcke_eines_Clusters [Ctr[length(Ctr)]] <- Blöcke_eines_Clusters [Ctr[length(Ctr)]] + 1 } # end of if...
         
      # wenn in der letzten Spalte, also zum letzten Zeitpunkt, noch mal ein
      # neuer Cluster beginnt, darf er in der Häufigkeitsverteilung nicht gezählt
      # und muss entsprechend daraus entfernt werden:
      if (Ctr[length(Ctr)-1] != Ctr[length(Ctr)])  
         { Cl_freq [Ctr[length(Ctr)]] <- Cl_freq [Ctr[length(Ctr)]] - 1 }
  
   } # end of for p.....



# Zahl der Blöcke füe jeden Cluster
Islets <- Blöcke_eines_Clusters


Stability <- Cl_freq / Islets
# Unendliche Stabilitäts-Werte entsprechen Terminal states

Stability_Table <- cbind(Cl_Nr, 
                         Cl_freq,
                         Islets, 
                         Stability)


# und als File ausgeben:
filename <- paste(sep="", "(15) Cluster_Stability_", FileTag, ".txt")
cat(capture.output(print(Stability_Table), file=filename))

filename <- paste(sep="", "(16) Cluster_Stability_", FileTag, ".xlsx")
# write.xlsx(Trajektorien_mit_Klassen, row.names=TRUE, col.names=TRUE, filename)
write.xlsx(Stability_Table, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)



# ENDE: Stabilitätsberechung der Cluster (PN-Nodes)
# ***********************************************************************










# if(!Nur_HeatMaps) {

# --------------------------------------------------------------------------
# Das Vorkommen der Cluster in den einzelnen Klassen vergleichen:
# --------------------------------------------------------------------------
# Leere Matrix erzeugen:
myMatrix <- matrix(" ", Plasmodien_Klassen, simprof_significant_Clusters)

for (k in 1:Plasmodien_Klassen) 
        { # begin
            mydf <- as.data.frame(Trajektorien_mit_Klassen)
            Block_der_Klasse <- subset(mydf, P==k)
            die_Cluster_der_Klasse <- Block_der_Klasse[,2:(Zeitpunkte+1)]
            die_Cluster_als_Vektor <- unlist(die_Cluster_der_Klasse)
            die_Cluster_unique_und_sortiert <- sort(unique(die_Cluster_als_Vektor))
            print (die_Cluster_unique_und_sortiert)
            uset <- die_Cluster_unique_und_sortiert
            
            for (Pos in 1:length (uset)) 
                { # begin of for pos...
                  myMatrix [k, uset[Pos]] <- uset[Pos]
                 } # end of for pos...
         } # end of for k

# transponieren und abspeichern: 
adf <- t (as.data.frame (myMatrix))
filename <- "(18) Cluster_of_Classes.xlsx"
# write.xlsx(adf, row.names=TRUE, col.names=TRUE, filename)
write.xlsx(adf, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)
# --------------------------------------------------------------------------



# Klassifizieren und sortieren:
TmC <- Trajektorien_mit_Klassen

TmC_Sorted <- TmC[order(TmC[,1],
                          (TmC[,2]),
                          (TmC[,3]),
                          (TmC[,4]),
                          (TmC[,5]),
                          (TmC[,6]),decreasing = F),]

# und als File ausgeben:
filename <- paste(sep="", "(19) Simprof_Trajektorien_Classified_Sorted_", FileTag, ".txt")
cat(capture.output(print(TmC_Sorted), file=filename))

filename <- paste(sep="", "(20) Simprof_Trajektorien_Classified_Sorted_", FileTag, ".xlsx")
# write.xlsx(TmC_Sorted, row.names=TRUE, col.names=TRUE, filename)
write.xlsx(TmC_Sorted, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)


# Cluster nach steigenden Nummern sortiert, KEINE Trajektorien (!!!)

# Leere Matrix erzeugen und beschriften:
Cluster_Sortiert <- matrix (0,Zahl_der_Zeilen,Zeitpunkte) 
leerSpalte <- character()
dimnames (Cluster_Sortiert) <- list (leerSpalte, Zeitpunkte.h)

for (t in 1:Zeitpunkte)
{ # begin
  Sortiert <- sort (Trajektorien_Matrix [,t], decreasing = FALSE)
  Cluster_Sortiert [,t] <- Sortiert
} # end of for t...

# und als File ausgeben:
filename <- paste(sep="", "(21) Simprof_Cluster_steigend_", FileTag, ".txt")
cat(capture.output(print(Cluster_Sortiert), file=filename))

                                                                      


# Eine Häufigkeitsverteilung für jede Plasmodien-Klasse ausgeben:   # !!!!!!!!!!!!!!!!!!!!!!!! #
                                                                    # height ggf an die Klassenzahl anpassen
{pdf (paste(sep="", "(22) Freq_of_Classes_", FileTag, ".pdf"), width = 4, height = 18)
  # Alle Plots in einem gemeinsamen Panel rows x colums anordnen
  # Format wurde oben via "width = 4, height = 12" eingestellt
  par(mfrow = c(Plasmodien_Klassen, 1))  # "Plasmodien_Klassen" rows and 1 columns, d.h. alle untereinander
  { # begin of par...
    for (t in 1:Plasmodien_Klassen)
    { # begin of for t....
      # Schreibt die Simprof-Trajektorien der aktuellen Plasmodien_Klasse aus TraCl  
      # in den data.frame Block, schreibt alle Cluster-Nummern in einen Vektor
      # von dem dann mit table die Häufigkeitsverteilung erzeugt wird:
   
      TraCl <- as.data.frame(TmC)
      Block <- filter(TraCl, P %in% t)
      Cluster.Verteilung <- table(as.vector (unlist(Block[,2:Zeitpunkte+1])))
      
      freq <- as.matrix(Cluster.Verteilung)
      height <- as.vector(freq)  
     
      # Für jeden Zeitpunkt wird eine Tabelle mit allen Cluster-Nummern
      # angelegt, die dann als barplot geplottet wird. In diese Tabelle
      # müssen die Häufigkeitsverteilungen eingetragen werden
      # Tabelle erzeugen und vorbereiten
      # Gesamt_Zahl der Cluster ermitteln:
      Zahl_der_Cluster <- nrow(table (Simprof_Trajctories))
      lines <- Zahl_der_Cluster  # Zahl der cluster
      cols   <- 2  # Cluster Nr, Häufigkeit
      Tabelle <- matrix(0, lines, cols) # mit Nullen füllen
      Tabelle [, 1] <- 1:Zahl_der_Cluster  # Erste Spalte von 1 bis 20
      
      CurrentCluster <- as.integer(rownames(freq))
      
      for (i in 1:nrow(freq))
      { # begin of for i....
        Tabelle[CurrentCluster[i], 2] <- height[i]
      } # end of for i...
      
      height <- as.vector(Tabelle[,2])  # Einschub zum Übergang  
      # Das Lesen aus der Tabelle erlaubt, dass alle Plots die selbe Skalierung der x-Achse haben
      PlotTitel <- paste0("Simprof-Cluster der Plasmodien-Klasse = ", t, "   // ", FileTag)
      
      barplot(height, 
              beside = TRUE,
              space = 0,
              # names.arg = rownames(freq), 
              # names.arg = rownames(Tabelle), 
              names.arg = Tabelle[,1], 
              ylim = NULL,
              main = PlotTitel, cex.main=0.7,
              axisnames = TRUE, 
              xlab = "Cluster Nr",
              ylab = "Frequency",
              cex.names = 0.65, 
              cex.axis = 0.7)
    } # end of for t.......
  } # end of par...
  dev.off()}
# ----------------------------------------------------------------------------------------






# Hier werden Histogramme (Häufigkeit der einzelnen Cluster) für jeden Zeitpunkt ermittelt:
# ----------------------------------------------------------------------------------------
# Leeren data.frame mit Überschriften erzeugen (initialisieren)
Verteilung = data.frame(TimePoint = character(0), 
                        Cluster_Nr  = numeric(0), 
                        Häufigkeit  = numeric(0));

# Einzelne Histogramme auslesen und zu einer Tabelle verknüpfen:

for (t in 1:Zeitpunkte)
    { 
       Plasmodien_eines_Zeitpunktes <- Simprof_Trajctories[grep(Zeitpunkte.h[t], row.names(Simprof_Trajctories)),]
       Cluster.Verteilung <- table(Plasmodien_eines_Zeitpunktes)
       freq <- as.matrix(Cluster.Verteilung)
       # in Vektoren umwandeln:
       Cluster_Nr <- as.vector (rownames(freq))
       Häufigkeit <- as.vector (freq[,1])
       TimePoint     <- rep(Zeitpunkte.h[t],nrow(freq))  # Spalte mit Zeitpunkten
       # Verteilungen aneinander fügen:
        Verteilung_at_TimePoint <- as.data.frame(cbind (TimePoint, Cluster_Nr, Häufigkeit))
        Verteilung <- rbind (Verteilung, Verteilung_at_TimePoint)
    } # end of for t.......

# Als File ausgeben:
filename <- paste(sep="", "(23) Histogramme_nach_Zeitpunkten", FileTag, ".txt")
cat(capture.output(print(Verteilung), file=filename))


# Geht nicht mehr, weil es hier zu viele Zeitpunkte sind!
# 
# # Als Histogramme untereinander plotten:
# #----------------------------------------------------------------------------------------------
# {pdf (paste(sep="", "(19) Häufigkeit_pro_Zeitpunkt_", FileTag, ".pdf"), width = 4, height = 12)
#     # Alle Plots in einem gemeinsamen Panel rows x colums anordnen
#     # Format wurde oben via "width = 4, height = 12" eingestellt
#     par(mfrow = c(Zeitpunkte, 1))  # "Zeitpunkte" rows and 1 columns, d.h. alle untereinander
#         { # begin of par...
#       for (t in 1:Zeitpunkte)
#           { # begin of for t....
#            Plasmodien_eines_Zeitpunktes <- Simprof_Trajctories[grep(Zeitpunkte.h[t], row.names(Simprof_Trajctories)),]
#            Cluster.Verteilung <- table(Plasmodien_eines_Zeitpunktes)
#            freq <- as.matrix(Cluster.Verteilung)
#            height <- as.vector(freq)  # doppelte zuordnung ?, später raus ?
#            
#            # Für jeden Zeitpunkt wird eine Tabelle mit allen Cluster-Nummern
#            # angelegt, die dann als barplot geplottet wird. In diese Tabelle
#            # müssen die Häufigkeitsverteilungen eingetragen werden
#            # Tabelle erzeugen und vorbereiten
#            # Gesamt_Zahl der Cluster ermitteln:
#            Zahl_der_Cluster <- nrow(table (Simprof_Trajctories))
#            lines <- Zahl_der_Cluster  # Zahl der cluster
#            cols   <- 2  # Cluster Nr, Häufigkeit
#            Tabelle <- matrix(0, lines, cols) # mit Nullen füllen
#            Tabelle [, 1] <- 1:Zahl_der_Cluster  # Erste Spalte von 1 bis 20
# 
#            CurrentCluster <- as.integer(rownames(freq))
# 
#            for (i in 1:nrow(freq))
#                 { # begin of for i....
#                    Tabelle[CurrentCluster[i], 2] <- height[i]
#                  } # end of for i...
#         
#            height <- as.vector(Tabelle[,2])  # Einschub zum Übergang  
#            # Das Lesen aus der Tabelle erlaubt, dass alle Plots die selbe Skalierung der x-Achse haben
#            PlotTitel <- paste0("Häufigkeit der Simprof-Cluster at t = ", Zeitpunkte.h[t], "   // ", FileTag)
#               
#           barplot(height, 
#                beside = TRUE,
#                space = 0,
#                # names.arg = rownames(freq), 
#                # names.arg = rownames(Tabelle), 
#                names.arg = Tabelle[,1], 
#                ylim = NULL,
#                main = PlotTitel, cex.main=0.7,
#                axisnames = TRUE, 
#                xlab = "Cluster Nr",
#                ylab = "Frequency",
#                cex.names = 0.65, 
#                cex.axis = 0.7)
#           } # end of for t.......
#         } # end of par...
#    dev.off()}
# # ----------------------------------------------------------------------------------------


# Und nun die gesamte Häufigkeit der Cluster über alle Zeitpunkte:

Gesamte_Verteilung <- table(Simprof_Trajctories)
freq <- as.matrix(Gesamte_Verteilung)

Cluster_Nr <- as.integer (rownames(freq))
Häufigkeit <- as.vector (freq[,1])

Verteilung <- as.data.frame (cbind (Cluster_Nr, Häufigkeit))

# Als Text File ausgeben:
filename <- paste(sep="", "(24) Häufigkeit_der_Cluster_", FileTag, ".txt")
cat(capture.output(print(Verteilung), file=filename))


# Als Verteilung plotten via barplot:
height <- as.vector(freq)
PlotTitel <- paste0("Gesamt-Häufigkeit der Simprof-Cluster  // ", FileTag)

{pdf (paste(sep="", "(25) Häufigkeit_der_SimProf_Cluster_", FileTag, ".pdf"))
  barplot(height, 
          beside = TRUE,
          space = 0,
          names.arg = rownames(freq), 
          ylim = NULL,
          main = PlotTitel, cex.main=1,
          axisnames = TRUE, 
          xlab = "Cluster Nr",
          ylab = "Frequency",
          cex.names = 0.7, 
          cex.axis = 0.7)
  
  dev.off()}

# ----------------------------------------------------------------------------------------





# Hier werden die Initiations-Events für das später zu erzeugende Petri-Netz ermittelt:
# -------------------------------------------------------------------------------------
# Hier werden alle Cluster, dem ein Plasmodium zum Zeitpunkt t=0h angehört 
# dem Integer Vektor nullh zugeordnet 
nullh <- Simprof_Trajctories[grep("_0h", row.names(Simprof_Trajctories)),]
nullh

# und deren Häufigkeitsverteilung bestimmt
nullh.freq <- table(nullh)

# class(nullh.freq)   # ist eine Table
# Lässt sich als Matrix betrachten:
# as.matrix(nullh.freq)  
# oder direkt als Tabelle
# erste Spalte = rownames = Cluster-Nummer zum Zeitpunkt t=0
# zweite Spalte = Zahl der Plasmodien in diesem Cluster zum Zeitpunkt t=0

# Nun werden die Einträge der Häufigkeits-Verteilung direkt in Vektoren umgewandelt:

# Vektor mit einer Zahl von Nullen erzeugen, die der Zahl der Einträge (=Intiations-Events) entspricht
Null_Cluster     <- rep(0,nrow(nullh.freq))  

# Cluster Cx zum Zeitpunkt t=0
Cluster_Nr_at_0h <- as.integer(rownames(nullh.freq))  

# Häufigkeit des Übergangs C0 nach Cx
Frequenz         <- as.vector(nullh.freq)  

# Diese drei Vektoren zu einer Matrix vereinigen:
PN_Init_Matrix <- t(cbind (Null_Cluster, Cluster_Nr_at_0h, Frequenz))
PN_Init_Matrix

# In einen Vektor umwandeln entsprechend dem Format des Petri-Netz Files
# Cvon, Cnach, Rate etc.:

Start_ins_Netz <- as.vector(PN_Init_Matrix)   # wird später verwendet


# Directory für die PN Files anlegen
mainDir <- getwd()
# mainDir <- "~/Desktop/myR"
subDir <- "(96) Petri Net Construction Doku"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))


# Nur zur Dokumentation als Tabelle speichern:
filename <- paste(sep="", "(26) Petri_Net_Init_Doku_", FileTag, ".txt")
write.table(PN_Init_Matrix, filename, sep="\t", col.names = F)


# Directory zurücksetzen
# setwd("~/Desktop/myR")
setwd(mainDir)

# -------------------------------------------------------------------------------------





# # Ursprünglicher Datensatz ist bereits mit Cluster-Nummern versehen:
# ---------------------------------------------------------------------
# Daten müssen zunächst transponiert werden
t_data_norm <- t(data_norm)

# und dann als Text File ausgegeben
filename <- paste(sep="", "(27) Daten_mit_clNr_", FileTag, ".xlsx")
# write.xlsx(t_data_norm, row.names=TRUE, col.names=TRUE, filename)
write.xlsx(t_data_norm, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)

# *******************************************************************************
# Vormals: Mediane der Genexpressionserte für jeden Cluster ermitteln und analysieren
#
# Neu: Geometrische Mittel der Genexpressionserte für jeden Cluster ermitteln und analysieren
# Die Logarithmen der geometrischen Mittel sind die Mittelwerte der 
# logarithmierten Einzelwerte 24-09-2019
# *******************************************************************************
# Hier werden die geometrischen Mittel der Einzelwerte der 
# Mitglieder jedes Clusters gebildet
# Die Einzelwerte sind bereits logarithmiert. 

# Im ersten Schritt Liste initialisieren.......
for(i in 1:1)
{ # begin of for i----   
  # cluster <-  10^(t_data_norm[my_Simprof_Cluster_Assignment == i,]);
  cluster <-  (t_data_norm[my_Simprof_Cluster_Assignment == i,]);
  Single <- cluster
  # mehr als eine Zeile in der Liste vorhanden?
  # wenn nicht, wird x = NULL
  x <- dim(t_data_norm[my_Simprof_Cluster_Assignment == i,]) # mehr als eine Zeile in der Liste vorhanden?
  
  # wenn x nicht NULL ist.... wird das geometrische Mittel gebildet
  if (!is.null(x)) 
  { # begin of if....
    # Median <- apply (cluster, 2, median);
    # LogGeoMean_Liste <-  rbind(Median) 
    LogGeoMean <- colMeans(cluster)
    LogGeoMean_Liste <- rbind(LogGeoMean)
  } # end of if....
  
  else { # begin of else
    # LogGeoMean_Liste <- rbind(Single) 
    LogGeoMean_Liste <- rbind(Single)
  } # end of else
  # ansonsten wird die Zeile genommen, wie sie ist.
} # end of for i.. 




# ..........dann alle Cluster durchiterieren:
for(i in 2:simprof_significant_Clusters)
{ # begin of for i----   
  # cluster <-  10^(t_data_norm[my_Simprof_Cluster_Assignment == i,]);
  cluster <- (t_data_norm[my_Simprof_Cluster_Assignment == i,]);
  Single <- cluster
  # mehr als eine Zeile in der Liste vorhanden?
  # wenn nicht, wird x = NULL
  x <- dim(t_data_norm[my_Simprof_Cluster_Assignment == i,]) # mehr als eine Zeile in der Liste vorhanden?
  
  # wenn x nicht NULL ist.... wird der Median gebildet
  if (!is.null(x)) 
  { # begin of if....
    # Median <- apply (cluster, 2, median);
    # LogGeoMean_Liste <- rbind(LogGeoMean_Liste, Median) 
    LogGeoMean <- colMeans(cluster)
    LogGeoMean_Liste <- rbind(LogGeoMean_Liste, LogGeoMean)
  } # end of if....
  
  else { # begin of else
    # LogGeoMean_Liste <- rbind(LogGeoMean_Liste, Single) 
    LogGeoMean_Liste <- rbind(LogGeoMean_Liste, Single)
  } # end of else
  # ansonsten wird die Zeile genommen, wie sie ist.
} # end of for i.. 

# if / else erforderlich, um Laufzeitfehler zu vermeiden 
# dann werden die Cluster wieder durchnummeriert
c <- c(1:simprof_significant_Clusters)

# LogGeoMean_Liste <- cbind(c, LogGeoMean_Liste)
LogGeoMean_Liste <- cbind(c, LogGeoMean_Liste)


# als Tabelle ausgeben:

filename <- paste(sep="", "(28) Mediane_der_Cluster_", FileTag, ".txt")
write.table(LogGeoMean_Liste, filename, sep="\t")



# **********************************************
# Histogramm der Mediane
# **********************************************

# Für das Histogramm die Klassifizierung als "Median" und "Single" entfernen:
nur_Cluster_Name <- as.vector(LogGeoMean_Liste [,1])
LogGeoMean_Liste_ohne_Typen_Bezeichnung <- LogGeoMean_Liste  # also ohne "Median" vs- "Single" siehe oben
rownames(LogGeoMean_Liste_ohne_Typen_Bezeichnung) <- nur_Cluster_Name
LogGeoMean_Liste_ohne_Typen_Bezeichnung <- LogGeoMean_Liste_ohne_Typen_Bezeichnung [, -1]
# Daraus einen Vektor, der die Werte aller Median-Werte aller Cluster enthält:
LogGeoMean_Wert_Liste <- as.vector(LogGeoMean_Liste_ohne_Typen_Bezeichnung)
# den logarithmieren, weil die Heatmap der Mediane auch mit den logarithmierten Werten erzeugt wird
# Log_Median_Wert_Liste <- log10(Median_Wert_Liste)

# Daraus die Häufigkeitsverteilung der Mediane als Histogramm plotten:
{pdf (paste(sep="", "(29) LogGeoMean_Verteilung_", FileTag, ".pdf"),width=20,height=15)
  histogram<-hist(LogGeoMean_Wert_Liste, 
                  main=(paste0 ("Datafile: ", Datenfile, " // Total number of values: ", length(LangeListe))), 
                  xlab= ("LogGeoMeans der Genexpressionswerte eines Clusters"), 
                  border="black", 
                  col="grey",
                  xlim=c(min(LogGeoMean_Wert_Liste),max(LogGeoMean_Wert_Liste)),
                  #ylim=c(0,800)
                  las=1, 
                  breaks=50)
  dev.off()}




# **********************************************
# Heatmap der Mediane
# **********************************************

# Dann die Median-Liste für die Heatmap vorbereiten 
# (mit Typenbezeichnung "Median" vs. "Single" (Occurrence) )
Typ_mit_Cluster_Name <- paste0(row.names(LogGeoMean_Liste), "_", LogGeoMean_Liste [,1])
LogGeoMean_Liste_mit_Typen_Bezeichnung <- LogGeoMean_Liste
rownames (LogGeoMean_Liste_mit_Typen_Bezeichnung) <- Typ_mit_Cluster_Name
LogGeoMean_Liste_mit_Typen_Bezeichnung <- LogGeoMean_Liste_mit_Typen_Bezeichnung [,-1]
# LogLogGeoMean_Liste_mit_Typen_Bezeichnung <- log10(LogGeoMean_Liste_mit_Typen_Bezeichnung)

# Heat_Map_Skala wird zu Beginn des Programms festgelegt, abhängig davon,
# ob die Werte logarithmisch geplottet werden

# Constanten unverändert lassen
Abschnitte  <- 7 
LengthSteps <- 100

# Vektor mit Wertintervallen "colors" automatisch erzeugen: 
Intervall_Breite <- (Heat_Map_Skala_bis - Heat_Map_Skala_von) / Abschnitte
Farbstufen <- seq (Heat_Map_Skala_von, Heat_Map_Skala_bis, by=Intervall_Breite)
Increment  <- Intervall_Breite / LengthSteps
Start <- 1
Stop  <- 2
colors <- numeric()  # Leere Variable "numeric" erzeugen

for (k in 1:7) 
{ colors <- c (colors, seq((Farbstufen[Start]+ Increment),Farbstufen[Stop],length=LengthSteps))
Start <- Start + 1
Stop  <- Stop  + 1
} # end of for....


# Sidebar Colors definieren:
my_LogGeoMean_Cluster_Side_Bar <- clusterCols


# Heatmap ohne Dendrogramm der Gene:

{pdf (paste(sep="", "(30) LogGeoMean_Cluster_HeatMap_1D_", FileTag, ".pdf"),width=8,height=8)
  heatmap<-heatmap.2(as.matrix(t(LogGeoMean_Liste_mit_Typen_Bezeichnung)),
                     main=FileTag,
                     Rowv= F, #as.dendrogram(hc), # Kein Dendrogramm
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(LogGeoMean_Liste_mit_Typen_Bezeichnung) 
                       hclust(LogGeoMean_Liste_mit_Typen_Bezeichnung,method="average"),
                     cexCol=0.5,cexRow=1, 
                     margins=c(6,4), # white space
                     lhei=c(1,6), keysize=1
                     , ColSideColors= my_LogGeoMean_Cluster_Side_Bar
  )
  dev.off()}


# Heatmap mit Dendrogramm der Gene:

{pdf (paste(sep="", "(31) LogGeoMean_Cluster_HeatMap_2D_", FileTag, ".pdf"),width=8,height=8)
  heatmap<-heatmap.2(as.matrix(t(LogGeoMean_Liste_mit_Typen_Bezeichnung)),
                     main=FileTag,
                     Rowv= T, #as.dendrogram(hc), # Kein Dendrogramm
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(LogGeoMean_Liste_mit_Typen_Bezeichnung) 
                       hclust(LogGeoMean_Liste_mit_Typen_Bezeichnung,method="average"),
                     cexCol=0.5,cexRow=1, 
                     margins=c(6,4), # white space
                     lhei=c(1,6), keysize=1
                     , ColSideColors= my_LogGeoMean_Cluster_Side_Bar
  )
  dev.off()}




# Heatmap -it Dendrogramm der Gene:

{pdf (paste(sep="", "(32) LogGeoMean_Cluster_HeatMap_ohne_", FileTag, ".pdf"),width=8,height=8)
  heatmap<-heatmap.2(as.matrix(LogGeoMean_Liste_mit_Typen_Bezeichnung),
                     main=FileTag,
                     Rowv= F, #as.dendrogram(hc), # Kein Dendrogramm
                     Colv= F,
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(LogGeoMean_Liste_mit_Typen_Bezeichnung) 
                       hclust(LogGeoMean_Liste_mit_Typen_Bezeichnung,method="average"),
                     cexCol=1 ,cexRow=0.5, 
                     margins=c(4,6), # white space
                     lhei=c(1,6), keysize=1
                     , RowSideColors= my_LogGeoMean_Cluster_Side_Bar
  )
  dev.off()}




# **********************************************************************************************
# Begin of EIN Petri-Netz für alle Plasmodien des Datensatzes
# **********************************************************************************************

# Wir brauchen einmal die Mittelwerte der Gene für jeden Cluster
# und die Differenzen für jeden Übergang von einem Cluster zum nächsten

# Dann brauchen wir die Trajektorien jedes einzelnen Plasmodiums um
# daraus ein PN zu erzeugen

# Aus den Differenzen der Expressionswerte kann dann ebenfalls ein PN
# erzeugt werden. 


# Die Übergänge eines Plasmodiums von einem Cluster in den anderen
# werden in eine Matrix von_Cluster x nach_Cluster eingetragen


# Erst wird eine Matrix mit Nullen erzeugt, dann wird sie 
# mit Vektoren aufgefüllt
# 20 = Zahl der Cluster




#***********************************************************************#
# Bestimmung der Übergänge zwischen Clustern                            #
#***********************************************************************#

# Zahl der Zeitpunkte muss im Header korrekt angeben sein (!)
# Zahl der Zeilen in "Trajektorien" 
zeilen <- length(my_Simprof_Cluster_Assignment)

# Zahl der Plasmodien entsprechend
P <- (zeilen / Zeitpunkte)

# Zahl der Cluster
# zdc <- max(as.matrix(my_Simprof_Cluster_Assignment))

zdc <- max(my_Simprof_Cluster_Assignment)

# Leere Matrix Cluster x Cluster erzeugen und mit Nullen füllen   
Transits <- matrix (0,zdc,zdc)



# Directory für die PN Files anlegen
# mainDir <- "~/Desktop/myR"
subDir <- "(96) Petri Net Construction Doku"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))





# Den Bildschirm-Output mitschreiben und abspeichern:
log_filename <- paste(sep="", "(33) Transit_detection_protocol_", FileTag, ".log")
con <- file(log_filename)
sink(con, append=TRUE)
sink(con, append=TRUE, type="message")
# # This will echo all input and not truncate 150+ character lines...
# source("script.R", echo=TRUE, max.deparse.length=10000)

# cID <- ncol(Simprof_Trajctories) # Spalte, in der die ClusterIDs stehen
p <- P-1
for (k in 0:p)
       { # begin of for k to p
         print (c("Plasmodium", k+1))
         s <- ((k*Zeitpunkte)+1)
         print(c("s = zeile ",s))
         # Initialisierung der Spur:
         von <- my_Simprof_Cluster_Assignment [s]     # Spalte 18 später software-mässig definieren
         print (c("von", von))
  
         z <-  Zeitpunkte -1 
         for(i in 1:z)   
            { # begin of for i to z
              q <- ((s+i))
              bis <- my_Simprof_Cluster_Assignment [q]           
              print (c("bis", bis))
              if (von!=bis)   
                { # begin of if.....
                  print("Übergang")
                  Transits[von,bis] <- (Transits[von,bis] + 1)
                 } # end of if.....
              von <- bis
              print(c("von ", von))
              } # end of for i to z
          } # end of for k to p

Transits

# Wird später für die Berechnung der Genexpressions-Unterschiede 
# benötigt, die bei jedem Übergang auftreten
AllTransits <- Transits

# Restore output to console
sink() 
sink(type="message")

# And look at the log...
cat(readLines(log_filename), sep="\n")

filename <- paste(sep="", "(34) Transits_Matrix_", FileTag, ".txt")
write.table(Transits, filename, sep="\t")


# End of Übergänge_bestimmen
#***********************************************************************#


# Nun wird aus der Matrix der Übergänge zwischen Clustern eine Tabelle 
# erzeugt, aus der das Pascal-Programm dann das ANDL-File erzeugt

# PNT = Petri net transitions mit folgenden Einträgen:
# -------------------------------------------------------------------------
# Snoopy FileName als erster Eintrag
# Zahl der Cluster = Petri net places als zweiter Eintrag. 
# Zahl der Stochastischen Transitionen als dritter Eintrag. Dann periodisch
# von
# nach
# Rate

# von
# nach
# Rate
# etc...

# Schliesslich die Initiationsereignisse mit Immediate Transitions 

# --------------------------------------------------------------------------

# Zunächst werden die Übergänge zwischen denClustern als Stochastische 
# Transitionen in einen Vektor geschrieben. Das erlaubt
# (1) die Bestimmung der Zahl der stochastischen Transitionen und
# (2) das Netz in der erforderlichen Reihenfolge zusammenzusetzen

STS <- integer()  #  Init STS, Stochastic Transition set
for (von in 1:zdc) 
    { for (nach in 1:zdc) 
        { if (Transits[von,nach]!=0)  
            { STS <- c (STS, von, nach, Transits[von,nach]) 
            } # end of if...
         } # end of for nach...
     } # end of for von...

# Zahl der Stochastic Transitions ermitteln
Stochastische_Transitionen <- ( length(STS) ) / 3

# Petri-Netz aus den einzelnen Vektoren zusammensetzen:
# zdc, Zahl der Cluster = Petri net places als erster Eintrag
PNT <- c(zdc, Stochastische_Transitionen, STS, Start_ins_Netz)

# # und als File abspeichern:
# filename <- paste(sep="", "(17) Petri_Net_Table_", FileTag, ".txt")
# write.table(PNT,filename, eol = "\r", row.names=F, col.names=F)
# eol = "\r" ist "nur" Carriage Return am Zeilenende? Kein zusätzliches LineFeed
# keine Namen für Zeilen und Spalten ausgeben


# Directory für die PN Files anlegen
# mainDir <- "~/Desktop/myR"
# mainDir <-getwd()
setwd(mainDir)
subDir <- "(95) Petri Nets"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))



# Besser ist es, den FileName als erste Zeile zu übergeben, 
# denn der erscheint dann als Name des Snoopy Files:

# Filenamen erzeugen:
filename <- paste(sep="", "PN_", FileTag, ".txt")
Snoopy_File_Name <- paste0("PN_", FileTag)

# PN Vektor in character umwandeln und den File Namen als erstes Element einfügen
PNchar <- as.character(PNT)
withName <- c(Snoopy_File_Name, PNchar)

# abspeichern, nur mit carriage return am Zeilenende:
fileConn<-file(filename)
writeLines(withName, sep = "\r", fileConn)
close(fileConn)


# Directory zurücksetzen
# setwd("~/Desktop/myR")
setwd(mainDir)

# **********************************************************************************************
# END of EIN Petri-Netz für alle Plasmodien des Datensatzes                                  #
# **********************************************************************************************




# **********************************************************************************************
# Begin of Separate Petri-Netze für die Plasmodien der jeweiligen Klasse erzeugen              #
# **********************************************************************************************
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #



for (Klasse in 1:Plasmodien_Klassen) 
  
            {  # begin of Petri-Netze erzeugen...geht von Zeile 1185-1400......   


  
mydf <- as.data.frame(Trajektorien_mit_Klassen)
Block_der_Klasse <- subset(mydf, P==Klasse)
die_Cluster_der_Klasse <- Block_der_Klasse[,2:(Zeitpunkte+1)]
myMatrix <-as.matrix (die_Cluster_der_Klasse)

Cluster_des_Blocks <- as.integer (t (myMatrix) ) # zeilenweise in Vektor umwandeln



######## Beginn der Initialisierung ########################################################

# Hier werden die Initiations-Events für das später zu erzeugende Petri-Netz ermittelt:
# -------------------------------------------------------------------------------------
# Hier werden alle Cluster, dem ein Plasmodium zum Zeitpunkt t=0h angehört 
# dem Integer Vektor nullh zugeordnet 

nullh <- myMatrix [,1] # also die _0h Werte
nullh



# und deren Häufigkeitsverteilung bestimmt
nullh.freq <- table(nullh)

# Vektor mit einer Zahl von Nullen erzeugen, die der Zahl der Einträge (=Intiations-Events) entspricht
Null_Cluster     <- rep(0,nrow(nullh.freq))  

# Cluster Cx zum Zeitpunkt t=0
Cluster_Nr_at_0h <- as.integer(rownames(nullh.freq))  

# Häufigkeit des Übergangs C0 nach Cx
Frequenz         <- as.vector(nullh.freq)  

# Diese drei Vektoren zu einer Matrix vereinigen:
PN_Init_Matrix <- t(cbind (Null_Cluster, Cluster_Nr_at_0h, Frequenz))
PN_Init_Matrix

Start_ins_Netz <- as.vector(PN_Init_Matrix) 

# In einen Vektor umwandeln entsprechend dem Format des Petri-Netz Files
# Cvon, Cnach, Rate etc.:



# Directory für die PN Files anlegen
# mainDir <- "~/Desktop/myR"
subDir <- "(96) Petri Net Construction Doku"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))


# Nur zur Dokumentation als Tabelle speichern:
filename <- paste0("(16-", Klasse, ") Petri_Net_Init_Doku_", FileTag, ".txt")
write.table(PN_Init_Matrix, filename, sep="\t", col.names = F)
# -------------------------------------------------------------------------------------

######## Ende der Initialisierung ########################################################




#***********************************************************************#
# Bestimmung der Übergänge zwischen Clustern (PN der jeweiligen Klasse) #
#***********************************************************************#


# Zahl der Zeitpunkte muss im Header korrekt angeben sein (!)
# Zahl der Zeilen in "Trajektorien" 
zeilen <- length(Cluster_des_Blocks)

# Zahl der Plasmodien entsprechend
P <- (zeilen / Zeitpunkte)

# # Zahl der Cluster muss für jeden Block ermittelt werden:
# Cluster_im_Block <- as.vector (myMatrix)
# zdc <- length (unique(Cluster_im_Block))


# Leere Matrix Cluster x Cluster erzeugen und mit Nullen füllen 
Transits <- matrix (0,zdc,zdc)


# Den Bildschirm-Output mitschreiben und abspeichern:
log_filename <- paste0("(22-", Klasse, ") Transit_detection_protocol_", FileTag, ".log")
con <- file(log_filename)
sink(con, append=TRUE)
sink(con, append=TRUE, type="message")
# # This will echo all input and not truncate 150+ character lines...
# source("script.R", echo=TRUE, max.deparse.length=10000)

# cID <- ncol(Simprof_Trajctories) # Spalte, in der die ClusterIDs stehen
p <- P-1
for (k in 0:p)
      { print (c("Plasmodium", k+1))
        s <- ((k*Zeitpunkte)+1)
        print(c("s = zeile ",s))
        # Initialisierung der Spur:
        von <- Cluster_des_Blocks [s]     # Spalte 18 später software-mässig definieren
        print (c("von", von))
  
        z <-  Zeitpunkte -1 
        for(i in 1:z)   
            { # begin of for i to z
              q <- ((s+i))
              bis <- Cluster_des_Blocks [q]           
              print (c("bis", bis))
              if (von!=bis)   
                 { print("Übergang")
                   Transits[von,bis] <- (Transits[von,bis] + 1)
                 } # end of if.....
              von <- bis
              print(c("von ", von))
    
              } # end of for i to z
         } # end of for k to p

Transits

# Restore output to console
sink() 
sink(type="message")

# And look at the log...
cat(readLines(log_filename), sep="\n")

filename <- paste0("(23-", Klasse, ") Transits_Matrix_", FileTag, ".txt")
write.table(Transits, filename, sep="\t")


# End of Übergänge_bestimmen  (für das PN der aktuellen Klasse)
#***********************************************************************#


# Nun wird aus der Matrix der Übergänge zwischen Clustern eine Tabelle 
# erzeugt, aus der das Pascal-Programm dann das ANDL-File erzeugt

# PNT = Petri net transitions mit folgenden Einträgen:
# -------------------------------------------------------------------------
# Snoopy FileName als erster Eintrag
# Zahl der Cluster = Petri net places als zweiter Eintrag. 
# Zahl der Stochastischen Transitionen als dritter Eintrag. Dann periodisch
# von
# nach
# Rate

# von
# nach
# Rate
# etc...

# Schliesslich die Initiationsereignisse mit Immediate Transitions 

# --------------------------------------------------------------------------

# Zunächst werden die Übergänge zwischen denClustern als Stochastische 
# Transitionen in einen Vektor geschrieben. Das erlaubt
# (1) die Bestimmung der Zahl der stochastischen Transitionen und
# (2) das Netz in der erforderlichen Reihenfolge zusammenzusetzen

STS <- integer()  #  Init STS, Stochastic Transition set
for (von in 1:zdc) 
       { for (nach in 1:zdc) 
           { 
              if (Transits[von,nach]!=0)  
                { 
                   STS <- c (STS, von, nach, Transits[von,nach]) 
               } # end of if...
           } # end of for nach...
        } # end of for von...

# Zahl der Stochastic Transitions ermitteln
Stochastische_Transitionen <- ( length(STS) ) / 3

# Petri-Netz aus den einzelnen Vektoren zusammensetzen:
# zdc, Zahl der Cluster = Petri net places als erster Eintrag
PNT <- c(zdc, Stochastische_Transitionen, STS, Start_ins_Netz)

# # # und als File abspeichern:
# filename <- paste0("(17-", Klasse,") Petri_Net_Table_", FileTag, ".txt")
# write.table(PNT,filename, eol = "\r", row.names=F, col.names=F)
# # eol = "\r" ist "nur" Carriage Return am Zeilenende? Kein zusätzliches LineFeed
# # keine Namen für Zeilen und Spalten ausgeben


# # Directory für die PN Files anlegen
# mainDir <- "~/Desktop/myR"
subDir <- "(95) Petri Nets"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))


# Besser ist es, den FileName als erste Zeile zu übergeben, 
# denn der erscheint dann als Name des Snoopy Files:

# Filenamen erzeugen:
filename <- paste0("PN", Klasse, "_", FileTag, ".txt")
Snoopy_File_Name <- paste0("PN", Klasse, "_", FileTag)

# PN Vektor in character umwandeln und den File Namen als erstes Element einfügen
PNchar <- as.character(PNT)
withName <- c(Snoopy_File_Name, PNchar)

# abspeichern, nur mit carriage return am Zeilenende:
fileConn<-file(filename)
writeLines(withName, sep = "\r", fileConn)
close(fileConn)




# Gesamten Programmlauf bis hier speichern
# save.image("~/Desktop/myR/(52) Until_PNs.RData")
save.image(paste0(mainDir, "/(52) Until_PNs_", FileTag, ".RData"))

# ggf wieder laden mit:
# load("~/Desktop/myR/(52) Until_PNs.RData")

# Directory zurücksetzen
# setwd("~/Desktop/myR")
setwd(mainDir)

                                   } # end of Petri-Netze erzeugen.....

# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# **********************************************************************************************
# END of Separate Petri-Netze für die Plasmodien der jeweiligen Klasse erzeugen                #
# **********************************************************************************************





# Jetzt werden die Änderungen der Genexpressionswerte für 
# alle Übergänge zwischen Clustern berechnet
# Dazu wird die Matrix Transits des Gesamt-Petri-Netzes 
# (AllTransits)noch einmal ausgewertet
# 

mRNAs  <- 0 # Initialisierung, die erste Zeile wird mit Nullen gefüllt
xfolds <- 0 # Initialisierung, die erste Zeile wird mit Nullen gefüllt
xfolds_only <- 0 # Initialisierung, die erste Zeile wird mit Nullen gefüllt
VonNach <- rbind(c(0,0)) # Wird als Matrix definiert und initialisiert
VonNachTransits <- rbind(c(0,0,0))

for (von in 1:zdc) 
      { for (nach in 1:zdc) 
        { if (AllTransits[von,nach]!=0)  
             { 
                FromCl <- LogGeoMean_Liste[von,]
                ToCl   <- LogGeoMean_Liste[nach,]
  
                Delta <- LogGeoMean_Liste[von,] - LogGeoMean_Liste[nach,]
                xfold <- LogGeoMean_Liste[nach,] / LogGeoMean_Liste[von,]
                INVxfold <- 1/xfold
  
                Delta[1]     <- 0
                xfold[1]     <- 0
                INVxfold [1] <- 0
  
                VonNach <- rbind(VonNach, c (von, nach))
                VonNachTransits <- rbind(VonNachTransits, c (von, nach, AllTransits[von,nach]))
  
                mRNAs <- rbind(mRNAs,  FromCl, ToCl, Delta, xfold, INVxfold)
                xfolds <-rbind(xfolds, FromCl, ToCl, xfold)
                xfolds_only <-rbind(xfolds_only, xfold)
              
              } # end of if...
           } # end of for nach...
         } # end of for von...


mRNAs  <- mRNAs [-1,] # Erste leere Zeile wieder entfernen
xfolds <- xfolds[-1,] # Erste leere Zeile wieder entfernen
xfolds_only <- xfolds_only[-1, ] # Erste leere Zeile wieder entfernen
VonNach <- VonNach[-1, ] # Erste leere Zeile wieder entfernen
VonNachTransits <- VonNachTransits [-1,] # Erste leere Zeile wieder entfernen
colnames(VonNachTransits) <- c("von", "nach", "Transits")

# Hinzufügen der Cluster-IDs und Transits
xfolds_mit_ClusterIDs <- cbind(VonNachTransits, xfolds_only)


# tra Verfünffachen, wg. 5 Zeilen pro Übergang
x <- integer()
tra <- VonNachTransits [,3]
lst <- as.list(tra)
Trans <- unlist(lapply(lst, function(x) c (0,0,x,x,x) ))

mRNAs_mit_Transits <- cbind(Trans, mRNAs)


# bzw. nur verdreifachen
Trans <- unlist(lapply(lst, function(x) c (0,0,x) ))
xfolds_mit_Transits <- cbind (Trans, xfolds)


Gesamt_Zahl_der_Transitionen <- nrow (VonNachTransits)  # Für Doku-File


# Dann die Files ausgeben:

filename <- paste(sep="", "(35) Transits_", FileTag, ".txt")
write.table(VonNachTransits,filename, row.names=F, col.names=TRUE)
# keine Namen für Zeilen und Spalten ausgeben


# Sortiert nach Häufigkeit der Übergänge ausgeben:
filename <- paste(sep="", "(36) Transits_sorted_", FileTag, ".txt")

Transits_Sorted <- VonNachTransits[order(VonNachTransits[,3], 
                                         rev(VonNachTransits[,1]), 
                                         rev(VonNachTransits[,2]), decreasing = T),]

write.table(Transits_Sorted,filename, row.names=F, col.names=TRUE)

# als Excel:
filename <- paste(sep="", "(37) Transits_sorted_", FileTag, ".xlsx")
# write.xlsx (Transits_Sorted, filename)
write.xlsx (Transits_Sorted, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)


# Häufigkeit der Übergänge
filename <- paste(sep="", "(38) Transits_Häufigkeit_", FileTag, ".xlsx")
Transit_Distribution <- as.matrix(table(VonNachTransits[,3]))
Number_of_Transits <- as.numeric(row.names(Transit_Distribution))
Transit_Distribution <- cbind(Number_of_Transits, Transit_Distribution)
colnames(Transit_Distribution) <-  c("Transits", "Freq")

write.xlsx (Transit_Distribution, filename)


# von, nach, Delta, xfold, INVxfold:
filename <- paste(sep="", "(39) Full_Changes_", FileTag, ".xlsx")
mydf <- as.data.frame (mRNAs_mit_Transits)
options(warn=-1) # warnings off
# write.xlsx (mydf, row.names=TRUE, col.names=TRUE, filename)
write.xlsx (mydf, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)
options(warn=0) # warnings on


# von, nach, x-folds:
filename <- paste0("(40) xfolds_", FileTag, ".xlsx")
options(warn=-1) # warnings off
mydf <- as.data.frame(xfolds_mit_Transits)
# write.xlsx(mydf, filename)
write.xlsx (mydf, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)
options(warn=0) # warnings on


# nur xfolds:
filename <- paste(sep="", "(41) xfolds_only_", FileTag, ".xlsx")
# write.xlsx (xfolds_mit_ClusterIDs, filename)
write.xlsx (xfolds_mit_ClusterIDs, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)



# x-folds als Heatmap ausgeben:

myRowNames <- paste(sep=" ", as.character(xfolds_mit_ClusterIDs [,1]),
                             as.character(xfolds_mit_ClusterIDs [,2]))

xfolds_for_heatmap <- xfolds_mit_ClusterIDs [,5:(length (Genes_analysed)+4)]
rownames (xfolds_for_heatmap) <- myRowNames
log_xfolds_for_heatmap <- log10(xfolds_for_heatmap)


# Heatmap der x-folds:

{pdf (paste(sep="", "(42) x-folds_HeatMap_", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(log_xfolds_for_heatmap),
                     main=FileTag,
                     Rowv= F, #as.dendrogram(hc), # Keine Dendrogramme
                     Colv= F,
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(log_xfolds_for_heatmap) 
                       hclust(log_xfolds_for_heatmap,method="average"),
                     cexCol=1 ,cexRow=0.7, 
                     margins=c(6,4), # white space
                     lhei=c(1,6), keysize=1
                     #, RowSideColors= myClusterSideBar
  )
  dev.off()}





# nun die Matrix nach der Spalte "nach" sortieren:
Sortiert_nach_nach <- xfolds_mit_ClusterIDs[order(xfolds_mit_ClusterIDs[,2]),]


# x-folds als Heatmap ausgeben, diesmal nach "nach" sortiert:

myRowNames <- paste(sep=" ", as.character(Sortiert_nach_nach [,1]),
                    as.character(Sortiert_nach_nach [,2]))

xfolds_for_heatmap <- Sortiert_nach_nach [,5:(length (Genes_analysed)+4)]
rownames (xfolds_for_heatmap) <- myRowNames
log_xfolds_for_heatmap <- log10(xfolds_for_heatmap)


# Heatmap der x-folds, diesmal nach "nach" sortiert:

{pdf (paste(sep="", "(43) x-folds_HeatMap_nach", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(log_xfolds_for_heatmap),
                     main=FileTag,
                     Rowv= F, #as.dendrogram(hc), # Keine Dendrogramme
                     Colv= F,
                     scale="none",
                     col=my_palette,breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(log_xfolds_for_heatmap) 
                       hclust(log_xfolds_for_heatmap,method="average"),
                     cexCol=1 ,cexRow=0.7, 
                     margins=c(6,4), # white space
                     lhei=c(1,6), keysize=1
                     #, RowSideColors= myClusterSideBar
  )
  dev.off()}




# Erzeugen des Doku Files, in dem die verwendeten 
# und erzeugten Dateien per Zeitstempel dokumentiert werden:

InputDateiName <- Datenfile

Sys.setenv(TZ="Europe/Berlin")  # Setzt die Zeitzone
Zeitstempel <- as.character(Sys.time())

Source_Code_File_Name <- rstudioapi::getActiveDocumentContext()$path

KlassenNr <- c(1:length(Klassenzimmer_Gesamter_Datensatz))

if (Take_Subset)
   { Subsetting_Message <- c("Nur Teilauswertung von",
                           paste0("Klasse Nr             : ", paste(Take_Klasse,collapse=", ")),
                           paste0("Plasmodien pro Klasse : ", paste(Sub_Klassenzimmer,collapse=", ")))
   } else 
   { Subsetting_Message <- "alle ausgewertet"}


fileConn<-file(paste (sep="","(0) RDoku_", FileTag, ".txt"))
writeLines(c(paste0("Erste Normierung: ", Normierung, " jeder Probe"),
             "Zweite Normierung: Geometrisches Mittel jedes Gens",
             "",
             "Eingelesen und ausgewertet: ", 
             InputDateiName, 
             "",
             paste0("Genes_analysed  : ", paste(Genes_analysed,collapse=", ")),
             "",
             paste0("Number of Genes : ", length(Genes_analysed)),
             "",
             paste0("Zeitpunkte      : ", Zeitpunkte),
             "",
             "Im gesamten Datensatz vorhanden",
             paste0("Plasmodien pro Klasse : ", paste(Klassenzimmer_Gesamter_Datensatz,collapse=", ")),
             paste0("Klasse Nr             : ", paste(KlassenNr,collapse=", ")),
             "",
             Subsetting_Message,
             "",
             # paste0("Mediane der Experimente:"),
             # paste0(Mediane_der_Experimente),
             # "",
             paste0("LogScale                     : ", LogScale),
             paste0("simprof_alpha                : ", simprof_alpha),
             paste0("Simprof_significant_Clusters : ", simprof_significant_Clusters),
             # paste0("hclust_node_height          = ", hclust_node_height),         
             # paste0("hclust_Cluster              = ", hclust_Cluster),
             "",
             paste0("ergibt ein Petri-Netz mit ", simprof_significant_Clusters, " Plätzen"),
             paste0("                      und ", Gesamt_Zahl_der_Transitionen, " Transitionen"),
             "", "",
             "Ergebnisse berechnet und gespeichert am : ",
             Zeitstempel,
             "",
             "mit R_Code Source File Name : ", 
             Source_Code_File_Name ), fileConn)
close(fileConn)





# Gesamten Programmlauf bis hier speichern
# save.image("~/Desktop/myR/(53) Before_MDS.RData")
save.image(paste0(mainDir, "/(53) Before_MDS_", FileTag, ".RData"))

# ggf wieder laden mit:
# load("~/Desktop/myR/(53) Before_MDS.RData")




# =========================================================================
# HIER BEGINNT DIE NEU IMPLEMENTIERTE MDS:
# =========================================================================
# =========================================================================

# # MDS
# d <- dist(t(data_norm))
# fit <- cmdscale(d,eig=TRUE, k=3)
# # fit
# 
# x <- fit$points[,1]
# y <- fit$points[,2]
# z <- fit$points[,3]
# 



# MDS
d <- dist(t(data_norm))

fit <- cmdscale(d,eig=TRUE, k=2)
# fit

x <- fit$points[,1]
y <- fit$points[,2]

# neu "% Variation in the data" ausgeben nach Josh Stramer at YouTube:
mds.var.per <- round (fit$eig  / sum(fit$eig)*100, 1)



# Nur die Namen als Plot in s/w:
filename <- paste0("(44) MDS_Plot_names: ", FileTag, ".pdf")
myTitle <- paste0(filename, "\n var %: ", paste (mds.var.per [mds.var.per!=0], collapse=", "))
# nur alle mds.var.per Werte ausgeben, die nicht 0 sind !

mySubTitle <- paste0("Genes: ", paste(Genes_analysed,collapse=", "))

{pdf(filename,width=10,height=7)
  plot(x,y, 
       main=myTitle, cex.main=0.7, 
       sub=mySubTitle, cex.sub=0.7, 
#      xlab="Coordinate 1", ylab="Coordinate 2",type="n")
       xlab= paste0("Coordinate 1 - ", mds.var.per[1], "%") , 
       ylab= paste0("Coordinate 2 - ", mds.var.per[2], "%"),    type="n")
  text(x, y, labels = row.names(t(data_norm)), cex=.25) 
  dev.off()}


# MDS-Plot einfarbig:
filename <- paste0("(45) MDS_Plot_bw: ", FileTag, ".pdf")
myTitle <- paste0(filename, "\n var %: ", paste (mds.var.per [mds.var.per!=0], collapse=", "))
mySubTitle <- paste0("Genes: ", paste(Genes_analysed,collapse=", "))

Namen_der_Plasmodien <- row.names(t(data_norm))
{pdf(filename,width=10,height=7)
  plot(x,y, 
       main=myTitle, cex.main=0.7, 
       sub=mySubTitle, cex.sub=.4, 
     # xlab="Coordinate 1", ylab="Coordinate 2",type="n")
       xlab= paste0("Coordinate 1 - ", mds.var.per[1], "%") , 
       ylab= paste0("Coordinate 2 - ", mds.var.per[2], "%"),    type="n")
  points(x[1:130],y[1:130], pch=19,cex=.2, col="black")
  text(x[1:130],y[1:130], labels = Namen_der_Plasmodien,pos=1, col="black", cex=.25) 
  dev.off()}





# Sortieren der Plasmodien nach Zeitunkten, damit jeder Zeitpunkt
# eine andere Farbe bekommen kann:

MDS_coordinates <- cbind(x, y)

# Koordinaten als Textfile ausgeben: 
filename <- paste0("(46) MDS_Tab_coordinates_", FileTag, ".txt")
write.table(MDS_coordinates,filename, sep="\t")

Namen_der_Plasmodien <- rownames(MDS_coordinates)

# in die durch Underscore getrennten Terme Spalten, ergibt eine Liste:
Plasmodien_Zeiten_Cluster <- str_split(Namen_der_Plasmodien, "_")

# aus dieser Liste können die einzelnen Spalten als Vekoren
# extrahiert werden (Zeiten befindet sich in der 2. Spalte:
Plasmodien_Namen   <- map_chr(Plasmodien_Zeiten_Cluster, 1)
Zeiten_als_strings <- map_chr(Plasmodien_Zeiten_Cluster, 2)
Cluster            <- map_chr(Plasmodien_Zeiten_Cluster, 3)

# Benennung "h" entfernen und Zeiten in einen Integer Vektor umwandeln:
# t_h <- as.integer(str_replace(Zeiten_als_strings, pattern = "h", ""))

# Neu für Zeitpunkte mit Dezimalstellen:
# Benennung "h" entfernen und Zeiten in einen "numeric" Vektor umwandeln:
t_h <- as.numeric(str_replace(Zeiten_als_strings, pattern = "h", ""))


# und dann die drei Vektoren zu einer Matrix vereinigt....
alsTabelle <- cbind (t_h, Cluster, MDS_coordinates)
# ....die in einen data.frame umgewandelt wird:
als_Sortierbare_Tabelle <- as.data.frame(alsTabelle)

# MDS_t = MDS Coordinaten nach Zeitpunkten aufsteigend sortiert:
MDS_t <- als_Sortierbare_Tabelle[order(t_h, decreasing = F),]


# als Tabelle ausgeben:
filename <- paste0("(47) MDS_Tab_nach_Zeit_", FileTag, ".txt")
write.table(MDS_t, filename, sep="\t")

# MDS-Koordinaten nach Zeitpunkten sortiert in nurmeric umwandeln:
# Achtung: MDS_t[,"x"] ist ein Faktor und kann so nicht geplottet werden. 
# Die Umwandlung in "numeric" ist tricky und erfolgt hier in zwei Stufen:
x <- as.vector(MDS_t[,"x"])
x <- as.numeric(x)

y <- as.vector(MDS_t[,"y"])
y <- as.numeric(y)

Namen_Zeit_Cluster <- rownames(MDS_t)
Plasmodien_pro_Zeitpunkt <- length(x) / Zeitpunkte





# MDS-Plot mit Cluster_Nr:
filename <- paste0("(48) MDS_Plot_mit_clnr_", FileTag, ".pdf")
myTitle <- paste0(filename, "\n var %: ", paste (mds.var.per [mds.var.per!=0], collapse=", "))
mySubTitle <- paste0("Genes: ", paste(Genes_analysed,collapse=", "))

{pdf(filename,width=10,height=7)
  plot(x,y, 
       main=myTitle, cex.main=0.7, 
       sub=mySubTitle, cex.sub=.4,  
     # xlab="Coordinate 1", ylab="Coordinate 2",type="n")
       xlab= paste0("Coordinate 1 - ", mds.var.per[1], "%") , 
       ylab= paste0("Coordinate 2 - ", mds.var.per[2], "%"),    type="n")   
  for (i in 0:Zeitpunkte-1)
    { # begin of for...
       von <- i * Plasmodien_pro_Zeitpunkt + 1
       bis <- (i+1) * Plasmodien_pro_Zeitpunkt
       points(x[von:bis],y[von:bis], pch=19,col=Time_Color [i+1])
       text(x[von:bis],y[von:bis], labels = Namen_Zeit_Cluster [von:bis],pos=1,col=Time_Color [i+1], cex=.6) 
     } # end of for....
  dev.off()}



# MDS-Plot ohne Cluster_Nr:
# in die durch Underscore getrennten Terme Spalten, ergibt eine Liste:
Plasmodien_Zeiten_Cluster <- str_split(rownames(MDS_t), "_")

# aus dieser Liste können die einzelnen Spalten als Vekoren
# extrahiert werden (Zeiten befindet sich in der 2. Spalte:
Plasmodien_Namen   <- map_chr(Plasmodien_Zeiten_Cluster, 1)
Zeiten_als_strings <- map_chr(Plasmodien_Zeiten_Cluster, 2)
Cluster            <- map_chr(Plasmodien_Zeiten_Cluster, 3)

Namen_mit_Zeit <- paste0(Plasmodien_Namen, "_", Zeiten_als_strings)

# Plot erzeugen
filename <- paste0("(49) MDS_Plot_ohne_clnr_", FileTag, ".pdf")
myTitle <- paste0(filename, "\n var %: ", paste (mds.var.per [mds.var.per!=0], collapse=", "))
mySubTitle <- paste0("Genes: ", paste(Genes_analysed,collapse=", "))

{pdf(filename,width=10,height=7)
  plot(x,y, main=myTitle, cex.main=0.7, 
            sub=mySubTitle, cex.sub=.4, 
          # xlab="Coordinate 1", ylab="Coordinate 2",type="n")
            xlab= paste0("Coordinate 1 - ", mds.var.per[1], "%") , 
            ylab= paste0("Coordinate 2 - ", mds.var.per[2], "%"),    type="n")
  for (i in 0:Zeitpunkte-1)
  { # begin of for...
    von <- i * Plasmodien_pro_Zeitpunkt + 1
    bis <- (i+1) * Plasmodien_pro_Zeitpunkt
    points(x[von:bis],y[von:bis], pch=19,col=Time_Color [i+1])
    text(x[von:bis],y[von:bis], labels = Namen_mit_Zeit [von:bis],pos=1,col=Time_Color [i+1], cex=.7) 
  } # end of for....
  dev.off()}



# MDS-Plot mit Farben, die den Cluster-Nummern entsprechen:

Cluster_Nummern <- as.integer(str_replace(Cluster, pattern = "c", ""))
Cluster_Zahl <- length(unique(Cluster_Nummern))

# clusterCols <- colorRampPalette(c("dark grey", "violet", "dark blue", "light blue", "green", "orange", "red", "deeppink", "purple"))(Cluster_Zahl)
# clusterCols <- colorRampPalette(c("dark grey",  "green", "orange", "red", "deeppink", "purple"))(Cluster_Zahl)
# clusterCols <- colorRampPalette(c("dark grey",  "green", "orange", "blue",  "red", "deeppink", "purple"))(Cluster_Zahl)
# clusterCols <- colorRampPalette(c("black",  "green", "orange", "red", "deeppink", "purple"))(Cluster_Zahl)
# clusterCols <- colorRampPalette(c("gray35",  "blue", "orange", "red", "deeppink", "purple"))(Cluster_Zahl)
# clusterCols <- colorRampPalette(c("gray35",  "blue", "coral1", "red", "deeppink", "purple"))(Cluster_Zahl)
# clusterCols <- colorRampPalette(c("gray35", "darkorchid2","blue", "coral1", "red", "deeppink", "purple"))(Cluster_Zahl)
# clusterCols <- colorRampPalette(c("black",  "blue", "orange", "red", "purple"))(Cluster_Zahl)
# clusterCols <- colorRampPalette(c("dark grey",  "blue", "red"))(Cluster_Zahl)
# clusterCols <- rainbow(Cluster_Zahl)

clusterCols <- colorRampPalette(c("gray35", "blue", "coral2", "red", "deeppink", "purple"))(Cluster_Zahl)
Cluster_Color_Pattern <- clusterCols[Cluster_Nummern]


# MDS-Plot mit Cluster_Nr:
Datenpunkte <- length(Cluster_Nummern)

filename <- paste0("(50) MDS_Plot_Cluster_colors_", FileTag, ".pdf")
myTitle <- paste0(filename, "\n var %: ", paste (mds.var.per [mds.var.per!=0], collapse=", "))
mySubTitle <- paste0("Genes: ", paste(Genes_analysed,collapse=", "))

{pdf(filename,width=10,height=7)
  plot(x,y, 
       main=myTitle, cex.main=0.7, 
       sub=mySubTitle, cex.sub=.4, 
     # xlab="Coordinate 1", ylab="Coordinate 2",type="n")
       xlab= paste0("Coordinate 1 - ", mds.var.per[1], "%") , 
       ylab= paste0("Coordinate 2 - ", mds.var.per[2], "%"),    type="n")  
       for (i in 1:Datenpunkte)
          { # begin of for...
            # von <- i * Plasmodien_pro_Zeitpunkt + 1
            # bis <- (i+1) * Plasmodien_pro_Zeitpunkt
            points(x[i],y[i], pch=19,col=Cluster_Color_Pattern [i])
            text(x[i],y[i], labels = Namen_Zeit_Cluster [i],pos=1,col=Cluster_Color_Pattern [i], cex=.6) 
           } # end of for....
       dev.off()}




# ***********************************************************************
# Nun werden die Trajektorien geplottet:
# ***********************************************************************

# MDS-Koordinaten müssen neu erzeugt werden, da sie umsortiert wurden
d <- dist(t(data_norm))
fit <- cmdscale(d,eig=TRUE, k=2)
# fit

x <- fit$points[,1]
y <- fit$points[,2]

# neu "% Variation in the data" ausgeben nach Josh Stramer at YouTube:
mds.var.per <- round (fit$eig  / sum(fit$eig)*100, 1)



MDS_coordinates <- cbind(x, y)
Namen_der_Plasmodien <- rownames(MDS_coordinates)



# Alle Trajektorien in einen Plot funktioniert:

# Time_Color <- c ("black", "blue", "green", "orange", "red")
# Stattedssen oben definierte Time_Color (erweitert für 21 Zeitpunkte) verwenden!
Zahl_der_Plasmodien <- nrow(Simprof_Trajctories) / Zeitpunkte

filename <- paste0("(51) MDS_Trajectories_", FileTag, ".pdf")
myTitle <- paste0(filename, "\n var %: ", paste (mds.var.per [mds.var.per!=0], collapse=", "))
mySubTitle <- paste0("Genes: ", paste(Genes_analysed,collapse=", "))

{pdf(filename,width=10,height=7)
  plot(x,y, 
       main=myTitle, cex.main=0.7, 
       sub=mySubTitle, cex.sub=.4, 
     # xlab="Coordinate 1", ylab="Coordinate 2",type="n")
       xlab= paste0("Coordinate 1 - ", mds.var.per[1], "%") , 
       ylab= paste0("Coordinate 2 - ", mds.var.per[2], "%"),    type="n")  
       for (P in 1:Zahl_der_Plasmodien)
          { # begin of for P.... to Zahl_der_Plasmodien
            von <- ((P-1) * Zeitpunkte) + 1
            bis <- P * Zeitpunkte
            PunktFarbe <- 0
            for (i in von:bis)
                { # begin of for i.....to Zeitpunkte
                  PunktFarbe <- PunktFarbe + 1
                  points(x[i],y[i], pch=19,col=Time_Color [PunktFarbe])
                  lines(x[von:bis],y[von:bis], pch=19,col="black")
                  text(x[i],y[i], labels = Namen_der_Plasmodien [i],pos=1,col="black", cex=.6) 
                } # end of for i.... to Zeitpunkte
            } # end of for P.... to Zahl_der_Plasmodien
   dev.off()}





# Directory für die Plots der Trajektorien anlegen
# mainDir <- "~/Desktop/myR"
subDir <- "(90) Trajektorien"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))


# Jede Trajektorie in einen separaten Plot:

# Time_Color <- c ("black", "blue", "green", "orange", "red")
# Stattedssen oben definierte Time_Color (erweitert für 21 Zeitpunkte) verwenden!
Zahl_der_Plasmodien <- nrow(Simprof_Trajctories) / Zeitpunkte

  for (P in 1:Zahl_der_Plasmodien)
       { # begin of for t.... to Zahl_der_Plasmodien
     
          TrackAsTitle <- paste0(FileTag, "_Track_", P)
          Trackname <- paste0("Track_", P, ".pdf")
          von <- ((P-1) * Zeitpunkte) + 1
          bis <- P * Zeitpunkte
          PunktFarbe <- 0
          
         {pdf(Trackname,width=10,height=7)
              plot(x,y, 
              main=TrackAsTitle, cex.main=0.7, 
              sub=mySubTitle, cex.sub=.4, 
              xlab="Coordinate 1", ylab="Coordinate 2",type="n")
              
                 for (i in von:bis)
                    { # begin of for i.....to Zeitpunkte
                       PunktFarbe <- PunktFarbe + 1
                       points(x[i],y[i], pch=19,cex=1.5, col=Time_Color [PunktFarbe])
                       lines(x[von:bis],y[von:bis], pch=19,col="black")
                       text(x[i],y[i], labels = Namen_der_Plasmodien [i],pos=1,col="black", cex=.6) 
                     } # end of for i.... to Zeitpunkte
           dev.off()}  
        } # end of for t.... to Zahl_der_Plasmodien
  
  
  

# Alle Trajektorien in einen gemeinsamen Plot:

filename <- paste0("Collage_", FileTag, ".pdf")

# {pdf(filename, width = 40, height = 40)
{pdf(filename, width = 40, height = 80)
  # Alle Plots in einem gemeinsamen Panel rows x colums anordnen
  # Format wurde oben via "width = 4, height = 12" eingestellt
  # par(mfrow = c(6, 5))  # "Zeitpunkte" rows and 1 columns, d.h. alle untereinander
  par(mfrow = c(8, 4))  # "Zeitpunkte" rows and 1 columns, d.h. alle untereinander
  { # begin of par...
    for (P in 1:Zahl_der_Plasmodien)
    { # begin of for t.... to Zahl_der_Plasmodien
      Trackname <- paste0(FileTag, "_Track_", P, ".pdf")
      von <- ((P-1) * Zeitpunkte) + 1
      bis <- P * Zeitpunkte
      PunktFarbe <- 0
      
      plot(x,y, 
           main=Trackname, cex.main=0.7, 
           sub=mySubTitle, cex.sub=.4, 
           # lwd=4,
           xlab="Coordinate 1", ylab="Coordinate 2",type="n")
      # axis(side=1, lwd=4)
      # axis(side=2, lwd=4)
      for (i in von:bis)
      { # begin of for i.....to Zeitpunkte
        PunktFarbe <- PunktFarbe + 1
        points(x[i],y[i], pch=19,cex=4, col=Time_Color [PunktFarbe])
        lines(x[von:bis],y[von:bis], pch=19, lwd=2, col="black") # lwd = Liniendicke
        text(x[i],y[i], labels = Namen_der_Plasmodien [i],pos=4,col="black", cex=2) 
      } # end of for i.... to Zeitpunkte
    } # end of for P.... to Zahl_der_Plasmodien
  } # end of par.....
  dev.off()}





# Nun werden die MDS-Kooridinaten nach Clusterzugehörigkeit geplottet:
# ====================================================================
# Directory für die Plots der einzelnen Cluster anlegen
# mainDir <- "~/Desktop/myR"
subDir <- "(91) Cluster"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))



# Im Folgenden sollen die MDS-Koordinaten jedes Clusters in 
# einem separaten pdf dargestellt werden. 
# Die Farbcodierung erfolgt entsprechend der Zeitpunkte:

# Hier werden die MDS-Koordinaten in eine abfragbare Tabelle gepackt, 
# um sie später nach verschiedenen Kriterien zu plotten:
# Die im Plasmodien-Namen codierten Eigenschaften in Vektoren zerlegen:
InSpalten <- str_split(Namen_der_Plasmodien, "_")
Plasmodium <- map_chr(InSpalten, 1)
Zeit       <- map_chr(InSpalten, 2)
Cluster.Nr <- map_chr(InSpalten, 3)

ClusterNr <- str_replace(Cluster.Nr, pattern = "c", "") # die c's entfernen!

SpoQuest <- str_split(Namen_der_Plasmodien, "P")
Sporulation <- map_chr(SpoQuest, 1)  # Sporulation steht in der ersten Spalte

# Daraus eine MDS-Tabelle zum Plotten nach wechselnden Kriterien erzeugen:
Zeit <- rep(Zeitpunkte.h, sum (Klassenzimmer)) # Zeit-Strings mit Dezimalstellen(!)
MDS_with_Details <- cbind(Plasmodium, Sporulation, Zeit, ClusterNr, x, y)

clst.chr <- MDS_with_Details[,4]
clst <- as.integer(clst.chr)
Zahl_der_Cluster <- max(clst)

# # Zahl der Einträge für jeden Cluster durch Erstellen 
# # und Auslesen der Häufigkeitsverteilung ermitteln:
# all_die_Cluster <- MDS_with_Details[,4]
# numbertable <- table(all_die_Cluster) # ergibt die Häufigkeitsverteilung
# # auf die dann zugegriffen wird.
# Einträge_im_Cluster <- 0


Tab_zeilen <- Zahl_der_Plasmodien * Zeitpunkte

for (C in 1:Zahl_der_Cluster) # Für jeden Cluster die Tabelle durchgehen
{ # begin of for t.... to Zahl_der_Plasmodien
                     # Einträge_im_Cluster <- numbertable[as.character(C)]
  Clustname <- paste0("Cluster_", C, ".pdf")
 
  {pdf(Clustname,width=10,height=7)
    plot(x,y, 
         main=Clustname, cex.main=0.7, 
         sub=mySubTitle, cex.sub=.4, 
         xlab="Coordinate 1", ylab="Coordinate 2",type="n")
    
         for (i in 1:Tab_zeilen) # geht für jeden Cluster die ganze Tabelle durch
            { # begin of for i.....to Tab_zeilen
              Zeitpkt <- MDS_with_Details[i,3]
              # if (Zeitpkt == "0h") {Time_Color <- "black"}
              # if (Zeitpkt == "2h") {Time_Color <- "blue"}
              # if (Zeitpkt == "6h") {Time_Color <- "green"}
              # if (Zeitpkt == "8h") {Time_Color <- "orange"}
              # if (Zeitpkt == "11h") {Time_Color <- "red"}
              # 
              if (Zeitpkt == "0.0h") {Time_Color <- "violet"}
              if (Zeitpkt == "0.5h") {Time_Color <- "black"}
              if (Zeitpkt == "1.0h") {Time_Color <- "black"}
              if (Zeitpkt == "1.5h") {Time_Color <- "black"}
              if (Zeitpkt == "2.0h") {Time_Color <- "black"}
              if (Zeitpkt == "2.5h") {Time_Color <- "blue"}
              if (Zeitpkt == "3.0h") {Time_Color <- "blue"}
              if (Zeitpkt == "3.5h") {Time_Color <- "blue"}
              if (Zeitpkt == "4.0h") {Time_Color <- "blue"}
              if (Zeitpkt == "4.5h") {Time_Color <- "green"}
              if (Zeitpkt == "5.0h") {Time_Color <- "green"}
              if (Zeitpkt == "5.5h") {Time_Color <- "green"}
              if (Zeitpkt == "6.0h") {Time_Color <- "green"}
              if (Zeitpkt == "6.5h") {Time_Color <- "orange"}
              if (Zeitpkt == "7.0h") {Time_Color <- "orange"}
              if (Zeitpkt == "7.5h") {Time_Color <- "orange"}
              if (Zeitpkt == "8.0h") {Time_Color <- "orange"}
              if (Zeitpkt == "8.5h") {Time_Color <- "red"}
              if (Zeitpkt == "9.0h") {Time_Color <- "red"}
              if (Zeitpkt == "9.5h") {Time_Color <- "red"}
              if (Zeitpkt == "10.0h") {Time_Color <- "red"}
              
              
              clst.chr <- MDS_with_Details[i,4]
              clst <- as.integer(clst.chr)
      
              if (clst==C) { # begin
                    points(x[i],y[i], pch=19,cex=1.5, col=Time_Color)
                    text(x[i],y[i], labels = MDS_with_Details[i,1],pos=1,col="black", cex=.6)
                            } # end of if....
            } # end of for i.... to Zeitpunkte
    dev.off()}  
} # end of for t.... to Zahl_der_Plasmodien






# Nun werden die MDS-Kooridinaten nach Zeitpunkten geplottet:
# Für jeden Zeitpunkt einen Plot
# ====================================================================
# Directory für die Plots der einzelnen Cluster anlegen
# mainDir <- "~/Desktop/myR"
subDir <- "(92) Zeitpunkte"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))


Tab_zeilen <- Zahl_der_Plasmodien * Zeitpunkte

for (z in 1:Zeitpunkte) # Für jeden Cluster die Tabelle durchgehen
{ # begin of for t.... to Zahl_der_Plasmodien
  ZeitStr <- Zeitpunkte.h[z]
  ZeitName <- paste0("Zeit==", ZeitStr, ".pdf")
  Head_Line <- paste0(FileTag, "_", ZeitName)
  
  {pdf(ZeitName,width=10,height=7)
    plot(x,y, 
         main=Head_Line, cex.main=0.7, 
         sub=mySubTitle, cex.sub=.4, 
         xlab="Coordinate 1", ylab="Coordinate 2",type="n")
    
    for (i in 1:Tab_zeilen) # geht für jeden Zeitpunkt die ganze Tabelle durch
    { # begin of for i.....to Tab_zeilen
      Zeitpkt <- MDS_with_Details[i,3] 
      # if (Zeitpkt == "0h")  {Time_Color <- "black" }
      # if (Zeitpkt == "2h")  {Time_Color <- "blue"  }
      # if (Zeitpkt == "6h")  {Time_Color <- "green" }
      # if (Zeitpkt == "8h")  {Time_Color <- "orange"}
      # if (Zeitpkt == "11h") {Time_Color <- "red"   }
      
      if (Zeitpkt == "0.0h") {Time_Color <- "violet"}
      if (Zeitpkt == "0.5h") {Time_Color <- "black"}
      if (Zeitpkt == "1.0h") {Time_Color <- "black"}
      if (Zeitpkt == "1.5h") {Time_Color <- "black"}
      if (Zeitpkt == "2.0h") {Time_Color <- "black"}
      if (Zeitpkt == "2.5h") {Time_Color <- "blue"}
      if (Zeitpkt == "3.0h") {Time_Color <- "blue"}
      if (Zeitpkt == "3.5h") {Time_Color <- "blue"}
      if (Zeitpkt == "4.0h") {Time_Color <- "blue"}
      if (Zeitpkt == "4.5h") {Time_Color <- "green"}
      if (Zeitpkt == "5.0h") {Time_Color <- "green"}
      if (Zeitpkt == "5.5h") {Time_Color <- "green"}
      if (Zeitpkt == "6.0h") {Time_Color <- "green"}
      if (Zeitpkt == "6.5h") {Time_Color <- "orange"}
      if (Zeitpkt == "7.0h") {Time_Color <- "orange"}
      if (Zeitpkt == "7.5h") {Time_Color <- "orange"}
      if (Zeitpkt == "8.0h") {Time_Color <- "orange"}
      if (Zeitpkt == "8.5h") {Time_Color <- "red"}
      if (Zeitpkt == "9.0h") {Time_Color <- "red"}
      if (Zeitpkt == "9.5h") {Time_Color <- "red"}
      if (Zeitpkt == "10.0h") {Time_Color <- "red"}
      
      
      if (MDS_with_Details[i,3]==Zeitpunkte.h[z])
           { # begin
               points(x[i],y[i], pch=19,cex=1.5, col=Time_Color)
               text(x[i],y[i], labels = MDS_with_Details[i,1],pos=1,col="black", cex=.6)
           } # end of if MDS.......
    } # end of for i.... to Tab_zeilen
    dev.off()}  
} # end of for t.... to Zahl_der_Plasmodien






# Nun werden die MDS-Kooridinaten nach Plasmodien-Klassen geplottet:
# Für jeden Plasmodien-Klasse einen Plot
# ====================================================================
# Directory für die Plots der einzelnen Klassen anlegen

# mainDir <- "~/Desktop/myR"
subDir <- "(93) Plasmodien-Klassen"
dir.create(file.path(mainDir, subDir), showWarnings = FALSE)
setwd(file.path(mainDir, subDir))

# Je 5 Einträge pro Plasmodien-Klasse erzeugen:
lst <- as.list(Plasmodial_Class_Assignment)
Class <- unlist(lapply(lst, function(x) rep (x, Zeitpunkte) ))
MDScl <- cbind(Class, MDS_with_Details)

Tab_zeilen <- Zahl_der_Plasmodien * Zeitpunkte

for (z in 1:Plasmodien_Klassen) # Für jeden Cluster die Tabelle durchgehen
{ # begin of for t.... to Zahl_der_Plasmodien
     PlotName <- paste0("Class_", z, ".pdf")
     Head_Line <- paste0(FileTag, "_", PlotName)
  
  {pdf(PlotName,width=10,height=7)
    plot(x,y, 
         main=Head_Line, cex.main=0.7, 
         sub=mySubTitle, cex.sub=.4, 
         xlab="Coordinate 1", ylab="Coordinate 2",type="n")
    
    for (i in 1:Tab_zeilen) # geht für jeden Zeitpunkt die ganze Tabelle durch
    { # begin of for i.....to Tab_zeilen
      Zeitpkt <- MDScl[i,4] 
      # if (Zeitpkt == "0h")  {Time_Color <- "black" }
      # if (Zeitpkt == "2h")  {Time_Color <- "blue"  }
      # if (Zeitpkt == "6h")  {Time_Color <- "green" }
      # if (Zeitpkt == "8h")  {Time_Color <- "orange"}
      # if (Zeitpkt == "11h") {Time_Color <- "red"   }
      
      if (Zeitpkt == "0.0h") {Time_Color <- "violet"}
      if (Zeitpkt == "0.5h") {Time_Color <- "black"}
      if (Zeitpkt == "1.0h") {Time_Color <- "black"}
      if (Zeitpkt == "1.5h") {Time_Color <- "black"}
      if (Zeitpkt == "2.0h") {Time_Color <- "black"}
      if (Zeitpkt == "2.5h") {Time_Color <- "blue"}
      if (Zeitpkt == "3.0h") {Time_Color <- "blue"}
      if (Zeitpkt == "3.5h") {Time_Color <- "blue"}
      if (Zeitpkt == "4.0h") {Time_Color <- "blue"}
      if (Zeitpkt == "4.5h") {Time_Color <- "green"}
      if (Zeitpkt == "5.0h") {Time_Color <- "green"}
      if (Zeitpkt == "5.5h") {Time_Color <- "green"}
      if (Zeitpkt == "6.0h") {Time_Color <- "green"}
      if (Zeitpkt == "6.5h") {Time_Color <- "orange"}
      if (Zeitpkt == "7.0h") {Time_Color <- "orange"}
      if (Zeitpkt == "7.5h") {Time_Color <- "orange"}
      if (Zeitpkt == "8.0h") {Time_Color <- "orange"}
      if (Zeitpkt == "8.5h") {Time_Color <- "red"}
      if (Zeitpkt == "9.0h") {Time_Color <- "red"}
      if (Zeitpkt == "9.5h") {Time_Color <- "red"}
      if (Zeitpkt == "10.0h") {Time_Color <- "red"}
      
      
      if (MDScl[i,1]==z)
      { # begin
        points(x[i],y[i], pch=19,cex=1.5, col=Time_Color)
        text(x[i],y[i], labels = MDScl[i,2],pos=1,col="black", cex=.6)
      } # end of if MDS.......
    } # end of for i.... to Tab_zeilen
    dev.off()}  
} # end of for z.... to Plasmodien_Klassen

 } # end of if Nur_HeatMaps = FALSE



# Directory zurücksetzen
setwd("~/Desktop/myR")


# Gesamten Programmlauf speichern
save.image(paste0(mainDir, "/", "(54) Complete_Session_", FileTag, ".RData"))

# ggf wieder laden mit:
# load("~/Desktop/myR/(50) Complete_Session.RData")

# load("(53) Before_MDS_Anna_down_Log.RData")
# load("(54) Complete_Session_Anna_down_Log.RData")









