Program GetPN;
{ Written by: Wolfgang Marwan                                   } 
{ Version vom 20-06-2019                                        }
{ Edited for publication 29-09-2020                             }

{ Generates Petri Nets from Petri Net Description Vectors       }
{ This source code is provided as Supplementary Material to:    }
{ Raetzel et al. (2020): Disentangling a Complex Response in    }
{ Cell Reprogramming and Probing the Waddington Landscape by    }
{ Automatic Construction of Petri Nets. Biosystems 189, 104092. }

 


uses
   crt, MacTP;

Const
   MaxNets        = 100;  { Maximale Zahl einzelner Petri-Netze, die }
                          { zu einem Netz vereinigt werden koennen   }
                          
   MainDir        = 'c:\PNets';
   FileDir        = 'c:\PNets\1_PN';

   Petri_Net_Name = 'PN';
   Source_Tag     = '_';       { Source ist das aus R exportierte PN-File }  
   Source_Ext     = '.TXT';
   Andl_Ext       = '.TXT';
   WorkFile_Name  = 'PN.DAT';
   NameSet_Name   = 'NAMES.DAT';
   ReadMe_Name    = 'README.TXT';
   
   PN_Type        = 'gspn';
   Start_of_PN    = '{';
   End_of_PN      = '}';
   Space          = '   ';    {3 LZ}
   LongSpace      = '     ';  {5 LZ}
 
   
 
 
 Type
    TransitionNameSet = record
        von             : integer;
        nach            : integer;
        Events          : integer;
        Transition_Name : string;
        Selected        : boolean;
      end; { of record } 

   
var   
   PN_Source        : Text;
   ANDL_File        : Text;
   WorkFile         : File of Integer;
   Zeile            : String;
   Snoopy_File_Name : string;
   EventDisplay     : boolean; { Transitions-Namen mit Zahl der Events    }
   
   TransitionNames  : file of TransitionNameSet;
   NameSet          : TransitionNameSet;
   
   Choosen_Petri_Nets      : Array [1..MaxNets] of byte;
   Number_of_Selected_Nets : integer;
   
   
   


Procedure Write_ReadMe;
{ Erzeugt ein ReadMe File zur Doku }

var
   ReadMe : Text;
   
Procedure Schreib (Zeile: string);
{ Schreibt die Zeile in die Textdatei }

begin { of Schreib }
    Writeln (ReadMe, Zeile);
end; { of Schreib }


begin { of Write_ReadMe }
     Assign (ReadMe, ReadMe_Name);
     Rewrite (ReadMe);

     Schreib('Readme for GetPN run under DOS'); 
     Schreib(' ');  
     Schreib('The naming of the files used and generated is as follows: ');  
     Schreib('');  
     Schreib('Source files taken by the program:');  
     Schreib('');  
     Schreib('PN_.txt    the Petri net of samples of all classes');  
     Schreib('');  
     Schreib('PN1_.txt   the Petri net of samples of class 1');  
     Schreib('PN2_.txt   the Petri net of samples of class 2');  
     Schreib('PN3_.txt   the Petri net of samples of class 3');  
     Schreib('...');           
     Schreib('...');         
     Schreib('etc.');  
     Schreib('');  
     Schreib('');  
     Schreib('Files generated by the program:');  
     Schreib('');  
     Schreib('PN.TXT     .andl file without initialisation (samples of all classes)');  
     Schreib('');  
     Schreib('PN1.TXT    .andl file without initialisation (samples of class 1)');  
     Schreib('PN2.TXT    .andl file without initialisation (samples of class 2)');  
     Schreib('...');         
     Schreib('...');  
     Schreib('etc.');  
     Schreib('');  
     Schreib('');  
     Schreib('IPN.TXT    .andl file with initialisation (all classes)');  
     Schreib('');
     Schreib('IPN1.TXT   .andl file with initialisation (samples of class 1)');  
     Schreib('IPN2.TXT   .andl file with initialisation (samples of class 2)');  
     Schreib('...');  
     Schreib('...');  
     Schreib('etc.');  
     Schreib('');  
     Schreib('');  
     Schreib('.andl file of the Petri net of all samples, transition names indicate');  
     Schreib('the class in which the transition occurred:');
     Schreib('');  
     Schreib('  PNXT.TXT   without initialisation');  
     Schreib(' IPNXT.TXT      with initialisation');  
     Schreib('');  
     Schreib('');  
     Schreib('');  
     Schreib('.andl files of Petri nets that were combined of the Petri nets');   
     Schreib('of classes of samples as selected by the user:');  
     Schreib('');  
     Schreib('PNA.TXT    PN combined of e.g. PN1, PN3, PN5');  
     Schreib('PNB.TXT    PN combined of e.g. PN2, PN4');  
     Schreib('PNC.TXT    PN combined of e.g. PN1, PN2, PN4, PN5, etc. ');  
     Schreib('...');  
     Schreib('...');  
     Schreib('etc.');  
     Schreib('');  
     Schreib('and accordingly:');
     Schreib('');  
     Schreib(' PNA.TXT   without initialisation');  
     Schreib('IPNA.TXT      with initialisation');  
     Schreib('');  
     Schreib('');  
     Schreib('');  
     Schreib('Important Note: The file extension ".TXT" created ');
     Schreib('in the DOS environment must be converted into ".andl" before');   
     Schreib('it can be imported into Snoopy.');  
     Schreib('');  
     Schreib('For details see: Raetzel et al. (2020): Disentangling a Complex'); 
     Schreib('Response in Cell Reprogramming and Probing the Waddington Landscape ');
     Schreib('by Automatic Construction of Petri Nets. Biosystems 189, 104092.');
     Schreib('');  

     Close (ReadMe);
end;  { of Write_ReadMe }



Procedure PN_Source_Einlesen (nth_net : integer);
    var
       Source_Name : string;
       Netz_Nr     : string;
       x           : integer;
       
    begin { of PN_Source_Einlesen }
       if nth_net = 0 then
            begin { of if }
               Source_Name := ConCat (Petri_Net_Name, Source_Tag, Source_Ext);
            end  { of if }
          else 
            begin { of else }
              Str (nth_net:0, Netz_Nr); 
              Source_Name := ConCat (Petri_Net_Name, Netz_Nr, Source_Tag, Source_Ext);
            end;  { of else }
            
       if not TextDateiVorhanden (Source_Name) then
           begin
              writeln ('Run time error: ');
              writeln ('you have chosen a non-existing file name: ', Source_Name);
              ChDir (MainDir);
              Halt (1);
           end;
       Assign (PN_Source, Source_Name);
       Reset (PN_Source);
       Assign (WorkFile, WorkFile_Name);
       Rewrite (WorkFile);
       Readln (PN_Source, Snoopy_File_Name);
       while not EoF (PN_Source) do
          begin { of while not EOF..}
             Readln (PN_Source, Zeile);
             x := StrToInt (Zeile);
             Write (Workfile, x);
          end;  { of while not EOF..}
       Close (PN_Source);
       Close (WorkFile);   
    end;  { of PN_Source_Einlesen }
   
   
   
Procedure Write_Header (PN_Type : String);

    var
       HeadLine : string;
       
    begin { of Write_Header}
       HeadLine := ConCat (PN_Type, ' [', Snoopy_File_Name, ']');
       Writeln (ANDL_File, HeadLine);
       Writeln (ANDL_File, Start_of_PN);
    end;  { of Write_Header}




Procedure Write_Constants; 
    begin { of Write_Constants }
        Writeln (ANDL_File, 'constants:');
    end;  { of Write_Constants }



Procedure Write_Places (Initialisierung : boolean);

    var
       Places  : integer;
       PlaceNr : integer;
       i       : integer;
       PlaceName : string;
       
    begin { of Write_Places }
       Read (WorkFile, Places);
       Writeln (ANDL_File, 'places:');
       
       if Initialisierung then
           begin { of if Initialisierung }
             { Platz C0, markiert zur Initiation des Netzes }
             Zeile := Concat (Space, 'C0', ' = 1;');
             Writeln (ANDL_File, Zeile);
           end; { of if Initialisierung }
       
       { dann die restlichen Plätze, nicht markiert }
       for i:= 1 to Places do
          begin { of for i..}
             Str (i:0, PlaceName);
             Zeile := Concat (Space, 'C', PlaceName, ' = 0;');
             Writeln (ANDL_File, Zeile);
          end;  { of for i..} 
        Writeln (ANDL_File, '');  
    end;  { of Write_Places } 
    





Procedure Write_Transitions (Initialisierung : boolean);

    var
       Stochastic_Transitions : integer;
       i : integer;

       Transition_Name     : string;
       Arc                 : string;
       RateExpr            : string;
       von, nach, Event    : integer;
       vons, nachs, Events : string;
       
       
    begin { of Write_Transitions }
     { Zahl der Stochastischen Transitionen ermitteln: }
       Read (WorkFile, Stochastic_Transitions);
       
       Writeln (ANDL_File, 'transitions:');
       
     { Schreiben der stochastischen Transitionen: } 
       for i := 1 to Stochastic_Transitions do
           begin { of for i.... }
              PowerBreak ('for i... to StochTrans'); 
              
              Read (WorkFile, von);
              Read (WorkFile, nach);
              Read (WorkFile, Event);
              Str (von:0, vons);
              Str (nach:0, nachs);
              Str (Event:0, Events);
              
              if EventDisplay then
                 Transition_Name := Concat (Space, 'T_', vons, '_', nachs, '_', Events, 'x')
              else
                 Transition_Name := Concat (Space, 'T_', vons, '_', nachs);
        
              Writeln (ANDL_File, Transition_Name);
        
              Zeile := Concat (LongSpace, ':');
              Writeln (ANDL_File, Zeile);
        
              Arc := Concat (LongSpace, ': [C', nachs, ' + 1] & [C', vons, ' - 1]');
              Writeln (ANDL_File, Arc);
        
              RateExpr := Concat (LongSpace, ': MassAction(1)');
              Writeln (ANDL_File, RateExpr);
        
              Zeile := Concat (LongSpace, ';');
              Writeln (ANDL_File, Zeile);           
         end; { of for i.... }
 
 
      if Initialisierung then 
        begin { of if Initialisierung } 
 
          { Schreiben der Immediate Transitions: } 
            Zeile := '';
            Writeln (ANDL_File, Zeile);
            Writeln (ANDL_File, 'immediate:');
            while not EoF (WorkFile) do
               begin { of while not EoF.... }
                  PowerBreak ('at while not... of Write_Transitions');
              
                  Read (WorkFile, von);
                  Read (WorkFile, nach);
                  Read (WorkFile, Event);
                  Str (von:0, vons);
                  Str (nach:0, nachs);
                  Str (Event:0, Events);
              
                  if EventDisplay then
                     Transition_Name := Concat (Space, 'T_', vons, '_', nachs, '_', Events, 'x')
                  else
                     Transition_Name := Concat (Space, 'T_', vons, '_', nachs);
        
                  Writeln (ANDL_File, Transition_Name);
        
                  Zeile := Concat (LongSpace, ':');
                  Writeln (ANDL_File, Zeile);
        
                  Arc := Concat (LongSpace, ': [C', nachs, ' + 1] & [C', vons, ' - 1]');
                  Writeln (ANDL_File, Arc);
        
                  RateExpr := Concat (LongSpace, ': ', Events);
                  Writeln (ANDL_File, RateExpr);
        
                  Zeile := Concat (LongSpace, ';');
                  Writeln (ANDL_File, Zeile);    
             end;  { of while not EoF.... }
        end; { of if Initialisierung }  
    end;  { of Write_Transitions }
    
    
    
Procedure ANDL_Fertigstellen;

    begin { of ANDL_Fertigstellen }
       Zeile := '';
       Writeln (ANDL_File, Zeile);
       Zeile := End_of_PN;
       Writeln (ANDL_File, Zeile);
    end;  { of ANDL_Fertigstellen }
    
    
    
Procedure Make_ANDL (nth_net : integer; Initialisierung : boolean);

    var
       Netz_Nr   : string;
       Andl_Name : string;
       
    begin { of Make_ANDL }
        if nth_net = 0 then
            begin { of if }
               Andl_Name := ConCat (Petri_Net_Name, Andl_Ext);
               if Initialisierung then
                  Andl_Name := ConCat ('i', Andl_Name);
            end  { of if }
          else 
            begin { of else }
              Str (nth_net:0, Netz_Nr); 
              Andl_Name := ConCat (Petri_Net_Name, Netz_Nr, Andl_Ext);
              if Initialisierung then
                  Andl_Name := ConCat ('i', Andl_Name);
            end;  { of else }
        Assign (ANDL_File, Andl_Name);
        Rewrite (ANDL_File);
        Assign (WorkFile, WorkFile_Name);
        Reset  (WorkFile);
        Write_Header (PN_Type);
        Write_Constants;
        Write_Places (Initialisierung);
        Write_Transitions (Initialisierung);
        ANDL_Fertigstellen;
        Close (ANDL_File);
        Close (WorkFile);
        Writeln ('ANDL File ', Andl_Name, ' generated successfully');
    end;  { of Make_ANDL }     
    
    
    
    
 Procedure Init_NameSet; 
{ Schreibt alle Transitionen des globalen Netzes in eine Datei }

var
    dummy            : integer;
    von, nach, Event : integer;
    vonS, nachS      : string;
    Original_Name    : string;
    
    
   begin { of Init_Namensfeld }
      PN_Source_Einlesen (0); { Liest das globale Netz ein } 
      
      Assign (TransitionNames, NameSet_Name);
      Rewrite (TransitionNames);
     
      Assign (WorkFile, WorkFile_Name);
      Reset  (WorkFile);
      Read (WorkFile, dummy); { Places; wird hier nicht verwendet }
      Read (WorkFile, dummy); { Stochastic Transitions ; wird hier nicht verwendet }
      while not EoF (WorkFile) do
          begin { of while not EOF..}
             PowerBreak ('at while not of Init_NameSet'); 
             Read (WorkFile, von);
             Read (WorkFile, nach);
             Read (WorkFile, Event);
             
             Str (von:0, vonS);  
             Str (nach:0, nachS);
             Original_Name := ConCat ('T_', vonS, '_', nachS);
             
             NameSet.von             := von;
             NameSet.nach            := nach;
             NameSet.Events          := Event;  { Alle Events des globalen Netzes }
             NameSet.Transition_Name := Original_Name;
             NameSet.Selected        := FALSE;
             
             Write (TransitionNames, NameSet);
          end;  { of while not EOF..}
    
    Close (WorkFile);
    Close (TransitionNames);
      
   end;  { of Init_Namensfeld }

  
   
   
   
 Procedure Make_Transition_Names (Current_Class : integer; Initialisierung : boolean);
{ Versieht die Transitions-Namen mit Klassen_Nummern, damit      }
{ sie in Snoopy selektiv farbig dargestellt werden können        }
{ dazu werden die Petri-Netze der einzelnen Klassen ausgewertet  }
{ Die Prozedur wertet das Petri_Netz der aktuellen Current_Class }
{ aus, das zuvor in der Datei "Workfile" gespeichert wurde       }

    var
       Stochastic_Transitions : integer;
       
       Old_Name, New_Name     : string;
       Current_Class_String   : string;
       von, nach, Event       : integer;
       Events                 : string;
       dummy                  : integer;
       TransitionFound        : boolean;
       
       
    begin { of Make_Transition_Names }
          { Zahl der Stochastischen Transitionen ermitteln: }   
          
       Assign (TransitionNames, NameSet_Name);
       
       Assign (WorkFile, WorkFile_Name);
       Reset  (WorkFile);
       Read (WorkFile, dummy); { Places; wird hier nicht verwendet }
       Read (WorkFile, dummy); { Stochastic Transitions ; wird hier nicht verwendet }
    
     { Umbenennen der stochastischen Transitionen: } 
       while not EoF (WorkFile) do
           begin { of while not EoF.... }
              PowerBreak ('at while not of Make_Transition_Names'); 
              
              Read (WorkFile, von);
              Read (WorkFile, nach);
              Read (WorkFile, Event);
              
              Str (Event:0, Events);
              
              Reset (TransitionNames);
              TransitionFound := FALSE;
              Str (Current_Class:0, Current_Class_String);
              if Current_Class < 10 then
                   Current_Class_String := ConCat ('K0', Current_Class_String)
                else
                   Current_Class_String := ConCat ('K', Current_Class_String);
                   
              Repeat
                 PowerBreak ('at Repeat of Make_Transition_Names');                 
                 Read (TransitionNames, NameSet);
                 if (NameSet.von = von) and (NameSet.nach = nach) then
                    begin { of if...}
                        if EventDisplay then
                           Current_Class_String := 
                           ConCat ('_', Current_Class_String, '_', Events, 'x') 
                        else
                           Current_Class_String := 
                           ConCat ('_', Current_Class_String);
                           
                        Old_Name := NameSet.Transition_Name;
                        New_Name := ConCat (Old_Name, Current_Class_String); 
                        NameSet.Transition_Name := New_Name;
                        NameSet.Selected       := TRUE;
                        Seek (TransitionNames, (FilePos (TransitionNames)-1));
                        Write (TransitionNames, NameSet); { Schreibt den Datensatz an die      }
                                                          { urspruengliche Stelle in der Datei }
                        TransitionFound := TRUE;    
                   end; { of if...}
              until TransitionFound;
           end; { of while not EoF... }
      Close (WorkFile);
      Close (TransitionNames);
    end;  { of Make_Transition_Names }   
    
    
    
  
Procedure Write_Indexed_Transitions (Initialisierung : boolean);

    var
       Stochastic_Transitions : integer;
       i                      : integer;

       Transition_Name     : string;
       Arc                 : string;
       RateExpr            : string;
       von, nach, Event    : integer;
       vons, nachs, Events : string;
       
       
    begin { of Write_Indexed_Transitions }
     { Zahl der Stochastischen Transitionen ermitteln: }    
       Assign (TransitionNames, NameSet_Name);
       Reset (TransitionNames);     
       Read (WorkFile, Stochastic_Transitions);      
       Writeln (ANDL_File, 'transitions:');
       
     { Schreiben der stochastischen Transitionen: } 
       for i := 1 to Stochastic_Transitions do
           begin { of for i.... }
             PowerBreak ('for i... to StochTrans');                       
             Read (TransitionNames, NameSet);        
             Transition_Name := ConCat (Space, NameSet.Transition_Name);
             Str (NameSet.von:0, vons);
             Str (NameSet.nach:0, nachs);
             
             if NameSet.Selected then { nur wenn sie selektiert wurden: }
                 begin { of if NameSet.Selected ...}
                
              Writeln (ANDL_File, Transition_Name);
        
              Zeile := Concat (LongSpace, ':');
              Writeln (ANDL_File, Zeile);
        
              Arc := Concat (LongSpace, ': [C', nachs, ' + 1] & [C', vons, ' - 1]');
              Writeln (ANDL_File, Arc);
        
              RateExpr := Concat (LongSpace, ': MassAction(1)');
              Writeln (ANDL_File, RateExpr);
        
              Zeile := Concat (LongSpace, ';');
              Writeln (ANDL_File, Zeile);           
              
                  end; { of if NameSet.Selected ...}                       
         end; { of for i.... }
 
 
 
 
      if Initialisierung then 
        begin { of if Initialisierung } 
 
          { Schreiben der Immediate Transitions: } 
            Zeile := '';
            Writeln (ANDL_File, Zeile);
            Writeln (ANDL_File, 'immediate:');
            while not EoF (TransitionNames) do
               begin { of while not EoF.... }
                  PowerBreak ('at while not... of Write_Transitions');             
                  Read (TransitionNames, NameSet);
                  Transition_Name := ConCat (Space, NameSet.Transition_Name);
                  Str (NameSet.von:0, vons);
                  Str (NameSet.nach:0, nachs);
                  Str (NameSet.Events:0, Events);
                  
                  if NameSet.Selected then { nur wenn sie selektiert wurden: }
                      begin { of if NameSet.Selected ...}             
                         
                  Writeln (ANDL_File, Transition_Name);
        
                  Zeile := Concat (LongSpace, ':');
                  Writeln (ANDL_File, Zeile);
        
                  Arc := Concat (LongSpace, ': [C', nachs, ' + 1] & [C', vons, ' - 1]');
                  Writeln (ANDL_File, Arc);
        
                  RateExpr := Concat (LongSpace, ': ', Events);
                  Writeln (ANDL_File, RateExpr);
        
                  Zeile := Concat (LongSpace, ';');
                  Writeln (ANDL_File, Zeile);    
                  
                      end; { of if NameSet.Selected ...}                            
             end;  { of while not EoF.... }
        end; { of if Initialisierung }  
 (* changed here: *)
        Close (TransitionNames);
    end;  { of Write_Indexed_Transitions }
      
    
    
    
    
Procedure Make_Indexed_ANDL (Initialisierung : boolean);
{ Erzeugt ein ANDL-File des globalen Netzes, bei dem die Namen der }
{ Transitionen die Klassen benennen, in denen sie vorgekommen sind }
{ Dient der selektiven Darstellung der Transitionen in Snoopy      }

    var
       Andl_Name : string;
       
    begin { of Make_Indexed_ANDL }      

          PN_Source_Einlesen (0);  { Liest das globale Netz ein }           
          Andl_Name := ConCat (Petri_Net_Name, 'XT', Andl_Ext);               
              if Initialisierung then
                  Andl_Name := ConCat ('i', Andl_Name);

        Assign (ANDL_File, Andl_Name);
        Rewrite (ANDL_File);
        Assign (WorkFile, WorkFile_Name);
        Reset  (WorkFile);
        Write_Header (PN_Type);
        Write_Constants;
        Write_Places (Initialisierung);
        Write_Indexed_Transitions (Initialisierung);    
        ANDL_Fertigstellen;
        Close (ANDL_File);
        Close (WorkFile);
        Writeln ('Indexed ANDL File ', Andl_Name, ' generated successfully');
    end;  { of Make_Indexed_ANDL }   
    
 

Procedure Make_ANDL_from_Selected_Nets (FileNumber : char; 
                                        Initialisierung : boolean);
{ Erzeugt ein ANDL File von ausgewaehlten Petri-Netzen }

var
   Andl_Name : string;

begin { of Make_ANDL_from_Selected_Nets}

        PN_Source_Einlesen (0);  { Liest das globale Netz ein }           
        Andl_Name := ConCat (Petri_Net_Name, FileNumber, Andl_Ext);               
           if Initialisierung then
              Andl_Name := ConCat ('i', Andl_Name);

        Assign (ANDL_File, Andl_Name);
        Rewrite (ANDL_File);
        Assign (WorkFile, WorkFile_Name);
        Reset  (WorkFile);
        Write_Header (PN_Type);
        Write_Constants;
        Write_Places (Initialisierung);
        Write_Indexed_Transitions (Initialisierung);    
        ANDL_Fertigstellen;
        Close (ANDL_File);
        Close (WorkFile);
        Writeln ('ANDL File ', Andl_Name, ' from selected nets generated successfully');
end;  { of Make_ANDL_from_Selected_Nets}





Procedure Choose_Selected_Petri_Nets;
{ Dialog via Bildschirm }

var
   Refresh        : boolean;
   Choice         : string;
   i              : integer;
   AlreadyChosen  : Array [1..MaxNets] of string;
   
   
   
 
Procedure Choose; 
{ Nimmt die Eingabe von Ziffern und verwendet sie nur,   }
{ wenn eine gleiche Eingabe  nicht bereits gemacht wurde }
{ ansonsten wird noch mal nachgefragt....                }

var
   j : integer;
   
 begin { of Choose }
     Choice := ReadFigs (Refresh);   
     for j := 1 to Number_of_Selected_Nets do
        begin { of for j..}  
           if AlreadyChosen [j] = Choice then
              begin { of if AlreadyChosen }
                 writeln;
                 writeln (Choice, ' was already chosen, make another choice!');
                 Choose;
              end; { of if AlreadyChosen }
        end; { of for j..}  
 end; { of Choose }  




begin { of Choose_Selected_Petri_Nets }
   for i := 1 to MaxNets do 
       Choosen_Petri_Nets [i] := 0;
   for i := 1 to MaxNets do
       AlreadyChosen [i] := 'none';  
   Number_of_Selected_Nets := 1; 
   Refresh := FALSE;
   Writeln;
   Writeln ('done    : 0');
   Writeln ('refresh : ESC');
   Writeln;
   i := 0;
   Repeat 
      PowerBreak ('at repeat until of Choose_Selected_Petri_Nets');
      Write   ('Select PN number : ');
      Choose; 
      Write ('        done : 0; refresh : ESC'); Writeln;
      if (Choice <> '0') and (i < MaxNets) and not Refresh then
         begin 
            i := i+1;
            Choosen_Petri_Nets [i] := StrToInt(Choice);
            AlreadyChosen [i] := Choice;
            Number_of_Selected_Nets := i; 
         end; { of if ... }
       if i >= MaxNets then
           begin { of if i >= MaxNets }
               Writeln ('Combination of ', MaxNets, ' Petri Nets at max is currently supported.');
               Writeln ('Hit any key to continue....');
               Hitany; 
           end; { of if i >= MaxNets }   
    until (Choice = '0') or Refresh or (i = MaxNets);
    
    if Refresh then Choose_Selected_Petri_Nets; 
    
end; { of Choose_Selected_Petri_Nets }




 
 
var
   Initialisierung : boolean;
   Klasse, n, i    : integer;
   CombineFiles    : string;
   PN_number       : char;
   Answer          : char;
   
    
 Begin
    ChDir (FileDir);
    ClrScr;
    
    EventDisplay := TRUE;
    
  { Erzeugen des globalen Petri-Netzes }
  { und der Petri-Netze aller Klassen  }
 
    Write ('Make ANDL-Files 0 to: ');
    Readln (n);
    for Klasse := 0 to n do
        begin { of for i... }
           PN_Source_Einlesen (Klasse);
           
           Initialisierung := FALSE;
           Make_ANDL (Klasse, Initialisierung);
           
           Initialisierung := TRUE;
           Make_ANDL (Klasse, Initialisierung);
        end;  { of for i... }
 
 
 
   { Erzeugen der indizierten globalen Petri-Netze: }
     
   { ohne Initialisierung: }
     Initialisierung := FALSE;
     Init_NameSet;       
     for Klasse := 1 to n do   
         begin { of for i.... }  
            PN_Source_Einlesen (Klasse);            
            Make_Transition_Names (Klasse, Initialisierung);
         end;  { of for i.... }
      Make_Indexed_ANDL (Initialisierung);
      
    { mit Initialisierung: }        
      Initialisierung := TRUE;       
      Init_NameSet;  
      for Klasse := 1 to n do   
         begin { of for i.... }  
            PN_Source_Einlesen (Klasse);
            Make_Transition_Names (Klasse, Initialisierung);
         end;  { of for i.... }
      Make_Indexed_ANDL (Initialisierung);   
    
  
  
  { Kombinieren der Petri-Netze einzelner }
  { Klassen zu einem Gesamt-Netz:         }   
     
   PN_number    := 'A';
   CombineFiles := 'X';
   
   Repeat 
        PowerBreak ('at outer repeat');
        Writeln;
        Writeln ('Combine Selected Petri Nets into one File, y/n ?');      
        repeat { bis Taste y oder n gedrueckt wurde }
           PowerBreak ('at inner repeat 2');
           CombineFiles := UpCase (ReadKey);
        until (CombineFiles = 'Y') or (CombineFiles = 'N');  
         
        if CombineFiles = 'Y' then
           begin { of if CombineFiles = 'Y' }
           
              Writeln ('Event Display (recommended) is ON per default');
              Writeln ('switch it OFF for now? y/n');
              Answer := UpCase (ReadKey); 
              if Answer = 'Y' then 
                  EventDisplay := FALSE else
                  EventDisplay := TRUE;
              
              Writeln;
              Writeln ('EventDisplay = ', EventDisplay);
                     
             { zu kombinierende Netze auswaehlen: }  
               Choose_Selected_Petri_Nets;
       
             { Netze ohne Initialisierung erzeugen: }
               Initialisierung := FALSE;
               Init_NameSet;
               for i := 1 to Number_of_Selected_Nets do
                  begin { of for i.... }
                    PN_Source_Einlesen (Choosen_Petri_Nets[i]);           
                    Make_Transition_Names (Choosen_Petri_Nets[i], Initialisierung);
                  end; { of for i.... }
                Make_ANDL_from_Selected_Nets (PN_number, Initialisierung); 

              { Netze mit Initialisierung erzeugen: }    
                Initialisierung := TRUE;
                Init_NameSet;
                for i := 1 to Number_of_Selected_Nets do
                    begin { of for i.... }
                       PN_Source_Einlesen (Choosen_Petri_Nets[i]);           
                       Make_Transition_Names (Choosen_Petri_Nets[i], Initialisierung);
                    end; { of for i.... }
                 Make_ANDL_from_Selected_Nets (PN_number, Initialisierung);   
    
           end;  { of if CombineFiles = 'Y' }
        Inc (PN_number);   
    until CombineFiles = 'N';   
   
    Erase (WorkFile);
    Erase (TransitionNames);
    
    Write_ReadMe;
    
    Writeln ('All ANDL Files generated successfully');
    Writeln ('Hit any key to quit...');
    Hitany;   
    ChDir (MainDir);
 END.  


































