
# Color_Petri_Nets_V6.R
# Written by: Wolfgang Marwan

# Version vom 09-09-2020 

# Umgeschrieben für das Färben von Netzen mit ausgewählten Klassen 
# Plätze werden nach Namen gesucht

# Vorversion erstellt am: 04-07-2019

# Um Petri-Netze konsistent färben zu können, die entweder Transitionen aller Klassen
# oder nur einer Klasse oder einiger Klassen enthalten, wurde das Programm entsprechend
# umgeschrieben, so daß nur die Plätze und Transitionen der angegebenen Klassen 
# zur Bestimmung von Stabilitäten und Häufigkeiten verwendet werden. 
# Diese Klassen müssen explicit in "Set_of_Classes" angegeben werden.


# Zur Bestimmung der Frequenz der Transitionen werden die Textfiles mit den 
# Petri Net Description Vectors eingelesen, diese müssen sich in myR befinden.
# Die Häufigkeit der Cluster wird  aus der Trajektorien-Matrix bestimmt

# Das Programm besteht aus zwei größeren Teilen
# In Teil_1 werden die Häufigkeiten und Stabilitäten bestimmt
# In Teil_2 werden die Farben erzeugt und die Petri-Netze eingefärbt





# Voreinstellungen & Settings

rm(list=ls()) # Clear Workspace: löscht alle Einträge in Variablen
cat("\014")   # Clear Console (entspricht ClrScr)

setwd("~/Desktop/myR")

library(stringr)
library(gplots)
library (tibble)
library (dplyr)
library(purrr)
library(stringr)
library (openxlsx)


# Das zu färbende Netz enthält folgende Klassen:
# Enter the classes of plasmodia from which the Petri net was built while using GetPN
# corresponding to PN1_.txt, PN2_.txt, etc... in the myR folder
Set_of_Classes <- c (1, 2, 3, 4)


# Session einlesen:
load("~/Desktop/myR/(54) Complete_Session_Anna_all_35_Log.RData")


# Dieses Snoopy File soll eingefärbt werden:
Snoopy_File_to_be_Colored <- "IPNA_Anna_all_35_Log_Tinv.spn"


Snoopy_File_Freq_Colored <- str_replace(Snoopy_File_to_be_Colored, pattern = "_Tinv", "_Tinv_Freq")
Snoopy_File_Stab_Colored <- str_replace(Snoopy_File_to_be_Colored, pattern = "_Tinv", "_Tinv_Stab")






# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#
#                                  T E I L  1
#
# In Teil_1 werden die Häufigkeiten und Stabilitäten bestimmt
#  
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



# Berechung der Stabilität der Cluster bzw. Plätze erfolgt neu,
# *************************************************************  
# d.h. nur für das Set_of_Classes durch Umschreiben des
# entsprechenden Codes aus "Cluster_to_HiR_PN_V45.R", 
# durch Ersetzen von "Trajektorien_Matrix" durch "Block" bzw. "Block":

# Stabilitätsberechung der Cluster (PN-Nodes):
# ***********************************************************************
# Die Trajektorien-Matrix wird in einzelne Zeilen zerlegt und in jeder
# Zeile die Stabilität der Cluster ermittelt:


# Plasmodien herausschreiben, die zu dem Set_of_Classes gehören:
TraCl <- as.data.frame(TmC)
PL <- row.names(TraCl)
Alle_Plasmodien <- cbind (PL, TraCl) 
# Plasmodien-Namen als Spalte einfügen (zur Doku)
Alle_Plasmodien <- add_column(TraCl, PL, .after = 1)

# mit dplyr:
Block <- filter(Alle_Plasmodien, P %in% Set_of_Classes)

# # Alternative ohne dplyr:
# Block <- data.frame()
# # Subset mehrfach in Scheife ausführen:
# 
# for (i in 1: length(Set_of_Classes))
#   { S <- subset.data.frame(Alle_Plasmodien, Alle_Plasmodien ["P"] == Set_of_Classes [i])
#     Block <- rbind (Block, S)
#   }


# Häufigkeitsverteilung der Cluster in den gewählten Klassen
Cluster.Verteilung <- table(as.vector (unlist(Block[,1:Zeitpunkte+2])))
Frequenz_der_Cluster <- as.matrix(Cluster.Verteilung)






Plasmodien_Zahl <- nrow(Block)
Cluster_im_Block <- as.matrix (Block [, -(1:2)])  # Erste & zweite Spalte  weg

# Matrix zeilenweise in Vektor umwandeln (muss dazu transponiert werden):
Clusters_observed <- as.integer(t (Cluster_im_Block))

# Häufigkeit jedes Clusters:
Cl_freq <- (table(Clusters_observed))

# Cluster Frequency of Islets
Cfreq_of_Isl <- Cl_freq
# wird unten runter gerechnet, so dass in Cfreq_of_Isl nur die Häufigkeit
# der Cluster enthält, aus denen die Inseln bestehen.

Vorhandene_Cluster <- as.integer (rownames(Cl_freq))

Verschiedene_Cluster <- length (unique(as.integer(Cluster_im_Block)))  # Zahl dieser Cluster

# Initialisieren als Vektor mit Nullen:
Blöcke_eines_Clusters <- integer (Verschiedene_Cluster)

# Zahl der Separaten Blöcke eines Clusters ermitteln:
# Dabei wird die Trajektorien-Matrix zeilenweise (p) abgearbeitet,
# und jede Zeile Zeitpunkt für Zeitpunkt (i) abgearbeitet
# Jede Zeile entspricht einem Plasmodium
for (p in 1:Plasmodien_Zahl)
{ 
  Ctr <- Cluster_im_Block [p,]  # Ctr, Current trajectory
  for (i in 1:(length(Ctr)-1))   
    { if (Ctr[i] != Ctr[i+1]) # wenn sich die Cluster-Nummer ändert, gibt es einen neuen Block
         {  # an welcher Position steht der Wert? (indirekte Adressierung)
            Pos <- which (Vorhandene_Cluster == Ctr[i])
            Blöcke_eines_Clusters [Pos] <- Blöcke_eines_Clusters [Pos] + 1 
         } # end of if...
    } # end of for i..... 
  
  # an welcher Position steht der Wert? (indirekte Adressierung)
  # gilt für beide Aufrufe
  Pos <- which (Vorhandene_Cluster == Ctr[length(Ctr)])
  
  # Berücksichtigt die in der Trajektorien-Tabelle endständigen Cluster
  # einer jeden Zeile, weil hier prinzipiell keine Änderung auftreten kann. Daher
  # besteht eine endständige Insel aus mindestens 2 Elementen
  
  if (Ctr[length(Ctr)-1] == Ctr[length(Ctr)]) 
  { Blöcke_eines_Clusters [Pos] <- Blöcke_eines_Clusters [Pos] + 1 } # end of if...
  
  # wenn in der letzten Spalte, also zum letzten Zeitpunkt, noch mal ein
  # neuer Cluster beginnt, darf er in der Häufigkeitsverteilung nicht gezählt
  # und muss entsprechend daraus entfernt werden, sonst stimmt die Formel 
  # Sability is Cl_freq / Islets nicht, weil eine endständige Insel aus nur
  # einem Element nicht mitgezählt wird (siehe oben):
  if (Ctr[length(Ctr)-1] != Ctr[length(Ctr)])
  { Cfreq_of_Isl [Pos] <- Cfreq_of_Isl [Pos] - 1 }
  
} # end of for p.....


# Zahl der Blöcke füe jeden Cluster
Islets <- Blöcke_eines_Clusters

length (Cl_freq)

Stability <- Cfreq_of_Isl / Islets
# Unendliche Stabilitäts-Werte entsprechen Terminal states

Cluster_ <- Vorhandene_Cluster




Stability_Table <- cbind(Cluster_, 
                         Cl_freq,
                         Islets, 
                         Cfreq_of_Isl,
                         Stability)



# Häufigkeitsverteilung formatieren & sortieren
Stab <- as.data.frame(Stability_Table)
Stability_desc <- arrange (Stab, desc(Stability))
Freq_desc      <- arrange (Stab, desc(Cl_freq))




# Ergebnisse als File ausgeben:
SoC <- paste (Set_of_Classes, collapse="_")



# Simprof-Trajektorien des gesamten Datensatzes:
filename <- paste(sep="", "(11) Simprof_Trajektorien_tab_", FileTag, ".txt")
cat(capture.output(print(Trajektorien_Matrix), file=filename))

filename <- paste0("(811) Simprof_Trajektorien_tab_", FileTag, ".xlsx")
write.xlsx(Trajektorien_Matrix, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)


# Nur die verwendeten Simprof-Trajektorien (des Set_of_Classes):
filename <- paste(sep="", "(12) Analysierte_Trajektorien_Class_", SoC, "_", FileTag, ".txt")
cat(capture.output(print(Block), file=filename))

filename <- paste0("(812) Analysierte_Trajektorien_Class_", SoC, "_", FileTag, ".xlsx")
write.xlsx(Block, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)


# Cluster-Frequency:
filename <- paste0("(13) Cluster_Freq_Class_", SoC, "_", FileTag, ".txt")
cat(capture.output(print(Stability_Table), file=filename))

filename <- paste0("(813) Cluster_Freq_Class_", SoC, "_", FileTag, ".xlsx")
write.xlsx(Stability_Table, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)


# Sortiert nach Cluster-Frequency:
filename <- paste0("(14) Cluster_desc_Freq_Class_", SoC, "_", FileTag, ".txt")
cat(capture.output(print(Freq_desc), file=filename))

filename <- paste0("(814) Cluster_desc_Freq_Class_", SoC, "_", FileTag, ".xlsx")
write.xlsx(Freq_desc, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)


# Sortiert nach Cluster-Stability:
filename <- paste0("(15) Cluster_desc_Stability_Class_", SoC, "_", FileTag, ".txt")
cat(capture.output(print(Stability_desc), file=filename))

filename <- paste0("(815) Cluster_desc_Stability_Class_", SoC, "_", FileTag, ".xlsx")
write.xlsx(Stability_desc, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)




# ENDE: Stabilitätsberechung der Cluster (PN-Nodes)
# *********************************************************************










# ----------------------------------------------------------------------------------
# Berechnung der Events pro Transition:
# ----------------------------------------------------------------------------------
# Zahl der Events pro Transition für eine Gruppe von Petri-Netzen bestimmen:
# Direkt von den Textdateien mit den Petri Net Description Vektoren starten (!)

# Welche Netze solle vereinigt werden: 
Take_Petri_Nets_Number <- Set_of_Classes


# data.frame initialisieren:
df <- data.frame(Transition_Name = character(0),
                 Events          = numeric(0),
                 stringsAsFactors = FALSE)

tabz <- 0

for (p in 1:length(Take_Petri_Nets_Number))
{
  # File-Namen erzeugen:
  PNDV_File_Name <- paste0("PN", Take_Petri_Nets_Number [p], "_.txt")
  
  # File mit dem Petri Net Description Vector einlesen
  PNDV <-read.csv(PNDV_File_Name,sep="\t",header=T)
  
  # Zahl der stochastischen Transitionen steht in der 2. Zeile
  Stochastische_Transitionen <- PNDV [2, 1]
  
  # Namen der Transitionen wieder herstellen:
  Zeile <- 0 
  for (i in 1:Stochastische_Transitionen)
  {
    Zeile <- Zeile + 3
    Transition_Name <- paste ("T", PNDV [Zeile, 1], PNDV [(Zeile + 1), 1], sep="_")
    Events <- PNDV [(Zeile + 2), 1]
    # Ergebnis eintragen:
    tabz <- tabz + 1
    df [tabz,1] <- Transition_Name
    df [tabz, 2] <- Events
    
  } # end of for i
} # end of for p

# Der df enthält die aneinander gefügten Listen Transition und Zahl der Events,
# aus den zusammengelegten PNDV's
# so daß Transitionsnamen also mehrfach vorkommen können.
# Daher werden alle Events für den selben Transitionsnamen addiert,
# so daß jeder Transitionsname nunmehr nur einmal vorkommt. 

# Für gleiche Transitions-Namen gruppieren und Events aufsummieren:
lumped <- dplyr::group_by(df, Transition_Name) %>% dplyr::summarise_all(sum)
Frequenz_der_Transitionen <- as.data.frame(lumped)

Frequenz_der_Transitionen <- as.data.frame(lumped)






# Namen der Transitionen in Teilstrings zerlegen (ist zum Sortieren erforderlich):
von_nach <- str_replace(Frequenz_der_Transitionen [,1], pattern = "T_", "")

von  <- as.integer (map_chr (str_split(von_nach, "_"), 1))
nach <- as.integer (map_chr (str_split(von_nach, "_"), 2))


Transition_Frequency_List <- cbind (von, nach, Frequenz_der_Transitionen)

Sortiert_nach_Transition <- arrange (Transition_Frequency_List, von, nach)

Sortiert_nach_Frequency <- arrange (Transition_Frequency_List, desc(Events))




# Sortiert nach Namen der Transition:
filename <- paste0("(16) Transition_Freq_Class_", SoC, "_", FileTag, ".txt")
cat(capture.output(print(Sortiert_nach_Transition), file=filename))

filename <- paste0("(816) Transition_Freq_Class_", SoC, "_", FileTag, ".xlsx")
write.xlsx(Sortiert_nach_Transition, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)



# Sortiert nach Häufigkeit (Frequenz) der Transition, bzw. der Transits:
filename <- paste0("(17) Transition_desc_Freq_Class_", SoC, "_", FileTag, ".txt")
cat(capture.output(print(Sortiert_nach_Frequency), file=filename))

filename <- paste0("(817) Transition_desc_Freq_Class_", SoC, "_", FileTag, ".xlsx")
write.xlsx(Sortiert_nach_Frequency, filename, asTable = FALSE, keepAN = TRUE, zoom = 150)









# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#
#                                  T E I L  2
#
# In Teil_2 werden die Farben erzeugt und die Petri-Netze eingefärbt
#  
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


# By Wolfgang Marwan
# Erstellt am: 04-07-2019
# Version vom: 08-07-2019
# Version vom: 17-09-2019

# Trägt den Datei-Namen in das Snoopy File ein, wenn es 
# eine Commentar-Box mit "Snoopy_Datei_Name" enthält

# Version vom 26-11-2019
# Färbt   Plätze nach Häufigkeit (der Cluster) & Transitionen nach Aktivität
# sowie   Plätze nach Stabilität (der Cluster) & Transitionen nach Aktivität

# sind Transitionen bereits gefärbt (z.B. nach Klassen), dann bleibt diese
# Färbung unverändert (!)

# Version vom 15-07-2020 
# Umgeschrieben für das Färben von Netzen mit ausgewählten Klassen 
# Plätze werden nach Namen gesucht




# Farbpaletten für die Plätze und Transitionen festlegen:
FarbenStufen <- 20  # der Farbgradienten
my_Stability_palette <- colorRampPalette(c("palevioletred1", "yellow", "mediumspringgreen", "lightslateblue" ))(n=FarbenStufen) 
my_Frequency_palette <- colorRampPalette(c("palevioletred1", "yellow", "mediumspringgreen", "lightslateblue" ))(n=FarbenStufen) 
my_Transition_palette <- colorRampPalette(c("lightgrey", "aquamarine", "green", "forestgreen"))(n=FarbenStufen)

# Im Snoopy xml-File steht die:
Erste_Platzfarbe_in_Zeile <- 78   # In dieser Zeile ist die Farbe des ersten Platzes "C1" vermerkt
Naechste_Platzfarbe_alle  <- 37   # Zeilen, also z.B. in jeder 37. Zeile ist die Farbe des jeweiligen Platzes vermerkt
Zeilen_Offset_Transitions_Farbe <- 47  # Transitions-Farbe steht n Zeilen nach Transitions-Name (der direkt gesucht wird)

Zeilen_Offset_Platz_Farbe <- 32  # Platz-Farbe steht n Zeilen nach Platz-Name (der direkt gesucht wird)


# ****************************************************
# Erzeugen von Color-Keys für Plätze und Transitionen:
# ****************************************************


# COLOR-KEY DER PLÄTZE durch Erzeugen einer Heatmap:
# Hierzu wird aus der 
#                       Stabilitätstabelle der Cluster
#                      ********************************
# zunächst  eine geeignete Matrix erzeugt:

Just_Stability <- cbind(Stability_Table[,"Stability"], Stability_Table[,"Stability"])
# Dann werden die "Unendlich"-Werte (der terminal states) in NA umgewandelt, weil 
# anderenfalls die heatmap abstürzt:
Just_Stability[!is.finite(Just_Stability)] <- NA


{pdf (paste(sep="", "(901) Place_Stability_Colors_", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(Just_Stability),
                     main=FileTag,
                     # Rowv=as.dendrogram(hc),
                     Rowv=FALSE,
                     Colv=FALSE,
                     scale="none",
                     col=my_Stability_palette, # breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(Just_Stability) hclust(Just_Stability,method="average"),
                     cexCol=xsize, cexRow=ysize,
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=2)
  # RowSideColors= myClusterSideBar)
  dev.off()}





# COLOR-KEY DER PLÄTZE durch Erzeugen einer Heatmap:
# Hierzu wird aus der 
#                        Häufigkeitsverteilung der Cluster 
#                       ***********************************
# zunächst  eine geeignete Matrix erzeugt:

Just_Häufigkeit <- cbind(Frequenz_der_Cluster, Frequenz_der_Cluster)


# Dann werden die "Unendlich"-Werte (der terminal states) in NA umgewandelt, weil 
# anderenfalls die heatmap abstürzt:
# Just_Stability[!is.finite(Just_Stability)] <- NA


{pdf (paste(sep="", "(902) Place_Frequency_Colors_", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(Just_Häufigkeit),
                     main=FileTag,
                     # Rowv=as.dendrogram(hc),
                     Rowv=FALSE,
                     Colv=FALSE,
                     scale="none",
                     col=my_Frequency_palette, # breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(Just_Häufigkeit) hclust(Just_Häufigkeit,method="average"),
                     cexCol=xsize, cexRow=ysize,
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=2)
  # RowSideColors= myClusterSideBar)
  dev.off()}



# COLOR-KEY DER TRANSITIONEN durch Erzeugen einer Heatmap:
# Hier sind die Transitionen mit Events gespeichert: 

Transits_for_HeatMap <- cbind(Frequenz_der_Transitionen[,2], Frequenz_der_Transitionen[,2])

{pdf (paste(sep="", "(903) Transition_Frequency_Colors_", FileTag, ".pdf"),width=8,height=25)
  heatmap<-heatmap.2(as.matrix(Transits_for_HeatMap),
                     main=FileTag,
                     # Rowv=as.dendrogram(hc),
                     Rowv=FALSE,
                     Colv=FALSE,
                     scale="none",
                     col=my_Transition_palette, # breaks=colors,
                     symkey=F,
                     density.info="none",
                     key=TRUE,trace="none",
                     hclustfun=function(Transits_for_HeatMap) hclust(Transits_for_HeatMap,method="average"),
                     cexCol=xsize, cexRow=ysize,
                     margins=c(12,8), # white space
                     lhei=c(1,15), keysize=2)
  # RowSideColors= myClusterSideBar)
  dev.off()}



# Berechnen der Farben der Plätze und Transitionen:
# *************************************************
# Farben müssen jeweils erneut berechnet werden, da sie aus der Heatmap scheinbar nicht abrufbar sind


# FARBEN DER TRANSITIONEN (als Vektor):
# Hier werden den Transits Farben zugeordnet, mit denen die Transitionen eingefärbt werden:

MinTransits <- min(Frequenz_der_Transitionen[,2])
MaxTransits <- max(Frequenz_der_Transitionen[,2])


Inkremente <- (MaxTransits - MinTransits) / FarbenStufen
Transits_Obergrenzen <- seq ((MinTransits + Inkremente), MaxTransits, by=Inkremente)
Transits_Obergrenzen
length (Transits_Obergrenzen)

Farb_Order_Transits <- ceiling ((Frequenz_der_Transitionen[,2]-MinTransits)/Inkremente)
Farb_Pos_Transits <- replace(Farb_Order_Transits, Farb_Order_Transits==0, 1)  # 0 in 1 umwandeln
TransitColors <- my_Transition_palette [Farb_Pos_Transits]

RGB_Transit_Colors <- matrix(0, length (TransitColors), 1)

for (i in 1:length (TransitColors))
{ HexFarbe <- TransitColors [i]
RGB_Farbe <- paste(as.vector(col2rgb(HexFarbe)), collapse = ",")
RGB_Transit_Colors [i,1] <- RGB_Farbe
}

length (TransitColors)      # nur zur Kontrolle







# Snoopy File einlesen, Plätze und Transitionen suchen und Farben eintragen:
# **************************************************************************

# Snoopy xml-File als Text einlesen:
fileName <- Snoopy_File_to_be_Colored
conn <- file(fileName,open="r")
linn <-readLines(conn)
close(conn)




# ***************************************************************************
# Färben:  Transitionen nach Aktivität & Plätze nach Häufigkeit (der Cluster)
# ***************************************************************************

# TRANSITIONEN im xml-File suchen und einfärben:
# ************

# Nach diesen Transitionsnamen wird im Snoopy-xml-File explicit gesucht
# Format allgemein für Netze mit der Klasse als Teil des Namens der Transitionen:
Transition_Names <- paste0(Frequenz_der_Transitionen [,1], "_")  



# Suchen, in welchen Zeilen die Namen der Transitionen stehen:
Zeilen_mit_Transition_Farbe <- numeric()

for (t in 1:length (Transition_Names))
{
  # Liefert den Inhalt der Textzeile zurück in der der gesuchte String steht (!), hier der Name der Transition
  Found <- grep(Transition_Names[t], linn, value = TRUE)
  # Liefert die Nummer der Textzeile zurück in der der gesuchte String steht (!)
  ZeilenNr <- which(linn==Found) 
  # Farbe ist weiter unten eingetragen:
  ZeilenNr <- ZeilenNr + Zeilen_Offset_Transitions_Farbe
  Zeilen_mit_Transition_Farbe <- c(Zeilen_mit_Transition_Farbe, ZeilenNr)
}

Zeilen_mit_Transition_Farbe 

# und Farben eintragen:
for (t in 1:nrow(RGB_Transit_Colors))
{
  x <- str_replace (linn[Zeilen_mit_Transition_Farbe[t]], pattern = "255,255,255", RGB_Transit_Colors [t])
  linn[Zeilen_mit_Transition_Farbe[t]] <- x
} 


length(Frequenz_der_Cluster) # nur zur Kontrolle



# FARBEN DER PLÄTZE (als Vektor):
# Hier werden den ----- Häufigkeitswerten ----- Farben zugeordnet, mit denen die Plätze eingefärbt werden:

# Tabelle mit Häufigkeitswerten für jeden Platz:

Häufigkeits_Werte <- Frequenz_der_Cluster

Min <- min (Häufigkeits_Werte [!is.na(Häufigkeits_Werte)])
Max <- max (Häufigkeits_Werte [!is.na(Häufigkeits_Werte)])

# Und dann die Häufigkeitswerte in Klassen einteilen und der jeweiligen Farbstufe zuordnen:

Inkremente <- (Max - Min) / FarbenStufen
Obergrenzen <- seq ((Min + Inkremente), Max, by=Inkremente)
Obergrenzen
length (Obergrenzen)

Farb_Order <- ceiling ((Häufigkeits_Werte-Min)/Inkremente)
Farb_Pos <- replace(Farb_Order, Farb_Order==0, 1)  # 0 in 1 umwandeln
StabColors <- my_Frequency_palette [Farb_Pos]

RGB_Stab_Colors <- matrix(0, length (StabColors), 1)

for (i in 1:length (StabColors))
{ HexFarbe <- StabColors [i]
RGB_Farbe <- paste(as.vector(col2rgb(HexFarbe)), collapse = ",")
RGB_Stab_Colors [i,1] <- RGB_Farbe
}

length(RGB_Stab_Colors)      # nur zur Kontrolle




# PLÄTZE im xml-File explicit nach Namen suchen und einfärben:
# *************
# Plätze werden nach Namen gesucht (!): 
Cluster_Names <- paste0 ("C", Cluster_)

Zeilen_mit_Platz_Farbe <- numeric()

for (t in 1:length (Cluster_Names))
{
  # Liefert den Inhalt der Textzeile zurück in der der gesuchte String steht (!), hier der Name des Platzes
  Such_String <- paste0(Cluster_Names[t],"]")   # ohne Begrenzer "]" würde statt nur C1 auch C11, C12 etc. gefunden
  Found <- grep(Such_String, linn, value = TRUE)
  
  # Liefert die Nummer der Textzeile zurück in der der gesuchte String steht (!)
  ZeilenNr <- which(linn==Found) 
  # Farbe ist einige Zeilen (Zeilen_Offset_Platz_Farbe) weiter unten eingetragen:
  ZeilenNr <- ZeilenNr + Zeilen_Offset_Platz_Farbe
  Zeilen_mit_Platz_Farbe <- c(Zeilen_mit_Platz_Farbe, ZeilenNr)
}


# Farben eintragen:

for (p in 1:nrow(RGB_Stab_Colors))
{
  x <- str_replace (linn[Zeilen_mit_Platz_Farbe[p]], pattern = "255,255,255", RGB_Stab_Colors [p])
  linn[Zeilen_mit_Platz_Farbe[p]] <- x
} 





# Trägt den Snoopy-Datei-Namen ein, wenn "Snoopy_Datei_Name" als Comment gesetzt wurde
# Liefert den Inhalt der Textzeile zurück in der der gesuchte String steht (!), hier der Name der Transition
Found <- grep("Snoopy_Datei_Name", linn, value = TRUE)
# Liefert die Nummer der Textzeile zurück in der der gesuchte String steht (!)
ZeilenNr <- which(linn==Found) 
x <- str_replace(Found, pattern = "Snoopy_Datei_Name", Snoopy_File_Freq_Colored)
linn[ZeilenNr] <- x




# Das eingefärbte Petri-Netz als Snoopy-File abspeichern:

fileConn<-file(Snoopy_File_Freq_Colored)
writeLines(linn, fileConn)
close(fileConn)





# ***************************************************************************
# Färben:  Transitionen nach Aktivität & Plätze nach Stabilität (der Cluster)
# ***************************************************************************


# Snoopy File einlesen, Plätze und Transitionen suchen und Farben eintragen:
# **************************************************************************

# Snoopy xml-File als Text einlesen:
fileName <- Snoopy_File_to_be_Colored
conn <- file(fileName,open="r")
linn <-readLines(conn)
close(conn)





# TRANSITIONEN im xml-File suchen und einfärben:
# ************

Transition_Names <- paste0 (Frequenz_der_Transitionen [,1], "_")

# Suchen, in welchen Zeilen die Namen der Transitionen stehen:
Zeilen_mit_Transition_Farbe <- numeric()

for (t in 1:length (Transition_Names))
{
  # Liefert den Inhalt der Textzeile zurück in der der gesuchte String steht (!), hier der Name der Transition
  Found <- grep(Transition_Names[t], linn, value = TRUE)
  # Liefert die Nummer der Textzeile zurück in der der gesuchte String steht (!)
  ZeilenNr <- which(linn==Found) 
  # Farbe ist weiter unten eingetragen:
  ZeilenNr <- ZeilenNr + Zeilen_Offset_Transitions_Farbe
  Zeilen_mit_Transition_Farbe <- c(Zeilen_mit_Transition_Farbe, ZeilenNr)
}

Zeilen_mit_Transition_Farbe 

# und Farben eintragen:
for (t in 1:nrow(RGB_Transit_Colors))
{
  x <- str_replace (linn[Zeilen_mit_Transition_Farbe[t]], pattern = "255,255,255", RGB_Transit_Colors [t])
  linn[Zeilen_mit_Transition_Farbe[t]] <- x
} 



# FARBEN DER PLÄTZE (als Vektor):
# Hier werden den Stabilitätswerten Farben zugeordnet, mit denen die Plätze eingefärbt werden:

# Tabelle mit Stabilitätswerten für jeden Platz:

Stabilitaets_Werte <- Stability_Table[,"Stability"]

Min <- min (Stabilitaets_Werte [!is.na(Stabilitaets_Werte)])
Max <- max (Stabilitaets_Werte [!is.na(Stabilitaets_Werte)])

# Und dann die Stabilitätswerte in Klassen einteilen und der jeweiligen Farbstufe zuordnen:

Inkremente <- (Max - Min) / FarbenStufen
Obergrenzen <- seq ((Min + Inkremente), Max, by=Inkremente)
Obergrenzen
length (Obergrenzen)

Farb_Order <- ceiling ((Stabilitaets_Werte-Min)/Inkremente)
Farb_Pos <- replace(Farb_Order, Farb_Order==0, 1)  # 0 in 1 umwandeln
StabColors <- my_Stability_palette [Farb_Pos]

RGB_Stab_Colors <- matrix(0, length (StabColors), 1)

for (i in 1:length (StabColors))
{ HexFarbe <- StabColors [i]
RGB_Farbe <- paste(as.vector(col2rgb(HexFarbe)), collapse = ",")
RGB_Stab_Colors [i,1] <- RGB_Farbe
}

length(RGB_Stab_Colors)      # nur zur Kontrolle


# PLÄTZE im xml-File suchen und einfärben:
# ******

# Plätze werden nach Namen gesucht (!): 
Cluster_Names <- paste0 ("C", Cluster_)

Zeilen_mit_Platz_Farbe <- numeric()

for (t in 1:length (Cluster_Names))
{
  # Liefert den Inhalt der Textzeile zurück in der der gesuchte String steht (!), hier der Name des Platzes
  Such_String <- paste0(Cluster_Names[t],"]")   # ohne "]" wird statt nur C1 auch C11, C12 etc. gefunden
  Found <- grep(Such_String, linn, value = TRUE)
  
  # Liefert die Nummer der Textzeile zurück in der der gesuchte String steht (!)
  ZeilenNr <- which(linn==Found) 
  # Farbe ist einige Zeilen (Zeilen_Offset_Platz_Farbe) weiter unten eingetragen:
  ZeilenNr <- ZeilenNr + Zeilen_Offset_Platz_Farbe
  Zeilen_mit_Platz_Farbe <- c(Zeilen_mit_Platz_Farbe, ZeilenNr)
}



# Farben werden eingetragen:

for (p in 1:nrow(RGB_Stab_Colors))
{
  x <- str_replace (linn[Zeilen_mit_Platz_Farbe[p]], pattern = "255,255,255", RGB_Stab_Colors [p])
  linn[Zeilen_mit_Platz_Farbe[p]] <- x
} 






# Trägt den Snoopy-Datei-Namen ein, wenn "Snoopy_Datei_Name" als Comment gesetzt wurde
# Liefert den Inhalt der Textzeile zurück in der der gesuchte String steht (!), hier der Name der Transition
Found <- grep("Snoopy_Datei_Name", linn, value = TRUE)
# Liefert die Nummer der Textzeile zurück in der der gesuchte String steht (!)
ZeilenNr <- which(linn==Found) 
x <- str_replace(Found, pattern = "Snoopy_Datei_Name", Snoopy_File_Stab_Colored)
linn[ZeilenNr] <- x





# Das eingefärbte Petri-Netz als Snoopy-File abspeichern:

fileConn<-file(Snoopy_File_Stab_Colored)
writeLines(linn, fileConn)
close(fileConn)


































